@@ -1,16 +1,14 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1998-2002
+ * Copyright (c) 1998-2004
  *	Sleepycat Software.  All rights reserved.
+ *
+ * $Id: bt_reclaim.c,v 11.15 2004/01/28 03:35:49 bostic Exp $
  */
 
 #include "db_config.h"
 
-#ifndef lint
-static const char revid[] = "Id: bt_reclaim.c,v 11.11 2002/03/29 20:46:26 bostic Exp ";
-#endif /* not lint */
-
 #ifndef NO_SYSTEM_INCLUDES
 #include <sys/types.h>
 
@@ -36,7 +34,7 @@ __bam_reclaim(dbp, txn)
 	int ret, t_ret;
 
 	/* Acquire a cursor. */
-	if ((ret = dbp->cursor(dbp, txn, &dbc, 0)) != 0)
+	if ((ret = __db_cursor(dbp, txn, &dbc, 0)) != 0)
 		return (ret);
 
 	/* Walk the tree, freeing pages. */
@@ -44,7 +42,7 @@ __bam_reclaim(dbp, txn)
 	    DB_LOCK_WRITE, dbc->internal->root, __db_reclaim_callback, dbc);
 
 	/* Discard the cursor. */
-	if ((t_ret = dbc->c_close(dbc)) != 0 && ret == 0)
+	if ((t_ret = __db_c_close(dbc)) != 0 && ret == 0)
 		ret = t_ret;
 
 	return (ret);
@@ -54,32 +52,23 @@ __bam_reclaim(dbp, txn)
  * __bam_truncate --
  *	Truncate a database.
  *
- * PUBLIC: int __bam_truncate __P((DB *, DB_TXN *, u_int32_t *));
+ * PUBLIC: int __bam_truncate __P((DBC *, u_int32_t *));
  */
 int
-__bam_truncate(dbp, txn, countp)
-	DB *dbp;
-	DB_TXN *txn;
+__bam_truncate(dbc, countp)
+	DBC *dbc;
 	u_int32_t *countp;
 {
-	DBC *dbc;
 	db_trunc_param trunc;
-	int ret, t_ret;
-
-	/* Acquire a cursor. */
-	if ((ret = dbp->cursor(dbp, txn, &dbc, 0)) != 0)
-		return (ret);
+	int ret;
 
 	trunc.count = 0;
 	trunc.dbc = dbc;
+
 	/* Walk the tree, freeing pages. */
 	ret = __bam_traverse(dbc,
 	    DB_LOCK_WRITE, dbc->internal->root, __db_truncate_callback, &trunc);
 
-	/* Discard the cursor. */
-	if ((t_ret = dbc->c_close(dbc)) != 0 && ret == 0)
-		ret = t_ret;
-
 	*countp = trunc.count;
 
 	return (ret);
