@@ -1,17 +1,17 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2002
+ * Copyright (c) 1996-2004
  *	Sleepycat Software.  All rights reserved.
+ *
+ * $Id: db_verify.c,v 1.49 2004/08/01 00:21:58 bostic Exp $
  */
 
 #include "db_config.h"
 
 #ifndef lint
 static const char copyright[] =
-    "Copyright (c) 1996-2002\nSleepycat Software Inc.  All rights reserved.\n";
-static const char revid[] =
-    "Id: db_verify.c,v 1.38 2002/08/08 03:51:38 bostic Exp ";
+    "Copyright (c) 1996-2004\nSleepycat Software Inc.  All rights reserved.\n";
 #endif
 
 #ifndef NO_SYSTEM_INCLUDES
@@ -53,20 +53,22 @@ db_verify_main(argc, argv)
 	const char *progname = "db_verify";
 	DB *dbp, *dbp1;
 	DB_ENV *dbenv;
-	u_int32_t cache;
-	int ch, d_close, e_close, exitval, nflag, oflag, private;
-	int quiet, resize, ret, t_ret;
+	u_int32_t flags, cache;
+	int ch, exitval, nflag, private;
+	int quiet, resize, ret;
 	char *home, *passwd;
 
 	if ((ret = db_verify_version_check(progname)) != 0)
 		return (ret);
 
 	dbenv = NULL;
+	dbp = NULL;
 	cache = MEGABYTE;
-	d_close = e_close = exitval = nflag = oflag = quiet = 0;
+	exitval = nflag = quiet = 0;
+	flags = 0;
 	home = passwd = NULL;
 	__db_getopt_reset = 1;
-	while ((ch = getopt(argc, argv, "h:NoP:qV")) != EOF)
+	while ((ch = getopt(argc, argv, "h:NoP:quV")) != EOF)
 		switch (ch) {
 		case 'h':
 			home = optarg;
@@ -84,11 +86,14 @@ db_verify_main(argc, argv)
 			}
 			break;
 		case 'o':
-			oflag = 1;
+			LF_SET(DB_NOORDERCHK);
 			break;
 		case 'q':
 			quiet = 1;
 			break;
+		case 'u':			/* Undocumented. */
+			LF_SET(DB_UNREF);
+			break;
 		case 'V':
 			printf("%s\n", db_version(NULL, NULL, NULL));
 			return (EXIT_SUCCESS);
@@ -114,7 +119,6 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 		    "%s: db_env_create: %s\n", progname, db_strerror(ret));
 		goto shutdown;
 	}
-	e_close = 1;
 
 	if (!quiet) {
 		dbenv->set_errfile(dbenv, stderr);
@@ -145,14 +149,18 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 	private = 0;
 	if ((ret =
 	    dbenv->open(dbenv, home, DB_INIT_MPOOL | DB_USE_ENVIRON, 0)) != 0) {
-		if ((ret = dbenv->set_cachesize(dbenv, 0, cache, 1)) != 0) {
-			dbenv->err(dbenv, ret, "set_cachesize");
-			goto shutdown;
+		if (ret != DB_VERSION_MISMATCH) {
+			if ((ret =
+			    dbenv->set_cachesize(dbenv, 0, cache, 1)) != 0) {
+				dbenv->err(dbenv, ret, "set_cachesize");
+				goto shutdown;
+			}
+			private = 1;
+			ret = dbenv->open(dbenv, home, DB_CREATE |
+			    DB_INIT_MPOOL | DB_PRIVATE | DB_USE_ENVIRON, 0);
 		}
-		private = 1;
-		if ((ret = dbenv->open(dbenv, home,
-	    DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_USE_ENVIRON, 0)) != 0) {
-			dbenv->err(dbenv, ret, "open");
+		if (ret != 0) {
+			dbenv->err(dbenv, ret, "DB_ENV->open");
 			goto shutdown;
 		}
 	}
@@ -162,11 +170,14 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 			dbenv->err(dbenv, ret, "%s: db_create", progname);
 			goto shutdown;
 		}
-		d_close = 1;
 
 		/*
 		 * We create a 2nd dbp to this database to get its pagesize
 		 * because the dbp we're using for verify cannot be opened.
+		 *
+		 * If the database is corrupted, we may not be able to open
+		 * it, of course.  In that case, just continue, using the
+		 * cache size we have.
 		 */
 		if (private) {
 			if ((ret = db_create(&dbp1, dbenv, 0)) != 0) {
@@ -175,12 +186,9 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 				goto shutdown;
 			}
 
-			if ((ret = dbp1->open(dbp1, NULL,
-			    argv[0], NULL, DB_UNKNOWN, DB_RDONLY, 0)) != 0) {
-				dbenv->err(dbenv, ret, "DB->open: %s", argv[0]);
-				(void)dbp1->close(dbp1, 0);
-				goto shutdown;
-			}
+			ret = dbp1->open(dbp1,
+			    NULL, argv[0], NULL, DB_UNKNOWN, DB_RDONLY, 0);
+
 			/*
 			 * If we get here, we can check the cache/page.
 			 * !!!
@@ -189,28 +197,24 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 			 * will still be working on the same argv when we
 			 * get back into the for-loop.
 			 */
-			ret = __db_util_cache(dbenv, dbp1, &cache, &resize);
-			(void)dbp1->close(dbp1, 0);
-			if (ret != 0)
-				goto shutdown;
+			if (ret == 0) {
+				if (__db_util_cache(
+				    dbp1, &cache, &resize) == 0 && resize) {
+					(void)dbp1->close(dbp1, 0);
+					(void)dbp->close(dbp, 0);
+					dbp = NULL;
 
-			if (resize) {
-				(void)dbp->close(dbp, 0);
-				d_close = 0;
-
-				(void)dbenv->close(dbenv, 0);
-				e_close = 0;
-				goto retry;
+					(void)dbenv->close(dbenv, 0);
+					dbenv = NULL;
+					goto retry;
+				}
 			}
+			(void)dbp1->close(dbp1, 0);
 		}
-		if ((ret = dbp->verify(dbp,
-		    argv[0], NULL, NULL, oflag ? DB_NOORDERCHK : 0)) != 0)
-			dbp->err(dbp, ret, "DB->verify: %s", argv[0]);
-		if ((t_ret = dbp->close(dbp, 0)) != 0 && ret == 0) {
-			dbenv->err(dbenv, ret, "DB->close: %s", argv[0]);
-			ret = t_ret;
-		}
-		d_close = 0;
+
+		/* The verify method is a destructor. */
+		ret = dbp->verify(dbp, argv[0], NULL, NULL, flags);
+		dbp = NULL;
 		if (ret != 0)
 			goto shutdown;
 	}
@@ -219,16 +223,19 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 shutdown:	exitval = 1;
 	}
 
-	if (d_close && (ret = dbp->close(dbp, 0)) != 0) {
+	if (dbp != NULL && (ret = dbp->close(dbp, 0)) != 0) {
 		exitval = 1;
 		dbenv->err(dbenv, ret, "close");
 	}
-	if (e_close && (ret = dbenv->close(dbenv, 0)) != 0) {
+	if (dbenv != NULL && (ret = dbenv->close(dbenv, 0)) != 0) {
 		exitval = 1;
 		fprintf(stderr,
 		    "%s: dbenv->close: %s\n", progname, db_strerror(ret));
 	}
 
+	if (passwd != NULL)
+		free(passwd);
+
 	/* Resend any caught signal. */
 	__db_util_sigresend();
 
@@ -251,12 +258,11 @@ db_verify_version_check(progname)
 
 	/* Make sure we're loaded with the right version of the DB library. */
 	(void)db_version(&v_major, &v_minor, &v_patch);
-	if (v_major != DB_VERSION_MAJOR ||
-	    v_minor != DB_VERSION_MINOR || v_patch != DB_VERSION_PATCH) {
+	if (v_major != DB_VERSION_MAJOR || v_minor != DB_VERSION_MINOR) {
 		fprintf(stderr,
-	"%s: version %d.%d.%d doesn't match library version %d.%d.%d\n",
+	"%s: version %d.%d doesn't match library version %d.%d\n",
 		    progname, DB_VERSION_MAJOR, DB_VERSION_MINOR,
-		    DB_VERSION_PATCH, v_major, v_minor, v_patch);
+		    v_major, v_minor);
 		return (EXIT_FAILURE);
 	}
 	return (0);
