@@ -1,10 +1,10 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002
+ * Copyright (c) 2002-2004
  *	Sleepycat Software.  All rights reserved.
  *
- * Id: ex_apprec.h,v 1.2 2002/08/08 15:47:00 bostic Exp 
+ * $Id: ex_apprec.h,v 1.5 2004/01/28 03:36:03 bostic Exp $
  */
 
 #ifndef _EX_APPREC_H_
@@ -20,5 +20,7 @@ int ex_apprec_mkdir_read
     __P((DB_ENV *, void *, ex_apprec_mkdir_args **));
 int ex_apprec_mkdir_recover
     __P((DB_ENV *, DBT *, DB_LSN *, db_recops, void *));
+int ex_apprec_init_print __P((DB_ENV *,
+    int (***)(DB_ENV *, DBT *, DB_LSN *, db_recops, void *), size_t *));
 
 #endif /* !_EX_APPREC_H_ */
