@@ -1,16 +1,14 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000-2002
+ * Copyright (c) 2000-2004
  *      Sleepycat Software.  All rights reserved.
+ *
+ * $Id: db_server_cxxutil.cpp,v 1.17 2004/09/22 17:30:13 bostic Exp $
  */
 
 #include "db_config.h"
 
-#ifndef lint
-static const char revid[] = "Id: db_server_cxxutil.cpp,v 1.8 2002/05/23 07:49:34 mjc Exp ";
-#endif /* not lint */
-
 #ifndef NO_SYSTEM_INCLUDES
 #include <sys/types.h>
 
@@ -34,7 +32,8 @@ static const char revid[] = "Id: db_server_cxxutil.cpp,v 1.8 2002/05/23 07:49:34
 #include <string.h>
 #include <unistd.h>
 #endif
-#include "dbinc_auto/db_server.h"
+
+#include "db_server.h"
 
 #include "db_int.h"
 #include "db_cxx.h"
@@ -159,7 +158,7 @@ main(
 	 */
 	if (__dbsrv_defto > __dbsrv_idleto)
 		fprintf(stderr,
-		    "%s: WARNING: Idle timeout %ld is less than resource timeout %ld\n",
+	    "%s: WARNING: Idle timeout %ld is less than resource timeout %ld\n",
 		    prog, __dbsrv_idleto, __dbsrv_defto);
 
 	LIST_INIT(&__dbsrv_head);
@@ -421,7 +420,8 @@ get_tableent(long id)
 }
 
 extern "C" ct_entry *
-__dbsrv_sharedb(ct_entry *db_ctp, const char *name, const char *subdb, DBTYPE type, u_int32_t flags)
+__dbsrv_sharedb(ct_entry *db_ctp,
+    const char *name, const char *subdb, DBTYPE type, u_int32_t flags)
 {
 	ct_entry *ctp;
 
@@ -597,7 +597,7 @@ __dbenv_close_int(long id, u_int32_t flags, int force)
 {
 	DbEnv *dbenv;
 	int ret;
-	ct_entry *ctp;
+	ct_entry *ctp, *dbctp, *nextctp;
 
 	ret = 0;
 	ctp = get_tableent(id);
@@ -617,6 +617,32 @@ __dbenv_close_int(long id, u_int32_t flags, int force)
 	if (__dbsrv_verbose)
 		printf("Closing env id %ld\n", id);
 
+	/*
+	 * If we're timing out an env, we want to close all of its
+	 * database handles as well.  All of the txns and cursors
+	 * must have been timed out prior to timing out the env.
+	 */
+	if (force)
+		for (dbctp = LIST_FIRST(&__dbsrv_head);
+		    dbctp != NULL; dbctp = nextctp) {
+			nextctp = LIST_NEXT(dbctp, entries);
+			if (dbctp->ct_type != CT_DB)
+				continue;
+			if (dbctp->ct_envparent != ctp)
+				continue;
+			/*
+			 * We found a DB handle that is part of this
+			 * environment.  Close it.
+			 */
+			__db_close_int(dbctp->ct_id, 0);
+			/*
+			 * If we timed out a dbp, we may have removed
+			 * multiple ctp entries.  Start over with a
+			 * guaranteed good ctp.
+			 */
+			nextctp = LIST_FIRST(&__dbsrv_head);
+		}
+
 	ret = dbenv->close(flags);
 	__dbdel_ctp(ctp);
 	return (ret);
@@ -640,8 +666,11 @@ add_home(char *home)
 	 * to assure hp->name points to the last component.
 	 */
 	hp->name = __db_rpath(home);
-	*(hp->name) = '\0';
-	hp->name++;
+	if (hp->name != NULL) {
+		*(hp->name) = '\0';
+		hp->name++;
+	} else
+		hp->name = home;
 	while (*(hp->name) == '\0') {
 		hp->name = __db_rpath(home);
 		*(hp->name) = '\0';
@@ -686,10 +715,13 @@ add_passwd(char *passwd)
 }
 
 extern "C" home_entry *
-get_home(char *name)
+get_fullhome(char *name)
 {
 	home_entry *hp;
 
+	if (name == NULL)
+		return (NULL);
+
 	for (hp = LIST_FIRST(&__dbsrv_home); hp != NULL;
 	    hp = LIST_NEXT(hp, entries))
 		if (strcmp(name, hp->name) == 0)
@@ -709,10 +741,8 @@ env_recover(char *progname)
 	    hp = LIST_NEXT(hp, entries)) {
 		exitval = 0;
 		dbenv = new DbEnv(DB_CXX_NO_EXCEPTIONS);
-		if (__dbsrv_verbose == 1) {
+		if (__dbsrv_verbose == 1)
 			(void)dbenv->set_verbose(DB_VERB_RECOVERY, 1);
-			(void)dbenv->set_verbose(DB_VERB_CHKPOINT, 1);
-		}
 		dbenv->set_errfile(stderr);
 		dbenv->set_errpfx(progname);
 		if (hp->passwd != NULL)
@@ -726,7 +756,7 @@ env_recover(char *progname)
 			printf("Running recovery on %s\n", hp->home);
 		flags = DB_CREATE | DB_INIT_LOCK | DB_INIT_LOG | DB_INIT_MPOOL |
 		    DB_INIT_TXN | DB_USE_ENVIRON | DB_RECOVER;
-		if ((ret = dbenv->open(hp->home, flags, 0)) != 0) {
+		if ((ret = (dbenv->open)(hp->home, flags, 0)) != 0) {
 			dbenv->err(ret, "DbEnv->open");
 			goto error;
 		}
