@@ -1,9 +1,9 @@
 # See the file LICENSE for redistribution information
 #
-# Copyright (c) 2000-2002
+# Copyright (c) 2000-2004
 #       Sleepycat Software.  All rights reserved.
 #
-# Id: logtrack.tcl,v 11.11 2002/09/03 16:44:37 sue Exp 
+# $Id: logtrack.tcl,v 11.15 2004/04/14 16:08:42 carol Exp $
 #
 # logtrack.tcl:  A collection of routines, formerly implemented in Perl
 # as log.pl, to track which log record types the test suite hits.
@@ -66,6 +66,7 @@ proc logtrack_read { dirname } {
 # seen and the log record types that were not seen but should have been seen.
 proc logtrack_summary { } {
 	global ltsname ltlist testdir
+	global one_test
 
 	set seendb [berkdb_open $ltsname]
 	error_check_good seendb_open [is_valid_db $seendb] TRUE
@@ -109,8 +110,8 @@ proc logtrack_summary { } {
 	    [is_valid_cursor [set ec [$existdb cursor]] $existdb] TRUE
 	while { [llength [set dbt [$ec get -next]]] != 0 } {
 		set rec [lindex [lindex $dbt 0] 0]
-		if { [$seendb count $rec] == 0 } {
-			puts "FAIL: log record type $rec not seen"
+		if { [$seendb count $rec] == 0 && $one_test == "ALL" } {
+			puts "WARNING: log record type $rec: not tested"
 		}
 	}
 	error_check_good exist_curs_close [$ec close] 0
@@ -121,9 +122,10 @@ proc logtrack_summary { } {
 		set rec [lindex [lindex $dbt 0] 0]
 		if { [$existdb count $rec] == 0 } {
 			if { [$deprecdb count $rec] == 0 } {
-			       puts "FAIL: unknown log record type $rec seen"
+			       puts "WARNING: log record type $rec: unknown"
 			} else {
-			       puts "FAIL: deprecated log record type $rec seen"
+			       puts \
+			   "WARNING: log record type $rec: deprecated"
 			}
 		}
 	}
