@@ -41,15 +41,16 @@ enum cpioErrorReturns {
 	CPIOERR_READLINK_FAILED	= (19   | CPIOERR_CHECK_ERRNO),
 	CPIOERR_READ_FAILED	= (20   | CPIOERR_CHECK_ERRNO),
 	CPIOERR_COPY_FAILED	= (21   | CPIOERR_CHECK_ERRNO),
-	CPIOERR_HDR_SIZE	= (22			),
-	CPIOERR_HDR_TRAILER	= (23			),
-	CPIOERR_UNKNOWN_FILETYPE= (24			),
-	CPIOERR_MISSING_HARDLINK= (25			),
-	CPIOERR_MD5SUM_MISMATCH	= (26			),
-	CPIOERR_INTERNAL	= (27			),
-	CPIOERR_UNMAPPED_FILE	= (28			),
-	CPIOERR_ENOENT		= (29			),
-	CPIOERR_ENOTEMPTY	= (30			)
+	CPIOERR_LSETFCON_FAILED	= (22   | CPIOERR_CHECK_ERRNO),
+	CPIOERR_HDR_SIZE	= (23			),
+	CPIOERR_HDR_TRAILER	= (24			),
+	CPIOERR_UNKNOWN_FILETYPE= (25			),
+	CPIOERR_MISSING_HARDLINK= (26			),
+	CPIOERR_MD5SUM_MISMATCH	= (27			),
+	CPIOERR_INTERNAL	= (28			),
+	CPIOERR_UNMAPPED_FILE	= (29			),
+	CPIOERR_ENOENT		= (30			),
+	CPIOERR_ENOTEMPTY	= (31			)
 };
 
 /** \ingroup payload
@@ -63,7 +64,8 @@ typedef enum cpioMapFlags_e {
     CPIO_MAP_ABSOLUTE	= (1 << 5),
     CPIO_MAP_ADDDOT	= (1 << 6),
     CPIO_ALL_HARDLINKS	= (1 << 7), /*!< fail if hardlinks are missing. */
-    CPIO_MAP_TYPE	= (1 << 8)  /*!< only for building. */
+    CPIO_MAP_TYPE	= (1 << 8),  /*!< only for building. */
+    CPIO_SBIT_CHECK	= (1 << 9)
 } cpioMapFlags;
 
 #define CPIO_NEWC_MAGIC	"070701"
@@ -90,7 +92,7 @@ struct cpioCrcPhysicalHeader {
     char checksum[8];			/* ignored !! */
 };
 
-#define	PHYS_HDR_SIZE	110		/*!< Don't depend on sizeof(struct) */
+#define	PHYS_HDR_SIZE	110		/* Don't depend on sizeof(struct) */
 
 #ifdef __cplusplus
 extern "C" {
@@ -102,7 +104,7 @@ extern "C" {
  * @return		0 on success
  */
 int cpioTrailerWrite(FSM_t fsm)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fsm, fileSystem, internalState @*/;
 
 /**
@@ -112,7 +114,7 @@ int cpioTrailerWrite(FSM_t fsm)
  * @return		0 on success
  */
 int cpioHeaderWrite(FSM_t fsm, struct stat * st)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fsm, fileSystem, internalState @*/;
 
 /**
@@ -122,7 +124,7 @@ int cpioHeaderWrite(FSM_t fsm, struct stat * st)
  * @return		0 on success
  */
 int cpioHeaderRead(FSM_t fsm, struct stat * st)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fsm, *st, fileSystem, internalState @*/;
 
 /** \ingroup payload
