@@ -78,8 +78,8 @@ static int intcmp(const void * a, const void * b)
  */
 static int removePackage(rpmts ts, Header h, int dboffset,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey depends)
-	/*@globals fileSystem @*/
-	/*@modifies ts, h, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     rpmte p;
 
@@ -127,11 +127,19 @@ static int removePackage(rpmts ts, Header h, int dboffset,
 int rpmtsAddInstallElement(rpmts ts, Header h,
 			fnpyKey key, int upgrade, rpmRelocation * relocs)
 {
-    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    uint_32 tscolor = rpmtsColor(ts);
+    uint_32 dscolor;
+    uint_32 hcolor;
+    rpmdbMatchIterator mi;
+    Header oh;
+    uint_32 ohcolor;
     int isSource;
     int duplicate = 0;
-    rpmtsi pi; rpmte p;
-    rpmds add;
+    rpmtsi pi = NULL; rpmte p;
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    const char * arch;
+    const char * os;
+    rpmds oldChk, newChk;
     rpmds obsoletes;
     alKey pkgKey;	/* addedPackages key */
     int xx;
@@ -140,11 +148,32 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     int oc;
 
     /*
-     * Check for previously added versions with the same name.
+     * Check for previously added versions with the same name and arch/os.
      * FIXME: only catches previously added, older packages.
      */
-    add = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_EQUAL|RPMSENSE_LESS));
+    arch = NULL;
+    xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
+    os = NULL;
+    xx = hge(h, RPMTAG_OS, NULL, (void **)&os, NULL);
+    hcolor = hGetColor(h);
     pkgKey = RPMAL_NOMATCH;
+
+    /* Check for supported payload format if it's a package */
+    if (key && headerCheckPayloadFormat(h) != RPMRC_OK) {
+	ec = 1;
+	goto exit;
+    }
+
+    /* XXX Always add source headers. */
+    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+    if (isSource) {
+	oc = ts->orderCount;
+	goto addheader;
+    }
+
+    oldChk = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_LESS));
+    newChk = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_EQUAL|RPMSENSE_GREATER));
+    /* XXX can't use rpmtsiNext() filter or oc will have wrong value. */
     for (pi = rpmtsiInit(ts), oc = 0; (p = rpmtsiNext(pi, 0)) != NULL; oc++) {
 	rpmds this;
 
@@ -152,27 +181,70 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	if (rpmteType(p) == TR_REMOVED)
 	    continue;
 
+	/* XXX Never check source headers. */
+	if (rpmteIsSource(p))
+	    continue;
+
+	if (tscolor) {
+	    const char * parch;
+	    const char * pos;
+
+	    if (arch == NULL || (parch = rpmteA(p)) == NULL)
+		continue;
+	    if (os == NULL || (pos = rpmteO(p)) == NULL)
+		continue;
+	    if (strcmp(arch, parch) || strcmp(os, pos))
+		continue;
+	}
+
+	/* OK, binary rpm's with same arch and os.  Check NEVR. */
 	if ((this = rpmteDS(p, RPMTAG_NAME)) == NULL)
 	    continue;	/* XXX can't happen */
 
-	rc = rpmdsCompare(add, this);
-	if (rc != 0) {
+	/* 
+ 	 * On upgrade, if newer NEVR was previously added, 
+ 	 * then skip adding older. 
+ 	 */
+	rc = rpmdsCompare(newChk, this);
+	if (upgrade && rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
-	    const char * addNEVR = rpmdsDNEVR(add);
-	    rpmMessage(RPMMESS_WARNING,
-		_("package %s was already added, replacing with %s\n"),
-		(pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
-		(addNEVR ? addNEVR + 2 : "?addNEVR?"));
+	    const char * addNEVR = rpmdsDNEVR(oldChk);
+	    if (rpmIsVerbose())
+		rpmMessage(RPMMESS_WARNING,
+		    _("package %s was already added, skipping %s\n"),
+		    (pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
+		    (addNEVR ? addNEVR + 2 : "?addNEVR?"));
+	    ec = 0;
+	    goto exit;
+	}
+
+	/*
+ 	 * On upgrade, if older NEVR was previously added, 
+ 	 * then replace old with new. 
+ 	 */
+	rc = rpmdsCompare(oldChk, this);
+	if (upgrade && rc != 0) {
+	    const char * pkgNEVR = rpmdsDNEVR(this);
+	    const char * addNEVR = rpmdsDNEVR(newChk);
+	    if (rpmIsVerbose())
+		rpmMessage(RPMMESS_WARNING,
+		    _("package %s was already added, replacing with %s\n"),
+		    (pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
+		    (addNEVR ? addNEVR + 2 : "?addNEVR?"));
 	    duplicate = 1;
 	    pkgKey = rpmteAddedKey(p);
 	    break;
 	}
     }
     pi = rpmtsiFree(pi);
-    add = rpmdsFree(add);
+    oldChk = rpmdsFree(oldChk);
+    newChk = rpmdsFree(newChk);
 
-    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+    /* If newer NEVR was already added, exit now. */
+    if (ec)
+	goto exit;
 
+addheader:
     if (oc >= ts->orderAlloced) {
 	ts->orderAlloced += (oc - ts->orderAlloced) + ts->delta;
 /*@-type +voidabstract @*/
@@ -200,7 +272,7 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     
     pkgKey = rpmalAdd(&ts->addedPackages, pkgKey, rpmteKey(p),
 			rpmteDS(p, RPMTAG_PROVIDENAME),
-			rpmteFI(p, RPMTAG_BASENAMES));
+			rpmteFI(p, RPMTAG_BASENAMES), tscolor);
     if (pkgKey == RPMAL_NOMATCH) {
 /*@-boundswrite@*/
 	ts->order[oc] = rpmteFree(ts->order[oc]);
@@ -210,35 +282,16 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     }
     (void) rpmteSetAddedKey(p, pkgKey);
 
-#ifdef	NOYET
-  /* XXX this needs a search over ts->order, not ts->addedPackages */
-  { uint_32 *pp = NULL;
-
-    /* XXX This should be added always so that packages look alike.
-     * XXX However, there is logic in files.c/depends.c that checks for
-     * XXX existence (rather than value) that will need to change as well.
-     */
-    if (hge(h, RPMTAG_MULTILIBS, NULL, (void **) &pp, NULL))
-	multiLibMask = *pp;
-
-    if (multiLibMask) {
-	for (i = 0; i < pkgNum - 1; i++) {
-	    if (!strcmp (rpmteN(p), al->list[i].name)
-		&& hge(al->list[i].h, RPMTAG_MULTILIBS, NULL,
-				  (void **) &pp, NULL)
-		&& !rpmVersionCompare(p->h, al->list[i].h)
-		&& *pp && !(*pp & multiLibMask))
-		    (void) rpmteSetMultiLib(p, multiLibMask);
-	}
-    }
-  }
-#endif
-
     if (!duplicate) {
 	ts->numAddedPackages++;
     }
 
-    if (!upgrade)
+    /* XXX rpmgi hack: Save header in transaction element if requested. */
+    if (upgrade & 0x2)
+	(void) rpmteSetHeader(p, h);
+
+    /* If not upgrading, then we're done. */
+    if (!(upgrade & 0x1))
 	goto exit;
 
     /* XXX binary rpms always have RPMTAG_SOURCERPM, source rpms do not */
@@ -251,31 +304,23 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	    goto exit;
     }
 
-/*@-boundsread@*/
-    {	rpmdbMatchIterator mi;
-	Header h2;
+    /* On upgrade, erase older packages of same color (if any). */
 
-	mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, rpmteN(p), 0);
-	while((h2 = rpmdbNextIterator(mi)) != NULL) {
-	    if (rpmVersionCompare(h, h2))
-		xx = removePackage(ts, h2, rpmdbGetIteratorOffset(mi), pkgKey);
-	    else {
-		uint_32 *pp, multiLibMask = 0, oldmultiLibMask = 0;
+    mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(p), 0);
+    while((oh = rpmdbNextIterator(mi)) != NULL) {
 
-		if (hge(h2, RPMTAG_MULTILIBS, NULL, (void **) &pp, NULL))
-		    oldmultiLibMask = *pp;
-		if (hge(h, RPMTAG_MULTILIBS, NULL, (void **) &pp, NULL))
-		    multiLibMask = *pp;
-		if (oldmultiLibMask && multiLibMask
-		 && !(oldmultiLibMask & multiLibMask))
-		{
-		    (void) rpmteSetMultiLib(p, multiLibMask);
-		}
-	    }
-	}
-	mi = rpmdbFreeIterator(mi);
+	/* Ignore colored packages not in our rainbow. */
+	ohcolor = hGetColor(oh);
+	if (tscolor && hcolor && ohcolor && !(hcolor & ohcolor))
+	    continue;
+
+	/* Skip packages that contain identical NEVR. */
+	if (rpmVersionCompare(h, oh) == 0)
+	    continue;
+
+	xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), pkgKey);
     }
-/*@=boundsread@*/
+    mi = rpmdbFreeIterator(mi);
 
     obsoletes = rpmdsLink(rpmteDS(p, RPMTAG_OBSOLETENAME), "Obsoletes");
     obsoletes = rpmdsInit(obsoletes);
@@ -286,29 +331,55 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	if ((Name = rpmdsN(obsoletes)) == NULL)
 	    continue;	/* XXX can't happen */
 
+	/* Ignore colored obsoletes not in our rainbow. */
+#if 0
+	dscolor = rpmdsColor(obsoletes);
+#else
+	dscolor = hcolor;
+#endif
+	/* XXX obsoletes are never colored, so this is for future devel. */
+	if (tscolor && dscolor && !(tscolor & dscolor))
+	    continue;
+
 	/* XXX avoid self-obsoleting packages. */
 	if (!strcmp(rpmteN(p), Name))
-		continue;
+	    continue;
 
-	{   rpmdbMatchIterator mi;
-	    Header h2;
+	if (Name[0] == '/')
+	    mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, Name, 0);
+	else
+	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, Name, 0);
 
-	    mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, Name, 0);
+	xx = rpmdbPruneIterator(mi,
+	    ts->removedPackages, ts->numRemovedPackages, 1);
 
-	    xx = rpmdbPruneIterator(mi,
-		ts->removedPackages, ts->numRemovedPackages, 1);
+	while((oh = rpmdbNextIterator(mi)) != NULL) {
+	    /* Ignore colored packages not in our rainbow. */
+	    ohcolor = hGetColor(oh);
+	    /* XXX provides *are* colored, effectively limiting Obsoletes:
+		to matching only colored Provides: based on pkg coloring. */
+	    if (tscolor && hcolor && ohcolor && !(hcolor & ohcolor))
+		/*@innercontinue@*/ continue;
 
-	    while((h2 = rpmdbNextIterator(mi)) != NULL) {
-		/*
-		 * Rpm prior to 3.0.3 does not have versioned obsoletes.
-		 * If no obsoletes version info is available, match all names.
-		 */
-		if (rpmdsEVR(obsoletes) == NULL
-		 || rpmdsAnyMatchesDep(h2, obsoletes, _rpmds_nopromote))
-		    xx = removePackage(ts, h2, rpmdbGetIteratorOffset(mi), pkgKey);
+	    /*
+	     * Rpm prior to 3.0.3 does not have versioned obsoletes.
+	     * If no obsoletes version info is available, match all names.
+	     */
+	    if (rpmdsEVR(obsoletes) == NULL
+	     || rpmdsAnyMatchesDep(oh, obsoletes, _rpmds_nopromote)) {
+		const char * ohNEVRA = hGetNEVRA(oh, NULL);
+#ifdef	DYING	/* XXX see http://bugzilla.redhat.com #134497 */
+		if (rpmVersionCompare(h, oh))
+#endif
+		    xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), pkgKey);
+/*@-nullptrarith@*/
+		rpmMessage(RPMMESS_DEBUG, _("  Obsoletes: %s\t\terases %s\n"),
+			rpmdsDNEVR(obsoletes)+2, ohNEVRA);
+/*@=nullptrarith@*/
+		ohNEVRA = _free(ohNEVRA);
 	    }
-	    mi = rpmdbFreeIterator(mi);
 	}
+	mi = rpmdbFreeIterator(mi);
     }
     obsoletes = rpmdsFree(obsoletes);
 
@@ -332,7 +403,7 @@ int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
  * @return		0 if satisfied, 1 if not satisfied, 2 if error
  */
 static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
-	/*@globals _cacheDependsRC, rpmGlobalMacroContext,
+	/*@globals _cacheDependsRC, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, _cacheDependsRC, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
@@ -377,7 +448,9 @@ static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
 		memset(data, 0, sizeof(*data));
 		data->data = datap;
 		data->size = datalen;
+/*@-nullstate@*/ /* FIX: data->data may be NULL */
 		xx = dbiGet(dbi, dbcursor, key, data, DB_SET);
+/*@=nullstate@*/
 		DNEVR = key->data;
 		DNEVRlen = key->size;
 		datap = data->data;
@@ -565,24 +638,24 @@ exit:
 /**
  * Check added requires/conflicts against against installed+added packages.
  * @param ts		transaction set
- * @param pkgNEVR	package name-version-release
+ * @param pkgNEVRA	package name-version-release.arch
  * @param requires	Requires: dependencies (or NULL)
  * @param conflicts	Conflicts: dependencies (or NULL)
  * @param depName	dependency name to filter (or NULL)
- * @param multiLib	skip multilib colored dependencies?
+ * @param tscolor	color bits for transaction set (0 disables)
  * @param adding	dependency is from added package set?
  * @return		0 no problems found
  */
-static int checkPackageDeps(rpmts ts, const char * pkgNEVR,
+static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 		/*@null@*/ rpmds requires, /*@null@*/ rpmds conflicts,
-		/*@null@*/ const char * depName, uint_32 multiLib, int adding)
-	/*@globals rpmGlobalMacroContext,
+		/*@null@*/ const char * depName, uint_32 tscolor, int adding)
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, requires, conflicts, rpmGlobalMacroContext,
 		fileSystem, internalState */
 {
+    uint_32 dscolor;
     const char * Name;
-    int_32 Flags;
     int rc;
     int ourrc = 0;
 
@@ -597,11 +670,9 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVR,
 	if (depName != NULL && strcmp(depName, Name))
 	    continue;
 
-	Flags = rpmdsFlags(requires);
-
-	/* If this requirement comes from the core package only, not libraries,
-	   then if we're installing the libraries only, don't count it in. */
-	if (multiLib && !isDependsMULTILIB(Flags))
+	/* Ignore colored requires not in our rainbow. */
+	dscolor = rpmdsColor(requires);
+	if (tscolor && dscolor && !(tscolor & dscolor))
 	    continue;
 
 	rc = unsatisfiedDepend(ts, requires, adding);
@@ -619,7 +690,7 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVR,
 	    }
 	    /*@=branchstate@*/
 
-	    rpmdsProblem(ts->probs, pkgNEVR, requires, suggestedKeys, adding);
+	    rpmdsProblem(ts->probs, pkgNEVRA, requires, suggestedKeys, adding);
 
 	}
 	    /*@switchbreak@*/ break;
@@ -641,11 +712,9 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVR,
 	if (depName != NULL && strcmp(depName, Name))
 	    continue;
 
-	Flags = rpmdsFlags(conflicts);
-
-	/* If this requirement comes from the core package only, not libraries,
-	   then if we're installing the libraries only, don't count it in. */
-	if (multiLib && !isDependsMULTILIB(Flags))
+	/* Ignore colored conflicts not in our rainbow. */
+	dscolor = rpmdsColor(conflicts);
+	if (tscolor && dscolor && !(tscolor & dscolor))
 	    continue;
 
 	rc = unsatisfiedDepend(ts, conflicts, adding);
@@ -653,7 +722,7 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVR,
 	/* 1 == unsatisfied, 0 == satsisfied */
 	switch (rc) {
 	case 0:		/* conflicts exist. */
-	    rpmdsProblem(ts->probs, pkgNEVR, conflicts, NULL, adding);
+	    rpmdsProblem(ts->probs, pkgNEVRA, conflicts, NULL, adding);
 	    /*@switchbreak@*/ break;
 	case 1:		/* conflicts don't exist. */
 	    /*@switchbreak@*/ break;
@@ -679,7 +748,7 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVR,
  */
 static int checkPackageSet(rpmts ts, const char * dep,
 		/*@only@*/ /*@null@*/ rpmdbMatchIterator mi, int adding)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, mi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     int scareMem = 1;
@@ -689,19 +758,19 @@ static int checkPackageSet(rpmts ts, const char * dep,
     (void) rpmdbPruneIterator(mi,
 		ts->removedPackages, ts->numRemovedPackages, 1);
     while ((h = rpmdbNextIterator(mi)) != NULL) {
-	const char * pkgNEVR;
+	const char * pkgNEVRA;
 	rpmds requires, conflicts;
 	int rc;
 
-	pkgNEVR = hGetNEVR(h, NULL);
+	pkgNEVRA = hGetNEVRA(h, NULL);
 	requires = rpmdsNew(h, RPMTAG_REQUIRENAME, scareMem);
 	(void) rpmdsSetNoPromote(requires, _rpmds_nopromote);
 	conflicts = rpmdsNew(h, RPMTAG_CONFLICTNAME, scareMem);
-	(void) rpmdsSetNoPromote(requires, _rpmds_nopromote);
-	rc = checkPackageDeps(ts, pkgNEVR, requires, conflicts, dep, 0, adding);
+	(void) rpmdsSetNoPromote(conflicts, _rpmds_nopromote);
+	rc = checkPackageDeps(ts, pkgNEVRA, requires, conflicts, dep, 0, adding);
 	conflicts = rpmdsFree(conflicts);
 	requires = rpmdsFree(requires);
-	pkgNEVR = _free(pkgNEVR);
+	pkgNEVRA = _free(pkgNEVRA);
 
 	if (rc) {
 	    ec = 1;
@@ -720,7 +789,7 @@ static int checkPackageSet(rpmts ts, const char * dep,
  * @return		0 no problems found
  */
 static int checkDependentPackages(rpmts ts, const char * dep)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     rpmdbMatchIterator mi;
@@ -735,7 +804,7 @@ static int checkDependentPackages(rpmts ts, const char * dep)
  * @return		0 no problems found
  */
 static int checkDependentConflicts(rpmts ts, const char * dep)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     int rc = 0;
@@ -761,28 +830,6 @@ static struct badDeps_s {
 /*@observer@*/ /*@null@*/ const char * pname;
 /*@observer@*/ /*@null@*/ const char * qname;
 } badDeps[] = {
-    { "libtermcap", "bash" },
-    { "modutils", "vixie-cron" },
-    { "ypbind", "yp-tools" },
-    { "ghostscript-fonts", "ghostscript" },
-    /* 7.2 only */
-    { "libgnomeprint15", "gnome-print" },
-    { "nautilus", "nautilus-mozilla" },
-    /* 7.1 only */
-    { "arts", "kdelibs-sound" },
-    /* 7.0 only */
-    { "pango-gtkbeta-devel", "pango-gtkbeta" },
-    { "XFree86", "Mesa" },
-    { "compat-glibc", "db2" },
-    { "compat-glibc", "db1" },
-    { "pam", "initscripts" },
-    { "initscripts", "sysklogd" },
-    /* 6.2 */
-    { "egcs-c++", "libstdc++" },
-    /* 6.1 */
-    { "pilot-link-devel", "pilot-link" },
-    /* 5.2 */
-    { "pam", "pamconfig" },
     { NULL, NULL }
 };
 #else
@@ -813,21 +860,26 @@ static void freeBadDeps(void)
 /**
  * Check for dependency relations to be ignored.
  *
- * @param p	successor element (i.e. with Requires: )
- * @param q	predecessor element (i.e. with Provides: )
- * @return	1 if dependency is to be ignored.
+ * @param ts		transaction set
+ * @param p		successor element (i.e. with Requires: )
+ * @param q		predecessor element (i.e. with Provides: )
+ * @return		1 if dependency is to be ignored.
  */
 /*@-boundsread@*/
-static int ignoreDep(const rpmte p, const rpmte q)
-	/*@globals badDeps, badDepsInitialized @*/
-	/*@modifies badDeps, badDepsInitialized @*/
+static int ignoreDep(const rpmts ts, const rpmte p, const rpmte q)
+	/*@globals badDeps, badDepsInitialized,
+		rpmGlobalMacroContext, h_errno @*/
+	/*@modifies badDeps, badDepsInitialized,
+		rpmGlobalMacroContext @*/
 {
     struct badDeps_s * bdp;
 
-/*@-globs -mods@*/
     if (!badDepsInitialized) {
 	char * s = rpmExpand("%{?_dependency_whiteout}", NULL);
 	const char ** av = NULL;
+	int anaconda = rpmtsFlags(ts) & RPMTRANS_FLAG_ANACONDA;
+	int msglvl = (anaconda || (rpmtsFlags(ts) & RPMTRANS_FLAG_DEPLOOPS))
+			? RPMMESS_WARNING : RPMMESS_DEBUG;
 	int ac = 0;
 	int i;
 
@@ -848,7 +900,7 @@ static int ignoreDep(const rpmte p, const rpmte q)
 		/*@-usereleased@*/
 		bdp->qname = qname;
 		/*@=usereleased@*/
-		rpmMessage(RPMMESS_DEBUG,
+		rpmMessage(msglvl,
 			_("ignore package name relation(s) [%d]\t%s -> %s\n"),
 			i, bdp->pname, (bdp->qname ? bdp->qname : "???"));
 	    }
@@ -859,7 +911,6 @@ static int ignoreDep(const rpmte p, const rpmte q)
 	s = _free(s);
 	badDepsInitialized++;
     }
-/*@=globs =mods@*/
 
     /*@-compdef@*/
     if (badDeps != NULL)
@@ -901,8 +952,6 @@ static void markLoop(/*@special@*/ tsortInfo tsi, rpmte q)
 static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
 	/*@*/
 {
-    if (isLegacyPreReq(f))
-	return "PreReq:";
     f = _notpre(f);
     if (f & RPMSENSE_SCRIPT_PRE)
 	return "Requires(pre):";
@@ -929,6 +978,7 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
  * @param requires	relation
  * @param zap		max. no. of co-requisites to remove (-1 is all)?
  * @retval nzaps	address of no. of relations removed
+ * @param msglvl	message level at which to spew
  * @return		(possibly NULL) formatted "q <- p" releation (malloc'ed)
  */
 /*@-boundswrite@*/
@@ -936,7 +986,7 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
 static /*@owned@*/ /*@null@*/ const char *
 zapRelation(rpmte q, rpmte p,
 		/*@null@*/ rpmds requires,
-		int zap, /*@in@*/ /*@out@*/ int * nzaps)
+		int zap, /*@in@*/ /*@out@*/ int * nzaps, int msglvl)
 	/*@modifies q, p, requires, *nzaps @*/
 {
     tsortInfo tsi_prev;
@@ -952,10 +1002,10 @@ zapRelation(rpmte q, rpmte p,
     {
 	int_32 Flags;
 
-	/*@-abstract@*/
+	/*@-abstractcompare@*/
 	if (tsi->tsi_suc != p)
 	    continue;
-	/*@=abstract@*/
+	/*@=abstractcompare@*/
 
 	if (requires == NULL) continue;		/* XXX can't happen */
 
@@ -969,10 +1019,10 @@ zapRelation(rpmte q, rpmte p,
 	 * Attempt to unravel a dependency loop by eliminating Requires's.
 	 */
 	/*@-branchstate@*/
-	if (zap && !(Flags & RPMSENSE_PREREQ)) {
-	    rpmMessage(RPMMESS_DEBUG,
+	if (zap && !(isErasePreReq(Flags) || isInstallPreReq(Flags))) {
+	    rpmMessage(msglvl,
 			_("removing %s \"%s\" from tsort relations.\n"),
-			(rpmteNEVR(p) ?  rpmteNEVR(p) : "???"), dp);
+			(rpmteNEVRA(p) ?  rpmteNEVRA(p) : "???"), dp);
 	    rpmteTSI(p)->tsi_count--;
 	    if (tsi_prev) tsi_prev->tsi_next = tsi->tsi_next;
 	    tsi->tsi_next = NULL;
@@ -1005,8 +1055,9 @@ static inline int addRelation(rpmts ts,
 		/*@dependent@*/ rpmte p,
 		unsigned char * selected,
 		rpmds requires)
-	/*@globals fileSystem @*/
-	/*@modifies ts, p, *selected, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, p, *selected, rpmGlobalMacroContext,
+		fileSystem, internalState @*/
 {
     rpmtsi qi; rpmte q;
     tsortInfo tsi;
@@ -1016,12 +1067,16 @@ static inline int addRelation(rpmts ts,
     int i = 0;
 
     if ((Name = rpmdsN(requires)) == NULL)
-	return 0;	/* XXX can't happen */
+	return 0;
 
     /* Avoid rpmlib feature dependencies. */
     if (!strncmp(Name, "rpmlib(", sizeof("rpmlib(")-1))
 	return 0;
 
+    /* Avoid package config dependencies. */
+    if (!strncmp(Name, "config(", sizeof("config(")-1))
+	return 0;
+
     pkgKey = RPMAL_NOMATCH;
     key = rpmalSatisfiesDepend(ts->addedPackages, requires, &pkgKey);
 
@@ -1045,7 +1100,7 @@ static inline int addRelation(rpmts ts,
 	return 0;
 
     /* Avoid certain dependency relations. */
-    if (ignoreDep(p, q))
+    if (ignoreDep(ts, p, q))
 	return 0;
 
     /* Avoid redundant relations. */
@@ -1063,6 +1118,8 @@ static inline int addRelation(rpmts ts,
 
     if (rpmteDepth(p) <= rpmteDepth(q))	/* Save max. depth in dependency tree */
 	(void) rpmteSetDepth(p, (rpmteDepth(q) + 1));
+    if (rpmteDepth(p) > ts->maxDepth)
+	ts->maxDepth = rpmteDepth(p);
 
     tsi = xcalloc(1, sizeof(*tsi));
     tsi->tsi_suc = p;
@@ -1070,10 +1127,8 @@ static inline int addRelation(rpmts ts,
     tsi->tsi_reqx = rpmdsIx(requires);
 
     tsi->tsi_next = rpmteTSI(q)->tsi_next;
-/*@-mods@*/
     rpmteTSI(q)->tsi_next = tsi;
     rpmteTSI(q)->tsi_qcnt++;			/* bump q successor count */
-/*@=mods@*/
     return 0;
 }
 /*@=mustmod@*/
@@ -1103,7 +1158,8 @@ static int orderListIndexCmp(const void * one, const void * two)	/*@*/
 /*@-mustmod@*/
 static void addQ(/*@dependent@*/ rpmte p,
 		/*@in@*/ /*@out@*/ rpmte * qp,
-		/*@in@*/ /*@out@*/ rpmte * rp)
+		/*@in@*/ /*@out@*/ rpmte * rp,
+		uint_32 prefcolor)
 	/*@modifies p, *qp, *rp @*/
 {
     rpmte q, qprev;
@@ -1123,6 +1179,10 @@ static void addQ(/*@dependent@*/ rpmte p,
 	 q != NULL;
 	 qprev = q, q = rpmteTSI(q)->tsi_suc)
     {
+	/* XXX Insure preferred color first. */
+	if (rpmteColor(p) != prefcolor && rpmteColor(p) != rpmteColor(q))
+	    continue;
+
 	if (rpmteTSI(q)->tsi_qcnt <= rpmteTSI(p)->tsi_qcnt)
 	    break;
     }
@@ -1150,12 +1210,8 @@ int rpmtsOrder(rpmts ts)
 {
     rpmds requires;
     int_32 Flags;
-
-#ifdef	DYING
-    int chainsaw = rpmtsFlags(ts) & RPMTRANS_FLAG_CHAINSAW;
-#else
-    int chainsaw = 1;
-#endif
+    int anaconda = rpmtsFlags(ts) & RPMTRANS_FLAG_ANACONDA;
+    uint_32 prefcolor = rpmtsPrefColor(ts);
     rpmtsi pi; rpmte p;
     rpmtsi qi; rpmte q;
     rpmtsi ri; rpmte r;
@@ -1169,22 +1225,26 @@ int rpmtsOrder(rpmts ts)
     int newOrderCount = 0;
     orderListIndex orderList;
     int numOrderList;
+    int npeer = 128;	/* XXX more than deep enough for now. */
+    int * peer = memset(alloca(npeer*sizeof(*peer)), 0, (npeer*sizeof(*peer)));
     int nrescans = 10;
     int _printed = 0;
     char deptypechar;
-#ifdef	DYING
-    int oType = TR_ADDED;
-#else
+    size_t tsbytes;
     int oType = 0;
-#endif
     int treex;
     int depth;
+    int breadth;
     int qlen;
     int i, j;
 
-#ifdef	DYING
+    /*
+     * XXX FIXME: this gets needlesly called twice on normal usage patterns,
+     * should track the need for generating the index somewhere
+     */
     rpmalMakeIndex(ts->addedPackages);
-#endif
+
+    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_ORDER), 0);
 
     /* T1. Initialize. */
     if (oType == 0)
@@ -1198,6 +1258,7 @@ int rpmtsOrder(rpmts ts)
      }
     ordering = alloca(sizeof(*ordering) * (numOrderList + 1));
     loopcheck = numOrderList;
+    tsbytes = 0;
 
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, oType)) != NULL)
@@ -1207,7 +1268,6 @@ int rpmtsOrder(rpmts ts)
     /* Record all relations. */
     rpmMessage(RPMMESS_DEBUG, _("========== recording tsort relations\n"));
     pi = rpmtsiInit(ts);
-    /* XXX Only added packages are ordered (for now). */
     while ((p = rpmtsiNext(pi, oType)) != NULL) {
 
 	if ((requires = rpmteDS(p, RPMTAG_REQUIRENAME)) == NULL)
@@ -1230,18 +1290,12 @@ int rpmtsOrder(rpmts ts)
 	    switch (rpmteType(p)) {
 	    case TR_REMOVED:
 		/* Skip if not %preun/%postun requires or legacy prereq. */
-		if (isInstallPreReq(Flags)
-		 || !( isErasePreReq(Flags)
-		    || isLegacyPreReq(Flags) )
-		    )
+		if (!( isErasePreReq(Flags) || isLegacyPreReq(Flags) ) )
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    case TR_ADDED:
 		/* Skip if not %pre/%post requires or legacy prereq. */
-		if (isErasePreReq(Flags)
-		 || !( isInstallPreReq(Flags)
-		    || isLegacyPreReq(Flags) )
-		    )
+		if (!( isInstallPreReq(Flags) || isLegacyPreReq(Flags) ) )
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    }
@@ -1262,17 +1316,13 @@ int rpmtsOrder(rpmts ts)
 	    case TR_REMOVED:
 		/* Skip if %preun/%postun requires or legacy prereq. */
 		if (isInstallPreReq(Flags)
-		 ||  ( isErasePreReq(Flags)
-		    || isLegacyPreReq(Flags) )
-		    )
+		 ||  ( isErasePreReq(Flags) || isLegacyPreReq(Flags) ) )
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    case TR_ADDED:
 		/* Skip if %pre/%post requires or legacy prereq. */
 		if (isErasePreReq(Flags)
-		 ||  ( isInstallPreReq(Flags)
-		    || isLegacyPreReq(Flags) )
-		    )
+		 ||  ( isInstallPreReq(Flags) || isLegacyPreReq(Flags) ) )
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    }
@@ -1293,6 +1343,7 @@ int rpmtsOrder(rpmts ts)
 	npreds = rpmteTSI(p)->tsi_count;
 
 	(void) rpmteSetNpreds(p, npreds);
+	(void) rpmteSetDepth(p, 1);
 
 	if (npreds == 0)
 	    (void) rpmteSetTree(p, treex++);
@@ -1304,9 +1355,10 @@ int rpmtsOrder(rpmts ts)
 
     }
     pi = rpmtsiFree(pi);
+    ts->ntrees = treex;
 
     /* T4. Scan for zeroes. */
-    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, depth)\n"));
+    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, depth, breadth)\n"));
 
 rescan:
     if (pi != NULL) pi = rpmtsiFree(pi);
@@ -1315,14 +1367,14 @@ rescan:
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, oType)) != NULL) {
 
-	/* Prefer packages in chainsaw or presentation order. */
-	if (!chainsaw)
+	/* Prefer packages in chainsaw or anaconda presentation order. */
+	if (anaconda)
 	    rpmteTSI(p)->tsi_qcnt = (ts->orderCount - rpmtsiOc(pi));
 
 	if (rpmteTSI(p)->tsi_count != 0)
 	    continue;
 	rpmteTSI(p)->tsi_suc = NULL;
-	addQ(p, &q, &r);
+	addQ(p, &q, &r, prefcolor);
 	qlen++;
     }
     pi = rpmtsiFree(pi);
@@ -1349,18 +1401,30 @@ rescan:
 	}
 	deptypechar = (rpmteType(q) == TR_REMOVED ? '-' : '+');
 
-	rpmMessage(RPMMESS_DEBUG, "%5d%5d%5d%5d%5d %*s%c%s\n",
-			orderingCount, rpmteNpreds(q),
-			rpmteTSI(q)->tsi_qcnt, rpmteTree(q), rpmteDepth(q),
-			(2 * rpmteDepth(q)), "",
-			deptypechar,
-			(rpmteNEVR(q) ? rpmteNEVR(q) : "???"));
-
 	treex = rpmteTree(q);
 	depth = rpmteDepth(q);
-	(void) rpmteSetDegree(q, 0);
+	breadth = ((depth < npeer) ? peer[depth]++ : 0);
+	(void) rpmteSetBreadth(q, breadth);
 
-	ordering[orderingCount] = rpmteAddedKey(q);
+	rpmMessage(RPMMESS_DEBUG, "%5d%5d%5d%5d%5d%5d %*s%c%s\n",
+			orderingCount, rpmteNpreds(q),
+			rpmteTSI(q)->tsi_qcnt,
+			treex, depth, breadth,
+			(2 * depth), "",
+			deptypechar,
+			(rpmteNEVRA(q) ? rpmteNEVRA(q) : "???"));
+
+	(void) rpmteSetDegree(q, 0);
+	tsbytes += rpmtePkgFileSize(q);
+
+	switch (rpmteType(q)) {
+	case TR_ADDED:
+            ordering[orderingCount] = rpmteAddedKey(q);
+            /*@switchbreak@*/ break;
+	case TR_REMOVED:
+            ordering[orderingCount] = RPMAL_NOMATCH;
+            /*@switchbreak@*/ break;
+        }
 	orderingCount++;
 	qlen--;
 	loopcheck--;
@@ -1381,7 +1445,7 @@ rescan:
 
 		/* XXX TODO: add control bit. */
 		rpmteTSI(p)->tsi_suc = NULL;
-		addQ(p, &rpmteTSI(q)->tsi_suc, &r);
+		addQ(p, &rpmteTSI(q)->tsi_suc, &r, prefcolor);
 		qlen++;
 	    }
 	    tsi = _free(tsi);
@@ -1390,7 +1454,7 @@ rescan:
 	    _printed++;
 	    (void) rpmtsUnorderedSuccessors(ts, orderingCount);
 	    rpmMessage(RPMMESS_DEBUG,
-		_("========== successors only (presentation order)\n"));
+		_("========== successors only (%d bytes)\n"), (int)tsbytes);
 
 	    /* Relink the queue in presentation order. */
 	    tsi = rpmteTSI(q);
@@ -1455,12 +1519,15 @@ rescan:
 	    while ((p = q) != NULL && (q = rpmteTSI(p)->tsi_chain) != NULL) {
 		const char * dp;
 		char buf[4096];
+		int msglvl = (anaconda || (rpmtsFlags(ts) & RPMTRANS_FLAG_DEPLOOPS))
+			? RPMMESS_WARNING : RPMMESS_DEBUG;
+;
 
 		/* Unchain predecessor loop. */
 		rpmteTSI(p)->tsi_chain = NULL;
 
 		if (!printed) {
-		    rpmMessage(RPMMESS_DEBUG, _("LOOP:\n"));
+		    rpmMessage(msglvl, _("LOOP:\n"));
 		    printed = 1;
 		}
 
@@ -1469,13 +1536,13 @@ rescan:
 		requires = rpmdsInit(requires);
 		if (requires == NULL)
 		    /*@innercontinue@*/ continue;	/* XXX can't happen */
-		dp = zapRelation(q, p, requires, 1, &nzaps);
+		dp = zapRelation(q, p, requires, 1, &nzaps, msglvl);
 
 		/* Print next member of loop. */
 		buf[0] = '\0';
-		if (rpmteNEVR(p) != NULL)
-		    (void) stpcpy(buf, rpmteNEVR(p));
-		rpmMessage(RPMMESS_DEBUG, "    %-40s %s\n", buf,
+		if (rpmteNEVRA(p) != NULL)
+		    (void) stpcpy(buf, rpmteNEVRA(p));
+		rpmMessage(msglvl, "    %-40s %s\n", buf,
 			(dp ? dp : "not found!?!"));
 
 		dp = _free(dp);
@@ -1554,12 +1621,12 @@ rescan:
 	    continue;
 
 	j = needle->orIndex;
-	if ((q = ts->order[j]) == NULL)
+	if ((q = ts->order[j]) == NULL || needle->pkgKey == RPMAL_NOMATCH)
 	    continue;
 
 	newOrder[newOrderCount++] = q;
 	ts->order[j] = NULL;
-	if (!chainsaw)
+	if (anaconda)
 	for (j = needle->orIndex + 1; j < ts->orderCount; j++) {
 	    if ((q = ts->order[j]) == NULL)
 		/*@innerbreak@*/ break;
@@ -1594,18 +1661,23 @@ assert(newOrderCount == ts->orderCount);
 #endif
     freeBadDeps();
 
+    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_ORDER), 0);
+
     return 0;
 }
 /*@=bounds@*/
 
 int rpmtsCheck(rpmts ts)
 {
+    uint_32 tscolor = rpmtsColor(ts);
     rpmdbMatchIterator mi = NULL;
     rpmtsi pi = NULL; rpmte p;
     int closeatexit = 0;
     int xx;
     int rc;
 
+    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_CHECK), 0);
+
     /* Do lazy, readonly, open of rpm database. */
     if (rpmtsGetRdb(ts) == NULL && ts->dbmode != -1) {
 	if ((rc = rpmtsOpenDB(ts, ts->dbmode)) != 0)
@@ -1626,28 +1698,22 @@ int rpmtsCheck(rpmts ts)
     while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
 	rpmds provides;
 
-        rpmMessage(RPMMESS_DEBUG,  "========== +++ %s\n" , rpmteNEVR(p));
-	rc = checkPackageDeps(ts, rpmteNEVR(p),
+/*@-nullpass@*/	/* FIX: rpmts{A,O} can return null. */
+	rpmMessage(RPMMESS_DEBUG, "========== +++ %s %s/%s 0x%x\n",
+		rpmteNEVR(p), rpmteA(p), rpmteO(p), rpmteColor(p));
+/*@=nullpass@*/
+	rc = checkPackageDeps(ts, rpmteNEVRA(p),
 			rpmteDS(p, RPMTAG_REQUIRENAME),
 			rpmteDS(p, RPMTAG_CONFLICTNAME),
 			NULL,
-			rpmteMultiLib(p), 1);
+			tscolor, 1);
 	if (rc)
 	    goto exit;
 
-#if defined(DYING) || defined(__LCLINT__)
-	/* XXX all packages now have Provides: name = version-release */
-	/* Adding: check name against conflicts matches. */
-	rc = checkDependentConflicts(ts, rpmteN(p));
-	if (rc)
-	    goto exit;
-#endif
-
 	rc = 0;
 	provides = rpmteDS(p, RPMTAG_PROVIDENAME);
 	provides = rpmdsInit(provides);
-	if (provides == NULL || rpmdsN(provides) == NULL)
-	    continue;
+	if (provides != NULL)
 	while (rpmdsNext(provides) >= 0) {
 	    const char * Name;
 
@@ -1673,7 +1739,10 @@ int rpmtsCheck(rpmts ts)
 	rpmds provides;
 	rpmfi fi;
 
-	rpmMessage(RPMMESS_DEBUG,  "========== --- %s\n" , rpmteNEVR(p));
+/*@-nullpass@*/	/* FIX: rpmts{A,O} can return null. */
+	rpmMessage(RPMMESS_DEBUG, "========== --- %s %s/%s 0x%x\n",
+		rpmteNEVR(p), rpmteA(p), rpmteO(p), rpmteColor(p));
+/*@=nullpass@*/
 
 #if defined(DYING) || defined(__LCLINT__)
 	/* XXX all packages now have Provides: name = version-release */
@@ -1724,6 +1793,9 @@ int rpmtsCheck(rpmts ts)
 exit:
     mi = rpmdbFreeIterator(mi);
     pi = rpmtsiFree(pi);
+
+    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_CHECK), 0);
+
     /*@-branchstate@*/
     if (closeatexit)
 	xx = rpmtsCloseDB(ts);
