@@ -6,6 +6,7 @@
 
 #include <rpmio_internal.h>
 #include <rpmlib.h>
+#include <rpmmacro.h>
 
 #include "stringbuf.h"
 #include "manifest.h"
@@ -65,7 +66,7 @@ char * rpmPermsString(int mode)
 
 /**@todo Infinite loops through manifest files exist, operator error for now. */
 /*@-boundsread@*/
-int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
+rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 {
     StringBuf sb = newStringBuf();
     char * s = NULL;
@@ -77,8 +78,8 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 /*@+voidabstract@*/
     FILE * f = (FILE *) fdGetFp(fd);
 /*@=voidabstract@*/
-    int rc = 0;
-    int i;
+    rpmRC rpmrc = RPMRC_OK;
+    int i, j, next, npre;
 
 /*@-boundswrite@*/
     if (f != NULL)
@@ -105,7 +106,7 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 
 	/* Insure that file contains only ASCII */
 	if (*s < 32) {
-	    rc = 1;
+	    rpmrc = RPMRC_NOTFOUND;
 	    goto exit;
 	}
 
@@ -121,27 +122,41 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
     /*@=branchstate@*/
 
     if (!(s && *s)) {
-	rc = 1;
+	rpmrc = RPMRC_NOTFOUND;
 	goto exit;
     }
 
     /* Glob manifest items. */
-    rc = rpmGlob(s, &ac, &av);
-    if (rc) goto exit;
+    rpmrc = rpmGlob(s, &ac, &av);
+    if (rpmrc != RPMRC_OK) goto exit;
 
-    /* Find 1st existing unprocessed arg. */
-    for (i = 0; i < argc; i++)
-	if (argv && argv[i]) break;
+    rpmMessage(RPMMESS_DEBUG, _("adding %d args from manifest.\n"), ac);
 
-    /* Concatenate existing unprocessed args after manifest contents. */
-    if (argv && i < argc) {
-	int nac = ac + (argc - i);
+    /* Count non-NULL args, keeping track of 1st arg after last NULL. */
+    npre = 0;
+    next = 0;
+    if (argv != NULL)
+    for (i = 0; i < argc; i++) {
+	if (argv[i] != NULL)
+	    npre++;
+	else if (i >= next)
+	    next = i + 1;
+    }
+
+    /* Copy old arg list, inserting manifest before argv[next]. */
+    if (argv != NULL) {
+	int nac = npre + ac;
 	const char ** nav = xcalloc((nac + 1), sizeof(*nav));
 
+	for (i = 0, j = 0; i < next; i++) {
+	    if (argv[i] != NULL)
+		nav[j++] = argv[i];
+	}
+
 	if (ac)
-	    memcpy(nav, av, ac * sizeof(*nav));
-	if ((argc - i) > 0)
-	    memcpy(nav + ac, argv + i, (argc - i) * sizeof(*nav));
+	    memcpy(nav + j, av, ac * sizeof(*nav));
+	if ((argc - next) > 0)
+	    memcpy(nav + j + ac, argv + next, (argc - next) * sizeof(*nav));
 	nav[nac] = NULL;
 
 	if (argvPtr)
@@ -162,7 +177,7 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 
 exit:
     /*@-branchstate@*/
-    if (argvPtr == NULL || (rc != 0 && av)) {
+    if (argvPtr == NULL || (rpmrc != RPMRC_OK && av)) {
 	if (av)
 /*@-boundswrite@*/
 	for (i = 0; i < ac; i++)
@@ -173,7 +188,7 @@ exit:
     /*@=branchstate@*/
     sb = freeStringBuf(sb);
     /*@-nullstate@*/ /* FIX: *argvPtr may be NULL. */
-    return rc;
+    return rpmrc;
     /*@=nullstate@*/
 }
 /*@=boundsread@*/
