@@ -14,7 +14,20 @@ extern time_t get_date(const char * p, void * now);	/* XXX expedient lies */
 /*@=redecl@*/
 
 /*@unchecked@*/
-struct rpmInstallArguments_s rpmIArgs;
+struct rpmInstallArguments_s rpmIArgs = {
+    0,			/* transFlags */
+    0,			/* probFilter */
+    0,			/* installInterfaceFlags */
+    0,			/* eraseInterfaceFlags */
+    0,			/* qva_flags */
+    0,			/* rbtid */
+    0,			/* numRelocations */
+    0,			/* noDeps */
+    0,			/* incldocs */
+    NULL,		/* relocations */
+    NULL,		/* prefix */
+    NULL		/* rootdir */
+};
 
 #define	POPT_RELOCATE		-1021
 #define	POPT_EXCLUDEPATH	-1022
@@ -120,6 +133,10 @@ static void installArgCallback( /*@unused@*/ poptContext con,
 	ia->transFlags |= RPMTRANS_FLAG_NOMD5;
 	break;
 
+    case RPMCLI_POPT_NOCONTEXTS:
+	ia->transFlags |= RPMTRANS_FLAG_NOCONTEXTS;
+	break;
+
     case RPMCLI_POPT_FORCE:
 	ia->probFilter |=
 		( RPMPROB_FILTER_REPLACEPKG
@@ -160,7 +177,7 @@ struct poptOption rpmInstallPoptTable[] = {
 	NULL},
 
  { "anaconda", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
- 	&rpmIArgs.transFlags, RPMTRANS_FLAG_ANACONDA,
+ 	&rpmIArgs.transFlags, RPMTRANS_FLAG_ANACONDA|RPMTRANS_FLAG_DEPLOOPS,
 	N_("use anaconda \"presentation order\""), NULL},
 
  { "apply", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN, &rpmIArgs.transFlags,
@@ -170,7 +187,12 @@ struct poptOption rpmInstallPoptTable[] = {
 
  { "badreloc", '\0', POPT_BIT_SET,
 	&rpmIArgs.probFilter, RPMPROB_FILTER_FORCERELOCATE,
-	N_("relocate files in non-relocateable package"), NULL},
+	N_("relocate files in non-relocatable package"), NULL},
+
+ { "deploops", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ 	&rpmIArgs.transFlags, RPMTRANS_FLAG_DEPLOOPS,
+	N_("print dependency loops as warning"), NULL},
+
  { "dirstash", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmIArgs.transFlags, RPMTRANS_FLAG_DIRSTASH,
 	N_("save erased package files by renaming into sub-directory"), NULL},
@@ -187,6 +209,9 @@ struct poptOption rpmInstallPoptTable[] = {
 	N_("skip files with leading component <path> "),
 	N_("<path>") },
 
+ { "fileconflicts", '\0', POPT_BIT_CLR, &rpmIArgs.probFilter,
+	(RPMPROB_FILTER_REPLACEOLDFILES | RPMPROB_FILTER_REPLACENEWFILES),
+	N_("detect file conflicts between packages"), NULL},
  { "force", '\0', 0, NULL, RPMCLI_POPT_FORCE,
 	N_("short hand for --replacepkgs --replacefiles"), NULL},
 
@@ -225,6 +250,9 @@ struct poptOption rpmInstallPoptTable[] = {
 
  { "nomd5", '\0', 0, NULL, RPMCLI_POPT_NOMD5,
 	N_("don't verify MD5 digest of files"), NULL },
+ { "nocontexts", '\0',0,  NULL, RPMCLI_POPT_NOCONTEXTS,
+	N_("don't install file security contexts"), NULL},
+
  { "noorder", '\0', POPT_BIT_SET,
 	&rpmIArgs.installInterfaceFlags, INSTALL_NOORDER,
 	N_("do not reorder package installation to satisfy dependencies"),
@@ -290,7 +318,7 @@ struct poptOption rpmInstallPoptTable[] = {
 	N_("save erased package files by repackaging"), NULL},
  { "replacefiles", '\0', POPT_BIT_SET, &rpmIArgs.probFilter,
 	(RPMPROB_FILTER_REPLACEOLDFILES | RPMPROB_FILTER_REPLACENEWFILES),
-	N_("install even if the package replaces installed files"), NULL},
+	N_("ignore file conflicts between packages"), NULL},
  { "replacepkgs", '\0', POPT_BIT_SET,
 	&rpmIArgs.probFilter, RPMPROB_FILTER_REPLACEPKG,
 	N_("reinstall if the package is already present"), NULL},
