@@ -15,6 +15,9 @@
 
 #include "rpmdb.h"
 #include "rpmfi.h"
+
+#define	_RPMGI_INTERNAL	/* XXX for gi->flags */
+#include "rpmgi.h"
 #include "rpmts.h"
 
 #include "manifest.h"
@@ -22,6 +25,8 @@
 
 #include "debug.h"
 
+/*@access rpmgi @*/
+
 /**
  */
 static void printFileInfo(char * te, const char * name,
@@ -119,32 +124,55 @@ static inline /*@null@*/ const char * queryHeader(Header h, const char * qfmt)
     return str;
 }
 
+/**
+ */
+static void flushBuffer(char ** tp, char ** tep, int nonewline)
+	/*@ modifies *tp, *tep @*/
+{
+    char *t, *te;
+
+    t = *tp;
+    te = *tep;
+    if (te > t) {
+	if (!nonewline) {
+	    *te++ = '\n';
+	    *te = '\0';
+	}
+	rpmMessage(RPMMESS_NORMAL, "%s", t);
+	te = t;
+	*t = '\0';
+    }
+    *tp = t;
+    *tep = te;
+}
+
 int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 {
     int scareMem = 0;
     rpmfi fi = NULL;
+    size_t tb = 2 * BUFSIZ;
+    size_t sb;
     char * t, * te;
     char * prefix = NULL;
     int rc = 0;		/* XXX FIXME: need real return code */
-    int nonewline = 0;
     int i;
 
-    te = t = xmalloc(BUFSIZ);
+    te = t = xmalloc(tb);
 /*@-boundswrite@*/
     *te = '\0';
 /*@=boundswrite@*/
 
     if (qva->qva_queryFormat != NULL) {
 	const char * str = queryHeader(h, qva->qva_queryFormat);
-	nonewline = 1;
 	/*@-branchstate@*/
 	if (str) {
-	    size_t tb = (te - t);
-	    size_t sb = strlen(str);
+	    size_t tx = (te - t);
 
-	    if (sb >= (BUFSIZ - tb)) {
-		t = xrealloc(t, BUFSIZ+sb);
-		te = t + tb;
+	    sb = strlen(str);
+	    if (sb) {
+		tb += sb;
+		t = xrealloc(t, tb);
+		te = t + tx;
 	    }
 /*@-boundswrite@*/
 	    /*@-usereleased@*/
@@ -152,6 +180,7 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	    /*@=usereleased@*/
 /*@=boundswrite@*/
 	    str = _free(str);
+	    flushBuffer(&t, &te, 1);
 	}
 	/*@=branchstate@*/
     }
@@ -217,9 +246,18 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	    continue;
 
 	/* If not querying %ghost, skip ghost files. */
-	if (!(qva->qva_fflags & RPMFILE_GHOST) && (fflags & RPMFILE_GHOST))
+	if ((qva->qva_fflags & RPMFILE_GHOST) && (fflags & RPMFILE_GHOST))
 	    continue;
 
+	/* Insure space for header derived data */
+	sb = strlen(fn) + strlen(fmd5) + strlen(fuser) + strlen(fgroup) + strlen(flink);
+	if ((sb + BUFSIZ) > tb) {
+	    size_t tx = (te - t);
+	    tb += sb + BUFSIZ;
+	    t = xrealloc(t, tb);
+	    te = t + tx;
+	}
+
 /*@-boundswrite@*/
 	if (!rpmIsVerbose() && prefix)
 	    te = stpcpy(te, prefix);
@@ -299,31 +337,13 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 			_("package has neither file owner or id lists\n"));
 	    }
 	}
-/*@-branchstate@*/
-	if (te > t) {
-/*@-boundswrite@*/
-	    *te++ = '\n';
-	    *te = '\0';
-	    rpmMessage(RPMMESS_NORMAL, "%s", t);
-	    te = t;
-	    *t = '\0';
-/*@=boundswrite@*/
-	}
-/*@=branchstate@*/
+	flushBuffer(&t, &te, 0);
     }
 	    
     rc = 0;
 
 exit:
-    if (te > t) {
-	if (!nonewline) {
-/*@-boundswrite@*/
-	    *te++ = '\n';
-	    *te = '\0';
-/*@=boundswrite@*/
-	}
-	rpmMessage(RPMMESS_NORMAL, "%s", t);
-    }
+    flushBuffer(&t, &te, 0);
     t = _free(t);
 
     fi = rpmfiFree(fi);
@@ -357,6 +377,28 @@ void rpmDisplayQueryTags(FILE * fp)
     }
 }
 
+static int rpmgiShowMatches(QVA_t qva, rpmts ts)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+        /*@modifies qva, rpmGlobalMacroContext, h_errno, internalState @*/
+{
+    rpmgi gi = qva->qva_gi;
+    int ec = 0;
+
+    while (rpmgiNext(gi) == RPMRC_OK) {
+	Header h;
+	int rc;
+
+	h = rpmgiHeader(gi);
+	if (h == NULL)		/* XXX perhaps stricter break instead? */
+	    continue;
+	if ((rc = qva->qva_showPackage(qva, ts, h)) != 0)
+	    ec = rc;
+	if (qva->qva_source == RPMQV_DBOFFSET)
+	    break;
+    }
+    return rpmgiNumErrors(gi);
+}
+
 int rpmcliShowMatches(QVA_t qva, rpmts ts)
 {
     Header h;
@@ -366,6 +408,8 @@ int rpmcliShowMatches(QVA_t qva, rpmts ts)
 	int rc;
 	if ((rc = qva->qva_showPackage(qva, ts, h)) != 0)
 	    ec = rc;
+	if (qva->qva_source == RPMQV_DBOFFSET)
+	    break;
     }
     qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
     return ec;
@@ -391,10 +435,7 @@ static inline unsigned char nibble(char c)
 /*@-bounds@*/ /* LCL: segfault (realpath annotation?) */
 int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 {
-    const char ** av = NULL;
     int res = 0;
-    Header h;
-    int rc;
     const char * s;
     int i;
     int provides_checked = 0;
@@ -407,153 +448,26 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
     /*@-branchstate@*/
     switch (qva->qva_source) {
     case RPMQV_RPM:
-    {	int ac = 0;
-	const char * fileURL = NULL;
-	rpmRC rpmrc;
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
-	rc = rpmGlob(arg, &ac, &av);
-	if (rc) return 1;
+    case RPMQV_ALL:
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
-restart:
-	for (i = 0; i < ac; i++) {
-	    FD_t fd;
+    case RPMQV_HDLIST:
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
-	    fileURL = _free(fileURL);
-	    fileURL = av[i];
-	    av[i] = NULL;
-
-	    /* Try to read the header from a package file. */
-	    fd = Fopen(fileURL, "r.ufdio");
-	    if (fd == NULL || Ferror(fd)) {
-		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), fileURL,
-			Fstrerror(fd));
-		if (fd != NULL) (void) Fclose(fd);
-		res = 1;
-		/*@loopbreak@*/ break;
-	    }
-
-    	    rpmrc = rpmReadPackageFile(ts, fd, fileURL, &h);
-
-	    (void) Fclose(fd);
-
-	    res = 0;
-	    switch (rpmrc) {
-	    default:
-#ifdef	DYING
-		rpmError(RPMERR_QUERY, _("query of %s failed\n"), fileURL);
-#endif
-		res = 1;
-		/*@switchbreak@*/ break;
-	    case RPMRC_NOTTRUSTED:
-	    case RPMRC_NOKEY:
-	    case RPMRC_OK:
-		if (h == NULL) {
-#ifdef	DYING
-		    rpmError(RPMERR_QUERY,
-			_("old format source packages cannot be queried\n"));
-#endif
-		    res = 1;
-		    /*@switchbreak@*/ break;
-		}
-
-		/* Query a package file. */
-		res = qva->qva_showPackage(qva, ts, h);
-		h = headerFree(h);
-		rpmtsEmpty(ts);
-		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
-	    case RPMRC_NOTFOUND:
-		res = 0;
-		/*@switchbreak@*/ break;
-	    }
-	    if (res)
-		/*@loopbreak@*/ break;
-
-	    /* Try to read a package manifest. */
-	    fd = Fopen(fileURL, "r.fpio");
-	    if (fd == NULL || Ferror(fd)) {
-		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), fileURL,
-			Fstrerror(fd));
-		if (fd != NULL) (void) Fclose(fd);
-		res = 1;
-		/*@loopbreak@*/ break;
-	    }
-	    
-	    /* Read list of packages from manifest. */
-/*@-nullstate@*/ /* FIX: *av may be NULL */
-	    res = rpmReadPackageManifest(fd, &ac, &av);
-/*@=nullstate@*/
-	    if (res != RPMRC_OK) {
-		rpmError(RPMERR_MANIFEST,
-			_("%s: not an rpm package (or package manifest): %s\n"),
-			fileURL, Fstrerror(fd));
-		res = 1;
-	    }
-	    (void) Fclose(fd);
-
-	    /* If successful, restart the query loop. */
-	    if (res == 0)
-		goto restart;
-
-	    /*@loopbreak@*/ break;
-	}
-
-	fileURL = _free(fileURL);
-	if (av) {
-	    for (i = 0; i < ac; i++)
-		av[i] = _free(av[i]);
-	    av = _free(av);
-	}
-    }	break;
+    case RPMQV_FTSWALK:
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
     case RPMQV_SPECFILE:
 	res = ((qva->qva_specQuery != NULL)
 		? qva->qva_specQuery(ts, qva, arg) : 1);
 	break;
 
-    case RPMQV_ALL:
-	qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, NULL, 0);
-	if (qva->qva_mi == NULL) {
-	    rpmError(RPMERR_QUERYINFO, _("no packages\n"));
-	    res = 1;
-	} else {
-	    if (arg != NULL)
-	    for (av = (const char **) arg; *av != NULL; av++) {
-		int tag = RPMTAG_NAME;
-		const char * pat;
-		char * a, * ae;
-
-		pat = a = xstrdup(*av);
-		tag = RPMTAG_NAME;
-
-		/* Parse for "tag=pattern" args. */
-		if ((ae = strchr(a, '=')) != NULL) {
-		    *ae++ = '\0';
-		    tag = tagValue(a);
-		    if (tag < 0) {
-			rpmError(RPMERR_QUERYINFO,
-				_("unknown tag: \"%s\"\n"), a);
-			res = 1;
-		    }
-		    pat = ae;
-		}
-
-		if (!res)
-		    res = rpmdbSetIteratorRE(qva->qva_mi, tag, RPMMIRE_DEFAULT, pat);
-		a = _free(a);
-
-		if (res == 0)
-		    continue;
-
-		qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
-		res = 1;
-		/*@loopbreak@*/ break;
-	    }
-	    if (!res)
-		res = rpmcliShowMatches(qva, ts);
-	}
-	break;
-
     case RPMQV_GROUP:
 	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_GROUP, arg, 0);
 	if (qva->qva_mi == NULL) {
@@ -690,7 +604,6 @@ restart:
 	/*@fallthrough@*/
     case RPMQV_PATH:
     {   char * fn;
-	int myerrno = 0;
 
 	for (s = arg; *s != '\0'; s++)
 	    if (!(*s == '.' || *s == '/'))
@@ -699,28 +612,26 @@ restart:
 	if (*s == '\0') {
 	    char fnbuf[PATH_MAX];
 	    fn = realpath(arg, fnbuf);
-	    if (fn)
-		fn = xstrdup(fn);
-	    else
-		fn = xstrdup(arg);
+	    fn = xstrdup( (fn != NULL ? fn : arg) );
+	} else if (*arg != '/') {
+	    const char *curDir = currentDirectory();
+	    fn = (char *) rpmGetPath(curDir, "/", arg, NULL);
+	    curDir = _free(curDir);
 	} else
 	    fn = xstrdup(arg);
 	(void) rpmCleanPath(fn);
 
 	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, fn, 0);
-	if (qva->qva_mi == NULL) {
-	    if (access(fn, F_OK) != 0)
-		myerrno = errno;
-	    else if (!provides_checked)
-		qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, fn, 0);
-	}
+	if (qva->qva_mi == NULL && !provides_checked)
+	    qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, fn, 0);
 
-	if (myerrno != 0) {
-	    rpmError(RPMERR_QUERY, _("file %s: %s\n"), fn, strerror(myerrno));
-	    res = 1;
-	} else if (qva->qva_mi == NULL) {
-	    rpmError(RPMERR_QUERYINFO,
-		_("file %s is not owned by any package\n"), fn);
+	if (qva->qva_mi == NULL) {
+	    struct stat sb;
+	    if (lstat(fn, &sb) != 0)
+		rpmlog(RPMLOG_ERR, _("file %s: %s\n"), fn, strerror(errno));
+	    else
+		rpmlog(RPMLOG_NOTICE,
+			_("file %s is not owned by any package\n"), fn);
 	    res = 1;
 	} else
 	    res = rpmcliShowMatches(qva, ts);
@@ -760,24 +671,117 @@ restart:
     }	break;
 
     case RPMQV_PACKAGE:
-	/* XXX HACK to get rpmdbFindByLabel out of the API */
-	qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
-	if (qva->qva_mi == NULL) {
+    {
+	int matches = 0;
+	rpmdbMatchIterator mi;
+	mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
+	while (rpmdbNextIterator(mi) != NULL) {
+	    matches++;
+	}
+	rpmdbFreeIterator(mi);
+	if (! matches) {
 	    rpmError(RPMERR_QUERYINFO, _("package %s is not installed\n"), arg);
 	    res = 1;
-	} else
+	} else {
+	    qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
 	    res = rpmcliShowMatches(qva, ts);
+	}
 	break;
     }
+    
+    }
     /*@=branchstate@*/
    
     return res;
 }
 /*@=bounds@*/
 
+int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
+{
+    rpmRC rpmrc = RPMRC_NOTFOUND;
+    int ec = 0;
+
+    switch (qva->qva_source) {
+    case RPMQV_ALL:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_PACKAGES, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, RPMGI_NONE);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, (const char *) argv);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    case RPMQV_RPM:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, NULL);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    case RPMQV_HDLIST:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_HDLIST, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, NULL);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    case RPMQV_FTSWALK:
+	if (ftsOpts == 0)
+	    ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_FTSWALK, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, NULL);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    default:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts,
+		(giFlags | (RPMGI_NOGLOB|RPMGI_NOHEADER)));
+	while (rpmgiNext(qva->qva_gi) == RPMRC_OK) {
+	    ec += rpmQueryVerify(qva, ts, rpmgiHdrPath(qva->qva_gi));
+	    rpmtsEmpty(ts);
+	}
+	break;
+    }
+
+    qva->qva_gi = rpmgiFree(qva->qva_gi);
+
+    return ec;
+}
+
 int rpmcliQuery(rpmts ts, QVA_t qva, const char ** argv)
 {
-    const char * arg;
     rpmVSFlags vsflags, ovsflags;
     int ec = 0;
 
@@ -802,19 +806,7 @@ int rpmcliQuery(rpmts ts, QVA_t qva, const char ** argv)
 	vsflags |= RPMVSF_NOHDRCHK;
 
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
-    if (qva->qva_source == RPMQV_ALL) {
-	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
-	ec = rpmQueryVerify(qva, ts, (const char *) argv);
-	/*@=nullpass@*/
-    } else {
-/*@-boundsread@*/
-	if (argv != NULL)
-	while ((arg = *argv++) != NULL) {
-	    ec += rpmQueryVerify(qva, ts, arg);
-	    rpmtsEmpty(ts);
-	}
-/*@=boundsread@*/
-    }
+    ec = rpmcliArgIter(ts, qva, argv);
     vsflags = rpmtsSetVSFlags(ts, ovsflags);
 
     if (qva->qva_showPackage == showQueryPackage)
