@@ -16,12 +16,14 @@
 #define	_RPMFI_INTERNAL
 #include "rpmfi.h"
 
+#include "rpmsx.h"
+
 #define	_RPMTE_INTERNAL	/* relocations */
 #include "rpmte.h"
 #include "rpmts.h"
 
+#include "legacy.h"     /* XXX domd5 */
 #include "misc.h"	/* XXX stripTrailingChar */
-#include "legacy.h"	/* XXX domd5 */
 #include "rpmmacro.h"	/* XXX rpmCleanPath */
 
 #include "debug.h"
@@ -306,6 +308,19 @@ const char * rpmfiFClass(rpmfi fi)
     return fclass;
 }
 
+const char * rpmfiFContext(rpmfi fi)
+{
+    const char * fcontext = NULL;
+
+    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+/*@-boundsread@*/
+	if (fi->fcontexts != NULL)
+	    fcontext = fi->fcontexts[fi->i];
+/*@=boundsread@*/
+    }
+    return fcontext;
+}
+
 int_32 rpmfiFDepends(rpmfi fi, const int_32 ** fddictp)
 {
     int fddictx = -1;
@@ -563,7 +578,7 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
     if (newWhat == XDIR)
 	return FA_CREATE;
 
-    if (diskWhat != newWhat)
+    if (diskWhat != newWhat && dbWhat != REG && dbWhat != LINK)
 	return save;
     else if (newWhat != dbWhat && diskWhat != dbWhat)
 	return save;
@@ -576,8 +591,17 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
      * This order matters - we'd prefer to CREATE the file if at all
      * possible in case something else (like the timestamp) has changed.
      */
+    memset(buffer, 0, sizeof(buffer));
     if (dbWhat == REG) {
 	const unsigned char * omd5, * nmd5;
+	omd5 = rpmfiMD5(ofi);
+	if (diskWhat == REG) {
+	    if (domd5(fn, (unsigned char *)buffer, 0, NULL))
+		return FA_CREATE;       /* assume file has been removed */
+	    if (omd5 && !memcmp(omd5, buffer, 16))
+		return FA_CREATE;       /* unmodified config file, replace. */
+	}
+
 	if (domd5(fn, buffer, 0, NULL))
 	    return FA_CREATE;	/* assume file has been removed */
 	omd5 = rpmfiMD5(ofi);
@@ -590,12 +614,13 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 /*@=nullpass@*/
     } else /* dbWhat == LINK */ {
 	const char * oFLink, * nFLink;
-	memset(buffer, 0, sizeof(buffer));
+	oFLink = rpmfiFLink(ofi);
+	if (diskWhat == LINK) {
 	if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
 	    return FA_CREATE;	/* assume file has been removed */
-	oFLink = rpmfiFLink(ofi);
 	if (oFLink && !strcmp(oFLink, buffer))
 	    return FA_CREATE;	/* unmodified config file, replace. */
+	}
 	nFLink = rpmfiFLink(nfi);
 /*@-nullpass@*/
 	if (oFLink && nFLink && !strcmp(oFLink, nFLink))
@@ -613,6 +638,49 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 }
 /*@=boundsread@*/
 
+/*@-boundsread@*/
+int rpmfiConfigConflict(const rpmfi fi)
+{
+    const char * fn = rpmfiFN(fi);
+    int flags = rpmfiFFlags(fi);
+    char buffer[1024];
+    fileTypes newWhat, diskWhat;
+    struct stat sb;
+
+    if (!(flags & RPMFILE_CONFIG) || lstat(fn, &sb)) {
+	return 0;
+    }
+
+    diskWhat = whatis((int_16)sb.st_mode);
+    newWhat = whatis(rpmfiFMode(fi));
+
+    if (newWhat != LINK && newWhat != REG)
+	return 1;
+
+    if (diskWhat != newWhat)
+	return 1;
+    
+    memset(buffer, 0, sizeof(buffer));
+    if (newWhat == REG) {
+	const unsigned char * nmd5;
+	if (domd5(fn, (unsigned char *)buffer, 0, NULL))
+	    return 0;	/* assume file has been removed */
+	nmd5 = rpmfiMD5(fi);
+	if (nmd5 && !memcmp(nmd5, buffer, 16))
+	    return 0;	/* unmodified config file */
+    } else /* newWhat == LINK */ {
+	const char * nFLink;
+	if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
+	    return 0;	/* assume file has been removed */
+	nFLink = rpmfiFLink(fi);
+	if (nFLink && !strcmp(nFLink, buffer))
+	    return 0;	/* unmodified config file */
+    }
+
+    return 1;
+}
+/*@=boundsread@*/
+
 /*@observer@*/
 const char *const rpmfiTypeString(rpmfi fi)
 {
@@ -639,7 +707,8 @@ const char *const rpmfiTypeString(rpmfi fi)
 static
 Header relocateFileList(const rpmts ts, rpmfi fi,
 		Header origH, fileAction * actions)
-	/*@modifies ts, fi, origH, actions @*/
+	/*@globals h_errno, rpmGlobalMacroContext @*/
+	/*@modifies ts, fi, origH, actions, rpmGlobalMacroContext @*/
 {
     rpmte p = rpmtsRelocateElement(ts);
     HGE_t hge = fi->hge;
@@ -659,6 +728,7 @@ Header relocateFileList(const rpmts ts, rpmfi fi,
     int_32 * newDirIndexes;
     int_32 fileCount;
     int_32 dirCount;
+    uint_32 mydColor = rpmExpandNumeric("%{?_autorelocate_dcolor}");
     uint_32 * fFlags = NULL;
     uint_32 * fColors = NULL;
     uint_32 * dColors = NULL;
@@ -745,7 +815,7 @@ assert(p != NULL);
 	    if (j == numValid && !allowBadRelocate && actions) {
 		rpmps ps = rpmtsProblems(ts);
 		rpmpsAppend(ps, RPMPROB_BADRELOCATE,
-			rpmteNEVR(p), rpmteKey(p),
+			rpmteNEVRA(p), rpmteKey(p),
 			relocations[i].oldPath, NULL, NULL, 0);
 		ps = rpmpsFree(ps);
 	    }
@@ -866,7 +936,14 @@ assert(fn != NULL);		/* XXX can't happen */
 	*fn = '\0';
 	fnlen = stpcpy( stpcpy(fn, dirNames[dirIndexes[i]]), baseNames[i]) - fn;
 
-dColors[dirIndexes[i]] |= fColors[i];
+if (fColors != NULL) {
+/* XXX pkgs may not have unique dirNames, so color all dirNames that match. */
+for (j = 0; j < dirCount; j++) {
+if (strcmp(dirNames[dirIndexes[i]], dirNames[j])) /*@innercontinue@*/ continue;
+dColors[j] |= fColors[i];
+}
+}
+
 	/*
 	 * See if this file path needs relocating.
 	 */
@@ -982,8 +1059,10 @@ dColors[dirIndexes[i]] |= fColors[i];
     for (i = dirCount - 1; i >= 0; i--) {
 	for (j = numRelocations - 1; j >= 0; j--) {
 
-	    /* XXX Don't autorelocate uncolored directories. */
-if (j == p->autorelocatex && (dColors[i] == 0 || !(dColors[i] & 0x1))) continue;
+           /* XXX Don't autorelocate uncolored directories. */
+           if (j == p->autorelocatex
+            && (dColors[i] == 0 || !(dColors[i] & mydColor)))
+               /*@innercontinue@*/ continue;
 
 	    if (relocations[j].oldPath == NULL) /* XXX can't happen */
 		/*@innercontinue@*/ continue;
@@ -1083,6 +1162,12 @@ if (_rpmfi_debug < 0)
 fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
 /*@=modfilesys@*/
 
+    /* Free pre- and post-transaction script and interpreter strings. */
+    fi->pretrans = _free(fi->pretrans);
+    fi->pretransprog = _free(fi->pretransprog);
+    fi->posttrans = _free(fi->posttrans);
+    fi->posttransprog = _free(fi->posttransprog);
+
     /*@-branchstate@*/
     if (fi->fc > 0) {
 	fi->bnl = hfd(fi->bnl, -1);
@@ -1096,9 +1181,7 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
 	fi->cdict = hfd(fi->cdict, -1);
 
 	fi->fuser = hfd(fi->fuser, -1);
-	fi->fuids = _free(fi->fuids);
 	fi->fgroup = hfd(fi->fgroup, -1);
-	fi->fgids = _free(fi->fgids);
 
 	fi->fstates = _free(fi->fstates);
 
@@ -1133,6 +1216,8 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
     fi->obnl = hfd(fi->obnl, -1);
     fi->odnl = hfd(fi->odnl, -1);
 
+    fi->fcontexts = hfd(fi->fcontexts, -1);
+
     fi->actions = _free(fi->actions);
     fi->replacedSizes = _free(fi->replacedSizes);
     fi->replaced = _free(fi->replaced);
@@ -1170,7 +1255,12 @@ static inline unsigned char nibble(char c)
 	(_fi)->_data = memcpy(xmalloc((_fi)->fc * sizeof(*(_fi)->_data)), \
 			(_fi)->_data, (_fi)->fc * sizeof(*(_fi)->_data))
 
-rpmfi rpmfiNew(rpmts ts, Header h, rpmTag tagN, int scareMem)
+/* XXX Ick, not SEF. */
+#define _fdupestring(_h, _tag, _data) \
+    if (hge((_h), (_tag), NULL, (void **) &(_data), NULL)) \
+	_data = xstrdup(_data)
+
+rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int scareMem)
 {
     HGE_t hge =
 	(scareMem ? (HGE_t) headerGetEntryMinMemory : (HGE_t) headerGetEntry);
@@ -1217,6 +1307,12 @@ rpmfi rpmfiNew(rpmts ts, Header h, rpmTag tagN, int scareMem)
     fi->archivePos = 0;
     fi->archiveSize = (xx ? *uip : 0);
 
+    /* Extract pre- and post-transaction script and interpreter strings. */
+    _fdupestring(h, RPMTAG_PRETRANS, fi->pretrans);
+    _fdupestring(h, RPMTAG_PRETRANSPROG, fi->pretransprog);
+    _fdupestring(h, RPMTAG_POSTTRANS, fi->posttrans);
+    _fdupestring(h, RPMTAG_POSTTRANSPROG, fi->posttransprog);
+
     if (!hge(h, RPMTAG_BASENAMES, NULL, (void **) &fi->bnl, &fi->fc)) {
 	fi->fc = 0;
 	fi->dc = 0;
@@ -1285,17 +1381,16 @@ if (fi->actions == NULL)
 	fi->fmd5s = hfd(fi->fmd5s, -1);
     }
 
-    /* XXX TR_REMOVED doesn;t need fmtimes, frdevs or finodes */
+    /* XXX TR_REMOVED doesn;t need fmtimes, frdevs, finodes, or fcontexts */
     xx = hge(h, RPMTAG_FILEMTIMES, NULL, (void **) &fi->fmtimes, NULL);
     xx = hge(h, RPMTAG_FILERDEVS, NULL, (void **) &fi->frdevs, NULL);
     xx = hge(h, RPMTAG_FILEINODES, NULL, (void **) &fi->finodes, NULL);
+    xx = hge(h, RPMTAG_FILECONTEXTS, NULL, (void **) &fi->fcontexts, NULL);
 
     fi->replacedSizes = xcalloc(fi->fc, sizeof(*fi->replacedSizes));
 
     xx = hge(h, RPMTAG_FILEUSERNAME, NULL, (void **) &fi->fuser, NULL);
-    fi->fuids = NULL;
     xx = hge(h, RPMTAG_FILEGROUPNAME, NULL, (void **) &fi->fgroup, NULL);
-    fi->fgids = NULL;
 
     if (ts != NULL)
     if (fi != NULL)
@@ -1314,26 +1409,28 @@ if (fi->actions == NULL)
 
 #if __ia64__
 	/* XXX On ia64, change leading /emul/ix86 -> /emul/ia32, ick. */
-	if (newPath != NULL && *newPath != '\0'
+ 	if (newPath != NULL && *newPath != '\0'
 	 && strlen(newPath) >= (sizeof("/emul/i386")-1)
 	 && newPath[0] == '/' && newPath[1] == 'e' && newPath[2] == 'm'
 	 && newPath[3] == 'u' && newPath[4] == 'l' && newPath[5] == '/'
 	 && newPath[6] == 'i' && newPath[8] == '8' && newPath[9] == '6')
-	{
+ 	{
 	    newPath[7] = 'a';
 	    newPath[8] = '3';
 	    newPath[9] = '2';
 	}
 #endif
-
+ 
 	/* XXX Make sure autoreloc is not already specified. */
 	i = p->nrelocs;
 	if (newPath != NULL && *newPath != '\0' && p->relocs != NULL)
 	for (i = 0; i < p->nrelocs; i++) {
+/*@-nullpass@*/ /* XXX {old,new}Path might be NULL */
 	   if (strcmp(p->relocs[i].oldPath, "/"))
 		continue;
 	   if (strcmp(p->relocs[i].newPath, newPath))
 		continue;
+/*@=nullpass@*/
 	   break;
 	}
 
@@ -1341,6 +1438,7 @@ if (fi->actions == NULL)
 	if (newPath != NULL && *newPath != '\0' && i == p->nrelocs
 	 && p->archScore == 0)
 	{
+
 	    p->relocs =
 		xrealloc(p->relocs, (p->nrelocs + 2) * sizeof(*p->relocs));
 	    p->relocs[p->nrelocs].oldPath = xstrdup("/");
@@ -1416,7 +1514,7 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, Type, (fi ? fi->fc : 0));
 void rpmfiBuildFClasses(Header h,
 	/*@out@*/ const char *** fclassp, /*@out@*/ int * fcp)
 {
-    int scareMem = 1;
+    int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
     const char * FClass;
     const char ** av;
@@ -1454,7 +1552,7 @@ void rpmfiBuildFClasses(Header h,
 	    t = stpcpy(t, FClass);
 	*t++ = '\0';
     }
-    av[ac] = NULL;
+    av[ac] = NULL;	/* XXX tag arrays are not NULL terminated. */
     /*@=branchstate@*/
 
 exit:
@@ -1468,10 +1566,213 @@ exit:
     if (fcp) *fcp = ac;
 }
 
+void rpmfiBuildFContexts(Header h,
+	/*@out@*/ const char *** fcontextp, /*@out@*/ int * fcp)
+{
+    int scareMem = 0;
+    rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
+    const char * fcontext;
+    const char ** av;
+    int ac;
+    size_t nb;
+    char * t;
+
+    if ((ac = rpmfiFC(fi)) <= 0) {
+	av = NULL;
+	ac = 0;
+	goto exit;
+    }
+
+    /* Compute size of argv array blob. */
+    nb = (ac + 1) * sizeof(*av);
+    fi = rpmfiInit(fi, 0);
+    if (fi != NULL)
+    while (rpmfiNext(fi) >= 0) {
+	fcontext = rpmfiFContext(fi);
+	if (fcontext && *fcontext != '\0')
+	    nb += strlen(fcontext);
+	nb += 1;
+    }
+
+    /* Create and load argv array. */
+    av = xmalloc(nb);
+    t = ((char *) av) + ((ac + 1) * sizeof(*av));
+    ac = 0;
+    fi = rpmfiInit(fi, 0);
+    if (fi != NULL)
+    while (rpmfiNext(fi) >= 0) {
+	fcontext = rpmfiFContext(fi);
+	av[ac++] = t;
+	if (fcontext && *fcontext != '\0')
+	    t = stpcpy(t, fcontext);
+	*t++ = '\0';
+    }
+    av[ac] = NULL;	/* XXX tag arrays are not NULL terminated. */
+    /*@=branchstate@*/
+
+exit:
+    fi = rpmfiFree(fi);
+    /*@-branchstate@*/
+    if (fcontextp)
+	*fcontextp = av;
+    else
+	av = _free(av);
+    /*@=branchstate@*/
+    if (fcp) *fcp = ac;
+}
+
+void rpmfiBuildFSContexts(Header h,
+	/*@out@*/ const char *** fcontextp, /*@out@*/ int * fcp)
+{
+    int scareMem = 0;
+    rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
+    const char ** av;
+    int ac;
+    size_t nb;
+    char * t;
+    char * fctxt = NULL;
+    size_t fctxtlen = 0;
+    int * fcnb;
+
+    if ((ac = rpmfiFC(fi)) <= 0) {
+	av = NULL;
+	ac = 0;
+	goto exit;
+    }
+
+    /* Compute size of argv array blob, concatenating file contexts. */
+    nb = ac * sizeof(*fcnb);
+    fcnb = memset(alloca(nb), 0, nb);
+    ac = 0;
+    fi = rpmfiInit(fi, 0);
+    if (fi != NULL)
+    while (rpmfiNext(fi) >= 0) {
+	const char * fn = rpmfiFN(fi);
+	security_context_t scon;
+
+	fcnb[ac] = lgetfilecon(fn, &scon);
+/*@-branchstate@*/
+	if (fcnb[ac] > 0) {
+	    fctxt = xrealloc(fctxt, fctxtlen + fcnb[ac]);
+	    memcpy(fctxt+fctxtlen, scon, fcnb[ac]);
+	    fctxtlen += fcnb[ac];
+	    freecon(scon);
+	}
+/*@=branchstate@*/
+	ac++;
+    }
+
+    /* Create and load argv array from concatenated file contexts. */
+    nb = (ac + 1) * sizeof(*av) + fctxtlen;
+    av = xmalloc(nb);
+    t = ((char *) av) + ((ac + 1) * sizeof(*av));
+    if (fctxt != NULL && fctxtlen > 0)
+	(void) memcpy(t, fctxt, fctxtlen);
+    ac = 0;
+    fi = rpmfiInit(fi, 0);
+    if (fi != NULL)
+    while (rpmfiNext(fi) >= 0) {
+	av[ac] = "";
+	if (fcnb[ac] > 0) {
+	    av[ac] = t;
+	    t += fcnb[ac];
+	}
+	ac++;
+    }
+    av[ac] = NULL;	/* XXX tag arrays are not NULL terminated. */
+
+exit:
+    fi = rpmfiFree(fi);
+    /*@-branchstate@*/
+    if (fcontextp)
+	*fcontextp = av;
+    else
+	av = _free(av);
+    /*@=branchstate@*/
+    if (fcp) *fcp = ac;
+}
+
+void rpmfiBuildREContexts(Header h,
+	/*@out@*/ const char *** fcontextp, /*@out@*/ int * fcp)
+{
+    int scareMem = 0;
+    rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
+    rpmsx sx = NULL;
+    const char ** av = NULL;
+    int ac;
+    size_t nb;
+    char * t;
+    char * fctxt = NULL;
+    size_t fctxtlen = 0;
+    int * fcnb;
+
+    if ((ac = rpmfiFC(fi)) <= 0) {
+	ac = 0;
+	goto exit;
+    }
+
+    /* Read security context patterns. */
+    sx = rpmsxNew(NULL);
+
+    /* Compute size of argv array blob, concatenating file contexts. */
+    nb = ac * sizeof(*fcnb);
+    fcnb = memset(alloca(nb), 0, nb);
+    ac = 0;
+    fi = rpmfiInit(fi, 0);
+    if (fi != NULL)
+    while (rpmfiNext(fi) >= 0) {
+	const char * fn = rpmfiFN(fi);
+	mode_t fmode = rpmfiFMode(fi);
+	const char * scon;
+
+	scon = rpmsxFContext(sx, fn, fmode);
+	if (scon != NULL) {
+	    fcnb[ac] = strlen(scon) + 1;
+/*@-branchstate@*/
+	    if (fcnb[ac] > 0) {
+		fctxt = xrealloc(fctxt, fctxtlen + fcnb[ac]);
+		memcpy(fctxt+fctxtlen, scon, fcnb[ac]);
+		fctxtlen += fcnb[ac];
+	    }
+/*@=branchstate@*/
+	}
+	ac++;
+    }
+
+    /* Create and load argv array from concatenated file contexts. */
+    nb = (ac + 1) * sizeof(*av) + fctxtlen;
+    av = xmalloc(nb);
+    t = ((char *) av) + ((ac + 1) * sizeof(*av));
+    (void) memcpy(t, fctxt, fctxtlen);
+    ac = 0;
+    fi = rpmfiInit(fi, 0);
+    if (fi != NULL)
+    while (rpmfiNext(fi) >= 0) {
+	av[ac] = "";
+	if (fcnb[ac] > 0) {
+	    av[ac] = t;
+	    t += fcnb[ac];
+	}
+	ac++;
+    }
+    av[ac] = NULL;	/* XXX tag arrays are not NULL terminated. */
+
+exit:
+    fi = rpmfiFree(fi);
+    sx = rpmsxFree(sx);
+    /*@-branchstate@*/
+    if (fcontextp)
+	*fcontextp = av;
+    else
+	av = _free(av);
+    /*@=branchstate@*/
+    if (fcp) *fcp = ac;
+}
+
 void rpmfiBuildFDeps(Header h, rpmTag tagN,
 	/*@out@*/ const char *** fdepsp, /*@out@*/ int * fcp)
 {
-    int scareMem = 1;
+    int scareMem = 0;
     rpmfi fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, scareMem);
     rpmds ds = NULL;
     const char ** av;
@@ -1553,7 +1854,7 @@ void rpmfiBuildFDeps(Header h, rpmTag tagN,
 	*t++ = '\0';
     }
     /*@=branchstate@*/
-    av[ac] = NULL;
+    av[ac] = NULL;	/* XXX tag arrays are not NULL terminated. */
 
 exit:
     fi = rpmfiFree(fi);
