@@ -70,10 +70,6 @@ struct rpmfi_s {
     const char ** fuser;	/*!< File owner(s) (from header) */
 /*@only@*/ /*@null@*/
     const char ** fgroup;	/*!< File group(s) (from header) */
-/*@only@*/ /*@null@*/
-    uid_t * fuids;		/*!< File uid(s) */
-/*@only@*/ /*@null@*/
-    gid_t * fgids;		/*!< File gid(s) */
 
 /*@only@*/ /*@null@*/
     char * fstates;		/*!< File state(s) (from header) */
@@ -81,6 +77,9 @@ struct rpmfi_s {
 /*@only@*/ /*@null@*/
     const uint_32 * fcolors;	/*!< File color bits (header) */
 
+/*@only@*/ /*@null@*/
+    const char ** fcontexts;	/*! FIle security contexts. */
+
 /*@only@*/ /*@null@*/
     const char ** cdict;	/*!< File class dictionary (header) */
     int_32 ncdict;		/*!< No. of class entries. */
@@ -129,6 +128,15 @@ struct rpmfi_s {
 /*@only@*/ /*@relnull@*/
     unsigned char * md5s;	/*!< File md5 sums in binary. */
 
+/*@only@*/ /*@null@*/
+    const char * pretrans;
+/*@only@*/ /*@null@*/
+    const char * pretransprog;
+/*@only@*/ /*@null@*/
+    const char * posttrans;
+/*@only@*/ /*@null@*/
+    const char * posttransprog;
+
 /*@only@*/ /*@null@*/
     char * fn;			/*!< File name buffer. */
     int fnlen;			/*!< FIle name buffer length. */
@@ -395,6 +403,17 @@ extern const char * rpmfiFClass(/*@null@*/ rpmfi fi)
 	/*@*/;
 /*@=exportlocal@*/
 
+/**
+ * Return current file security context from file info set.
+ * @param fi		file info set
+ * @return		current file context, 0 on invalid
+ */
+/*@-exportlocal@*/
+/*@observer@*/ /*@null@*/
+extern const char * rpmfiFContext(/*@null@*/ rpmfi fi)
+	/*@*/;
+/*@=exportlocal@*/
+
 /**
  * Return current file depends dictionary from file info set.
  * @param fi		file info set
@@ -488,6 +507,7 @@ rpmfi rpmfiFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi)
 
 /**
  * Create and load a file info set.
+ * @deprecated Only scareMem = 0 will be permitted.
  * @param ts		transaction set (NULL skips path relocation)
  * @param h		header
  * @param tagN		RPMTAG_BASENAMES
@@ -495,9 +515,9 @@ rpmfi rpmfiFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi)
  * @return		new file info set
  */
 /*@null@*/
-rpmfi rpmfiNew(/*@null@*/ rpmts ts, Header h, rpmTag tagN, int scareMem)
-	/*@globals fileSystem @*/
-	/*@modifies ts, h, fileSystem @*/;
+rpmfi rpmfiNew(/*@null@*/ const rpmts ts, Header h, rpmTag tagN, int scareMem)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem @*/;
 
 /**
  * Retrieve file classes from header.
@@ -510,8 +530,50 @@ rpmfi rpmfiNew(/*@null@*/ rpmts ts, Header h, rpmTag tagN, int scareMem)
  */
 void rpmfiBuildFClasses(Header h,
 		/*@out@*/ const char *** fclassp, /*@out@*/ int * fcp)
-	/*@globals fileSystem @*/
-	/*@modifies h, *fclassp, *fcp, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies h, *fclassp, *fcp, rpmGlobalMacroContext, fileSystem @*/;
+
+/**
+ * Retrieve file security contexts from header.
+ *
+ * This function is used to retrieve file contexts from the header.
+ * 
+ * @param h		header
+ * @retval *fcontextp	array of file contexts
+ * @retval *fcp		number of files
+ */
+void rpmfiBuildFContexts(Header h,
+		/*@out@*/ const char *** fcontextp, /*@out@*/ int * fcp)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies h, *fcontextp, *fcp, rpmGlobalMacroContext, fileSystem @*/;
+
+/**
+ * Retrieve file security contexts from file system.
+ *
+ * This function is used to retrieve file contexts from the file system.
+ * 
+ * @param h		header
+ * @retval *fcontextp	array of file contexts
+ * @retval *fcp		number of files
+ */
+void rpmfiBuildFSContexts(Header h,
+		/*@out@*/ const char *** fcontextp, /*@out@*/ int * fcp)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies h, *fcontextp, *fcp, rpmGlobalMacroContext, fileSystem @*/;
+
+/**
+ * Retrieve file security contexts from policy RE's.
+ *
+ * This function is used to retrieve file contexts from policy RE's.
+ * 
+ * @param h		header
+ * @retval *fcontextp	array of file contexts
+ * @retval *fcp		number of files
+ */
+void rpmfiBuildREContexts(Header h,
+		/*@out@*/ const char *** fcontextp, /*@out@*/ int * fcp)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
+	/*@modifies h, *fcontextp, *fcp, rpmGlobalMacroContext, fileSystem @*/;
 
 /**
  * Retrieve per-file dependencies from header.
@@ -525,8 +587,9 @@ void rpmfiBuildFClasses(Header h,
  */
 void rpmfiBuildFDeps(Header h, rpmTag tagN,
 		/*@out@*/ const char *** fdepsp, /*@out@*/ int * fcp)
-	/*@globals fileSystem @*/
-	/*@modifies h, *fdepsp, *fcp, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies h, *fdepsp, *fcp,
+		rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /**
  * Return file type from mode_t.
@@ -553,9 +616,17 @@ int rpmfiCompare(const rpmfi afi, const rpmfi bfi)
  * @return		file dispostion
  */
 fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies nfi, fileSystem, internalState @*/;
 
+/**
+ * Return whether file is conflicting config
+ * @param fi		file info
+ * @return		1 if config file and file on disk conflicts
+ */
+int rpmfiConfigConflict(const rpmfi fi)
+	/*@*/;
+
 /**
  * Return formatted string representation of package disposition.
  * @param fi		file info set
