@@ -9,6 +9,7 @@
 #include "rpmdb.h"
 #include "rpmds.h"
 
+#include "rpmte.h"	/* XXX: rpmts.h needs this for rpmtsScoreEntries */
 #define	_RPMTS_INTERNAL		/* ts->goal, ts->dbmode, ts->suggests */
 #include "rpmts.h"
 
@@ -307,7 +308,21 @@ int rpmInstall(rpmts ts,
     if (rpmExpandNumeric("%{?_repackage_all_erasures}"))
 	ia->transFlags |= RPMTRANS_FLAG_REPACKAGE;
 
+    /* Initialize security context patterns (if not already done). */
+    if (!(ia->transFlags & RPMTRANS_FLAG_NOCONTEXTS)) {
+	rpmsx sx = rpmtsREContext(ts);
+	if (sx == NULL) {
+	    const char *fn = rpmGetPath("%{?_install_file_context_path}", NULL);
+	    if (fn != NULL && *fn != '\0') {
+		sx = rpmsxNew(fn);
+		(void) rpmtsSetREContext(ts, sx);
+	    }
+	    fn = _free(fn);
+	}
+	sx = rpmsxFree(sx);
+    }
     (void) rpmtsSetFlags(ts, ia->transFlags);
+
     probFilter = ia->probFilter;
     relocations = ia->relocations;
 
@@ -341,8 +356,11 @@ int rpmInstall(rpmts ts,
     /*@-temptrans@*/
     for (eiu->fnp = fileArgv; *eiu->fnp != NULL; eiu->fnp++) {
     /*@=temptrans@*/
+	char * fn;
 	av = _free(av);	ac = 0;
-	rc = rpmGlob(*eiu->fnp, &ac, &av);
+	fn = rpmEscapeSpaces(*eiu->fnp);
+	rc = rpmGlob(fn, &ac, &av);
+	fn = _free(fn);
 	if (rc || ac == 0) {
 	    rpmError(RPMERR_OPEN, _("File not found by glob: %s\n"), *eiu->fnp);
 	    continue;
@@ -401,8 +419,9 @@ if (fileURL[0] == '=') {
 #endif
 
 	switch (urlIsURL(fileURL)) {
-	case URL_IS_FTP:
+	case URL_IS_HTTPS:
 	case URL_IS_HTTP:
+	case URL_IS_FTP:
 	{   const char *tfn;
 
 	    if (rpmIsVerbose())
@@ -435,6 +454,8 @@ if (fileURL[0] == '=') {
 	    eiu->pkgx++;
 	}   /*@switchbreak@*/ break;
 	case URL_IS_PATH:
+	case URL_IS_DASH:	/* WRONG WRONG WRONG */
+	case URL_IS_HKP:	/* WRONG WRONG WRONG */
 	default:
 	    eiu->pkgURL[eiu->pkgx] = fileURL;
 	    fileURL = NULL;
@@ -495,7 +516,7 @@ if (fileURL[0] == '=') {
 	eiu->isSource = headerIsEntry(eiu->h, RPMTAG_SOURCEPACKAGE);
 
 	if (eiu->isSource) {
-	    rpmMessage(RPMMESS_DEBUG, "\tadded source package [%d]\n",
+	    rpmMessage(RPMMESS_DEBUG, _("\tadded source package [%d]\n"),
 		eiu->numSRPMS);
 	    eiu->sourceURL = xrealloc(eiu->sourceURL,
 				(eiu->numSRPMS + 2) * sizeof(*eiu->sourceURL));
@@ -520,7 +541,7 @@ if (fileURL[0] == '=') {
 		const char * name;
 		xx = headerNVR(eiu->h, &name, NULL, NULL);
 		rpmMessage(RPMMESS_ERROR,
-			       _("package %s is not relocateable\n"), name);
+			       _("package %s is not relocatable\n"), name);
 		eiu->numFailed++;
 		goto exit;
 		/*@notreached@*/
@@ -565,7 +586,7 @@ if (fileURL[0] == '=') {
 
 	switch(rc) {
 	case 0:
-	    rpmMessage(RPMMESS_DEBUG, "\tadded binary package [%d]\n",
+	    rpmMessage(RPMMESS_DEBUG, _("\tadded binary package [%d]\n"),
 			eiu->numRPMS);
 	    /*@switchbreak@*/ break;
 	case 1:
@@ -581,6 +602,10 @@ if (fileURL[0] == '=') {
 	    eiu->numFailed++;
 	    goto exit;
 	    /*@notreached@*/ /*@switchbreak@*/ break;
+	default:
+	    eiu->numFailed++;
+	    goto exit;
+	    /*@notreached@*/ /*@switchbreak@*/ break;
 	}
 
 	eiu->numRPMS++;
@@ -601,7 +626,9 @@ maybe_manifest:
 	}
 
 	/* Read list of packages from manifest. */
+/*@-nullstate@*/ /* FIX: *eiu->argv can be NULL */
 	rc = rpmReadPackageManifest(eiu->fd, &eiu->argc, &eiu->argv);
+/*@=nullstate@*/
 	if (rc != RPMRC_OK)
 	    rpmError(RPMERR_MANIFEST, _("%s: not an rpm package (or package manifest): %s\n"),
 			*eiu->fnp, Fstrerror(eiu->fd));
@@ -690,6 +717,7 @@ maybe_manifest:
     if (eiu->numSRPMS && !stopInstall) {
 	if (eiu->sourceURL != NULL)
 	for (i = 0; i < eiu->numSRPMS; i++) {
+	    rpmdbCheckSignals();
 	    if (eiu->sourceURL[i] == NULL) continue;
 	    eiu->fd = Fopen(eiu->sourceURL[i], "r.ufdio");
 	    if (eiu->fd == NULL || Ferror(eiu->fd)) {
@@ -785,7 +813,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 		    rpmMessage(RPMMESS_ERROR, _("\"%s\" specifies multiple packages\n"),
 			*arg);
 		    numFailed++;
-		    break;
+		    /*@innerbreak@*/ break;
 		}
 		if (recOffset) {
 		    (void) rpmtsAddEraseElement(ts, h, recOffset);
@@ -796,6 +824,8 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	mi = rpmdbFreeIterator(mi);
     }
 
+    if (numFailed) goto exit;
+
     if (!(ia->eraseInterfaceFlags & UNINSTALL_NODEPS)) {
 
 	if (rpmtsCheck(ts)) {
@@ -813,13 +843,20 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	ps = rpmpsFree(ps);
     }
 
-    if (!stopUninstall) {
+    if (!stopUninstall && !(ia->installInterfaceFlags & INSTALL_NOORDER)) {
+	if (rpmtsOrder(ts)) {
+	    numFailed += numPackages;
+	    stopUninstall = 1;
+	}
+    }
+
+    if (numPackages && !stopUninstall) {
 	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | RPMTRANS_FLAG_REVERSE));
 
 	/* Drop added/available package indices and dependency sets. */
 	rpmtsClean(ts);
 
-	numPackages = rpmtsRun(ts, NULL, 0);
+	numPackages = rpmtsRun(ts, NULL, ia->probFilter & (RPMPROB_FILTER_DISKSPACE|RPMPROB_FILTER_DISKNODES));
 	ps = rpmtsProblems(ts);
 	if (rpmpsNumProblems(ps) > 0)
 	    rpmpsPrint(NULL, ps);
@@ -828,6 +865,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	ps = rpmpsFree(ps);
     }
 
+exit:
     rpmtsEmpty(ts);
 
     return numFailed;
@@ -1027,7 +1065,7 @@ IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag)
 
 	tidp = NULL;
 	/*@-branchstate@*/
-	if (hge(h, tag, &type, (void **) &tidp, &count) && tidp) {
+	if (hge(h, tag, &type, (void **) &tidp, &count) && tidp != NULL) {
 
 	    idtx = IDTXgrow(idtx, 1);
 	    if (idtx == NULL || idtx->idt == NULL)
@@ -1095,6 +1133,11 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 
     (void) rpmtsSetFlags(ts, transFlags);
 
+    /*  Make the transaction a rollback transaction.  In a rollback
+     *  a best effort is what we want 
+     */
+    rpmtsSetType(ts, RPMTRANS_TYPE_ROLLBACK);
+
     itids = IDTXload(ts, RPMTAG_INSTALLTID);
     if (itids != NULL) {
 	ip = itids->idt;
@@ -1250,13 +1293,16 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 	/* Clean up after successful rollback. */
 	if (rtids && !rpmIsDebug()) {
 	    int i;
+	    rpmMessage(RPMMESS_NORMAL, _("Cleaning up repackaged packages:\n"));
 	    if (rtids->idt)
 	    for (i = 0; i < rtids->nidt; i++) {
 		IDT rrp = rtids->idt + i;
 		if (rrp->val.u32 != thistid)
 		    /*@innercontinue@*/ continue;
-		if (rrp->key)	/* XXX can't happen */
-		    (void) unlink(rrp->key);
+		if (rrp->key) {	/* XXX can't happen */
+		    rpmMessage(RPMMESS_NORMAL, _("\tRemoving %s:\n"), rrp->key);
+		    (void) unlink(rrp->key);	/* XXX: Should check rc??? */
+		}
 	    }
 	}
 
