@@ -16,6 +16,7 @@
 
 #include <rpmlib.h>
 #include <rpmmacro.h>
+#include <rpmlua.h>
 
 #include "misc.h"
 #include "debug.h"
@@ -142,7 +143,7 @@ static int defaultsInitialized = 0;
 
 /* prototypes */
 static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies fd, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 static void rpmSetVarArch(int var, const char * val,
@@ -155,7 +156,7 @@ static void rebuildCompatTables(int type, const char * name)
 	/*@modifies internalState @*/;
 
 static void rpmRebuildTargetVars(/*@null@*/ const char **target, /*@null@*/ const char ** canontarget)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies *canontarget, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
@@ -463,7 +464,7 @@ static void setVarDefault(int var, const char * macroname, const char * val,
 }
 
 static void setPathDefault(int var, const char * macroname, const char * subdir)
-	/*@globals rpmGlobalMacroContext, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/
 {
 
@@ -514,7 +515,7 @@ export RPM_BUILD_ROOT\n}\
 ";
 
 static void setDefaults(void)
-	/*@globals rpmGlobalMacroContext, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
 	/*@modifies rpmGlobalMacroContext, internalState @*/
 {
 
@@ -552,7 +553,7 @@ static void setDefaults(void)
 
 /*@-usedef@*/	/*@ FIX: se usage inconsistent, W2DO? */
 static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies fd, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     const char *s;
@@ -640,7 +641,9 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		while (*se && !xisspace(*se)) se++;
 		if (*se != '\0') *se++ = '\0';
 
+#if 0 /* XXX doesn't seem to do anything useful, only break things... */
 		rpmRebuildTargetVars(NULL, NULL);
+#endif
 
 		fn = rpmGetPath(s, NULL);
 		if (fn == NULL || *fn == '\0') {
@@ -777,7 +780,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 /*@-bounds@*/
 static int rpmPlatform(const char * platform)
 	/*@globals nplatpat, platpat,
-		rpmGlobalMacroContext, fileSystem, internalState @*/
+		rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies nplatpat, platpat,
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 {
@@ -892,16 +895,13 @@ static inline void cpuid(unsigned int op, int *eax, int *ebx, int *ecx, int *edx
 #ifdef	__LCLINT__
     *eax = *ebx = *ecx = *edx = 0;
 #endif
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
-		: "=a"(*eax), "=g"(*ebx), "=&c"(*ecx), "=&d"(*edx)
-		: "a" (op));
-#else
-	__asm__("cpuid"
-		: "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx)
-		: "a" (op));
-#endif
-
+    asm volatile (
+	"pushl	%%ebx		\n"
+	"cpuid			\n"
+	"movl	%%ebx,	%%esi	\n"
+	"popl	%%ebx		\n"
+    : "=a" (*eax), "=S" (*ebx), "=c" (*ecx), "=d" (*edx)
+    : "a" (op));
 }
 
 /*
@@ -910,15 +910,8 @@ static inline void cpuid(unsigned int op, int *eax, int *ebx, int *ecx, int *edx
 static inline unsigned int cpuid_eax(unsigned int op)
 	/*@*/
 {
-	unsigned int val;
-
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-		: "=a" (val) : "a" (op) : "ecx", "edx");
-#else
-	__asm__("cpuid"
-		: "=a" (val) : "a" (op) : "ebx", "ecx", "edx");
-#endif
+	unsigned int tmp, val;
+	cpuid(op, &val, &tmp, &tmp, &tmp);
 	return val;
 }
 
@@ -926,14 +919,7 @@ static inline unsigned int cpuid_ebx(unsigned int op)
 	/*@*/
 {
 	unsigned int tmp, val;
-
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
-		: "=a" (tmp), "=g" (val) : "a" (op) : "ecx", "edx");
-#else
-	__asm__("cpuid"
-		: "=a" (tmp), "=b" (val) : "a" (op) : "ecx", "edx");
-#endif
+	cpuid(op, &tmp, &val, &tmp, &tmp);
 	return val;
 }
 
@@ -941,30 +927,16 @@ static inline unsigned int cpuid_ecx(unsigned int op)
 	/*@*/
 {
 	unsigned int tmp, val;
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-		: "=a" (tmp), "=c" (val) : "a" (op) : "edx");
-#else
-	__asm__("cpuid"
-		: "=a" (tmp), "=c" (val) : "a" (op) : "ebx", "edx");
-#endif
+	cpuid(op, &tmp, &tmp, &val, &tmp);
 	return val;
-
 }
 
 static inline unsigned int cpuid_edx(unsigned int op)
 	/*@*/
 {
 	unsigned int tmp, val;
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-		: "=a" (tmp), "=d" (val) : "a" (op) : "ecx");
-#else
-	__asm__("cpuid"
-		: "=a" (tmp), "=d" (val) : "a" (op) : "ebx", "ecx");
-#endif
+	cpuid(op, &tmp, &tmp, &tmp, &val);
 	return val;
-
 }
 
 /*@unchecked@*/
@@ -983,20 +955,28 @@ static inline int RPMClass(void)
 {
 	int cpu;
 	unsigned int tfms, junk, cap, capamd;
+	struct sigaction oldsa;
 	
+	sigaction(SIGILL, NULL, &oldsa);
 	signal(SIGILL, model3);
 	
-	if (sigsetjmp(jenv, 1))
+	if (sigsetjmp(jenv, 1)) {
+		sigaction(SIGILL, &oldsa, NULL);
 		return 3;
+	}
 		
-	if (cpuid_eax(0x000000000)==0)
+	if (cpuid_eax(0x000000000)==0) {
+		sigaction(SIGILL, &oldsa, NULL);
 		return 4;
+	}
 
 	cpuid(0x00000001, &tfms, &junk, &junk, &cap);
 	cpuid(0x80000001, &junk, &junk, &junk, &capamd);
 	
 	cpu = (tfms>>8)&15;
 	
+	sigaction(SIGILL, &oldsa, NULL);
+
 	if (cpu < 6)
 		return cpu;
 		
@@ -1037,6 +1017,99 @@ static int is_athlon(void)
 	return 1;
 }
 
+static int is_pentium3()
+{
+    unsigned int eax, ebx, ecx, edx, family, model;
+    char vendor[16];
+    cpuid(0, &eax, &ebx, &ecx, &edx);
+    memset(vendor, 0, sizeof(vendor));
+    *((unsigned int *)&vendor[0]) = ebx;
+    *((unsigned int *)&vendor[4]) = edx;
+    *((unsigned int *)&vendor[8]) = ecx;
+    if (strncmp(vendor, "GenuineIntel", 12) != 0)
+	return 0;
+    cpuid(1, &eax, &ebx, &ecx, &edx);
+    family = (eax >> 8) & 0x0f;
+    model = (eax >> 4) & 0x0f;
+    if (family == 6)
+	switch (model)
+	{
+	    case 7:	// Pentium III, Pentium III Xeon (model 7)
+	    case 8:	// Pentium III, Pentium III Xeon, Celeron (model 8)
+	    case 9:	// Pentium M
+			/*
+			    Intel recently announced its new technology for mobile platforms,
+			    named Centrino, and presents it as a big advance in mobile PCs.
+			    One of the main part of Centrino consists in a brand new CPU,
+			    the Pentium M, codenamed Banias, that we'll study in this review.
+			    A particularity of this CPU is that it was designed for mobile platform
+			    exclusively, unlike previous mobile CPU (Pentium III-M, Pentium 4-M)
+			    that share the same micro-architecture as their desktop counterparts.
+			    The Pentium M introduces a new micro-architecture, adapted for mobility
+			    constraints, and that is halfway between the Pentium III and the Pentium 4.
+						    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			*/
+	    case 10:	// Pentium III Xeon (model A)
+	    case 11:	// Pentium III (model B)
+		return 1;
+	}
+    return 0;
+}
+
+static int is_pentium4()
+{
+    unsigned int eax, ebx, ecx, edx, family, model;
+    char vendor[16];
+    cpuid(0, &eax, &ebx, &ecx, &edx);
+    memset(vendor, 0, sizeof(vendor));
+    *((unsigned int *)&vendor[0]) = ebx;
+    *((unsigned int *)&vendor[4]) = edx;
+    *((unsigned int *)&vendor[8]) = ecx;
+    if (strncmp(vendor, "GenuineIntel", 12) != 0)
+	return 0;
+    cpuid(1, &eax, &ebx, &ecx, &edx);
+    family = (eax >> 8) & 0x0f;
+    model = (eax >> 4) & 0x0f;
+    if (family == 15)
+	switch (model)
+	{
+	    case 0:	// Pentium 4, Pentium 4 Xeon                 (0.18um)
+	    case 1:	// Pentium 4, Pentium 4 Xeon MP, Celeron     (0.18um)
+	    case 2:	// Pentium 4, Mobile Pentium 4-M,
+			// Pentium 4 Xeon, Pentium 4 Xeon MP,
+			// Celeron, Mobile Celron                    (0.13um)
+	    case 3:	// Pentium 4, Celeron                        (0.09um)
+		return 1;
+	}
+    return 0;
+}
+
+static int is_geode()
+{
+    unsigned int eax, ebx, ecx, edx, family, model;
+    char vendor[16];
+    /* If you care about space, you can just check ebx, ecx and edx directly
+       instead of forming a string first and then doing a strcmp */
+    memset(vendor, 0, sizeof(vendor));
+    
+    cpuid(0, &eax, &ebx, &ecx, &edx);
+    memset(vendor, 0, sizeof(vendor));
+    *((unsigned int *)&vendor[0]) = ebx;
+    *((unsigned int *)&vendor[4]) = edx;
+    *((unsigned int *)&vendor[8]) = ecx;
+    if (strncmp(vendor, "AuthenticAMD", 12) != 0)  
+        return 0;
+    cpuid(1, &eax, &ebx, &ecx, &edx);
+    family = (eax >> 8) & 0x0f;
+    model = (eax >> 4) & 0x0f;
+    if (family == 5)
+	switch (model)
+	{
+            case 10: // Geode 
+                return 1;
+        }
+    return 0;
+}
 #endif
 
 #if defined(__linux__) && defined(__powerpc__)
@@ -1052,7 +1125,7 @@ static void mfspr_ill(int notused)
  */
 static void defaultMachine(/*@out@*/ const char ** arch,
 		/*@out@*/ const char ** os)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies *arch, *os, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     static struct utsname un;
@@ -1094,6 +1167,13 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 	    strcpy(un.machine, __power_pc() ? "ppc" : "rs6000");
 	    sprintf(un.sysname,"aix%s.%s", un.version, un.release);
 	}
+	else if(!strcmp(un.sysname, "Darwin")) { 
+#ifdef __ppc__
+	    strcpy(un.machine, "ppc");
+#else ifdef __i386__
+	    strcpy(un.machine, "i386");
+#endif 
+	}
 	else if (!strcmp(un.sysname, "SunOS")) {
 	    if (!strncmp(un.release,"4", 1)) /* SunOS 4.x */ {
 		int fd;
@@ -1273,32 +1353,17 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 
 	    if ((class == '6' && is_athlon()) || class == '7')
 	    	strcpy(un.machine, "athlon");
+	    else if (is_pentium4())
+		strcpy(un.machine, "pentium4");
+	    else if (is_pentium3())
+		strcpy(un.machine, "pentium3");
+	    else if (is_geode())
+		strcpy(un.machine, "geode");
 	    else if (strchr("3456", un.machine[1]) && un.machine[1] != class)
 		un.machine[1] = class;
 	}
 #	endif
 
-#	if defined(__linux__) && defined(__powerpc__)
-	{
-	    unsigned pvr = 0;
-	    __sighandler_t oldh = signal(SIGILL, mfspr_ill);
-	    if (setjmp(mfspr_jmpbuf) == 0) {
-		__asm__ __volatile__ ("mfspr %0, 287" : "=r" (pvr));
-	    }
-	    signal(SIGILL, oldh);
-
-	    if ( pvr ) {
-		pvr >>= 16;
-		if ( pvr >= 0x40)
-		    strcpy(un.machine, "ppcpseries");
-		else if ( (pvr == 0x36) || (pvr == 0x37) )
-		    strcpy(un.machine, "ppciseries");
-		else
-		    strcpy(un.machine, "ppc");
-	    }
-	}
-#	endif
-
 	/* the uname() result goes through the arch_canon table */
 	canon = lookupInCanonTable(un.machine,
 				   tables[RPM_MACHTABLE_INSTARCH].canons,
@@ -1405,7 +1470,8 @@ static void rpmSetVarArch(int var, const char * val, const char * arch)
 	}
     }
 
-    next->value = xstrdup(val);		/* XXX memory leak, hard to plug */
+    next->value = _free(next->value);
+    next->value = xstrdup(val);
     next->arch = (arch ? xstrdup(arch) : NULL);
 }
 
@@ -1525,7 +1591,7 @@ static void getMachineInfo(int type, /*@null@*/ /*@out@*/ const char ** name,
 
 	if (tables[currTables[type]].hasCanon) {
 	    rpmMessage(RPMMESS_WARNING, _("Unknown system: %s\n"), current[type]);
-	    rpmMessage(RPMMESS_WARNING, _("Please contact rpm-list@redhat.com\n"));
+	    rpmMessage(RPMMESS_WARNING, _("Please contact rpm-maint@lists.rpm.org\n"));
 	}
     }
 }
@@ -1723,7 +1789,7 @@ void rpmFreeRpmrc(void)
  */
 static int rpmReadRC(/*@null@*/ const char * rcfiles)
 	/*@globals defaultsInitialized, rpmGlobalMacroContext,
-		rpmCLIMacroContext, fileSystem, internalState @*/
+		rpmCLIMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies defaultsInitialized, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
 {
@@ -1814,6 +1880,9 @@ static int rpmReadRC(/*@null@*/ const char * rcfiles)
 
 int rpmReadConfigFiles(const char * file, const char * target)
 {
+    mode_t mode = 0022;
+    /* Reset umask to its default umask(2) value. */
+    mode = umask(mode);
 
     /* Preset target macros */
     /*@-nullstate@*/	/* FIX: target can be NULL */
@@ -1834,6 +1903,11 @@ int rpmReadConfigFiles(const char * file, const char * target)
 	os = _free(os);
     }
 
+    /* Force Lua state initialization */
+#ifdef WITH_LUA
+    (void)rpmluaGetPrintBuffer(NULL);
+#endif
+
     return 0;
 }
 
