@@ -13,6 +13,7 @@
 #include "rpmal.h"
 #include "rpmds.h"
 #include "rpmfi.h"
+#include "rpmlock.h"
 
 #define	_RPMTE_INTERNAL		/* XXX te->h */
 #include "rpmte.h"
@@ -52,12 +53,11 @@ extern int statvfs (const char * file, /*@out@*/ struct statvfs * buf)
 
 /*@access rpmdb @*/		/* XXX db->db_chrootDone, NULL */
 
-/*@access FD_t @*/		/* XXX compared with NULL */
 /*@access rpmps @*/
 /*@access rpmDiskSpaceInfo @*/
+/*@access rpmsx @*/
 /*@access rpmte @*/
 /*@access rpmtsi @*/
-/*@access rpmts @*/
 /*@access fnpyKey @*/
 /*@access pgpDig @*/
 /*@access pgpDigParams @*/
@@ -65,6 +65,9 @@ extern int statvfs (const char * file, /*@out@*/ struct statvfs * buf)
 /*@unchecked@*/
 int _rpmts_debug = 0;
 
+/*@unchecked@*/
+int _rpmts_stats = 0;
+
 char * hGetNEVR(Header h, const char ** np)
 {
     const char * n, * v, * r;
@@ -84,6 +87,52 @@ char * hGetNEVR(Header h, const char ** np)
     return NVR;
 }
 
+char * hGetNEVRA(Header h, const char ** np)
+{
+    const char * n, * v, * r, * a;
+    char * NVRA, * t;
+    int xx;
+
+    (void) headerNVR(h, &n, &v, &r);
+    xx = headerGetEntry(h, RPMTAG_ARCH, NULL, (void **) &a, NULL);
+    NVRA = t = xcalloc(1, strlen(n) + strlen(v) + strlen(r) + strlen(a) + sizeof("--."));
+/*@-boundswrite@*/
+    t = stpcpy(t, n);
+    t = stpcpy(t, "-");
+    t = stpcpy(t, v);
+    t = stpcpy(t, "-");
+    t = stpcpy(t, r);
+    t = stpcpy(t, ".");
+    t = stpcpy(t, a);
+    if (np)
+	*np = n;
+/*@=boundswrite@*/
+    return NVRA;
+}
+
+uint_32 hGetColor(Header h)
+{
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    uint_32 hcolor = 0;
+    uint_32 * fcolors;
+    int_32 ncolors;
+    int i;
+
+    fcolors = NULL;
+    ncolors = 0;
+    if (hge(h, RPMTAG_FILECOLORS, NULL, (void **)&fcolors, &ncolors)
+     && fcolors != NULL && ncolors > 0)
+    {
+/*@-boundsread@*/
+	for (i = 0; i < ncolors; i++)
+	    hcolor |= fcolors[i];
+/*@=boundsread@*/
+    }
+    hcolor &= 0x0f;
+
+    return hcolor;
+}
+
 rpmts XrpmtsUnlink(rpmts ts, const char * msg, const char * fn, unsigned ln)
 {
 /*@-modfilesys@*/
@@ -109,6 +158,9 @@ int rpmtsCloseDB(rpmts ts)
     int rc = 0;
 
     if (ts->rdb != NULL) {
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBGET), &ts->rdb->db_getops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBPUT), &ts->rdb->db_putops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBDEL), &ts->rdb->db_delops);
 	rc = rpmdbClose(ts->rdb);
 	ts->rdb = NULL;
     }
@@ -140,16 +192,24 @@ int rpmtsOpenDB(rpmts ts, int dbmode)
 
 int rpmtsInitDB(rpmts ts, int dbmode)
 {
-    return rpmdbInit(ts->rootDir, dbmode);
+    void *lock = rpmtsAcquireLock(ts);
+    int rc = -1;
+    if (lock)
+	    rc = rpmdbInit(ts->rootDir, dbmode);
+    rpmtsFreeLock(lock);
+    return rc;
 }
 
 int rpmtsRebuildDB(rpmts ts)
 {
     int rc;
+    void *lock = rpmtsAcquireLock(ts);
+    if (!lock) return -1;
     if (!(ts->vsflags & RPMVSF_NOHDRCHK))
 	rc = rpmdbRebuild(ts->rootDir, ts, headerCheck);
     else
 	rc = rpmdbRebuild(ts->rootDir, NULL, NULL);
+    rpmtsFreeLock(lock);
     return rc;
 }
 
@@ -158,42 +218,158 @@ int rpmtsVerifyDB(rpmts ts)
     return rpmdbVerify(ts->rootDir);
 }
 
+/*@-boundsread@*/
+static int isArch(const char * arch)
+	/*@*/
+{
+    const char ** av;
+/*@-nullassign@*/
+    /*@observer@*/
+    static const char *arches[] = {
+	"i386", "i486", "i586", "i686", "athlon", "pentium3", "pentium4", "x86_64", "amd64", "ia32e", "geode",
+	"alpha", "alphaev5", "alphaev56", "alphapca56", "alphaev6", "alphaev67",
+	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv8", "sparcv9", "sparcv9v",
+	"sparc64", "sparc64v", "sun4u",
+	"mips", "mipsel", "IP",
+	"ppc", "ppciseries", "ppcpseries",
+	"ppc64", "ppc64iseries", "ppc64pseries",
+	"m68k",
+	"rs6000",
+	"ia64",
+	"armv3l", "armv4b", "armv4l", "armv4tl", "armv5tel", "armv5tejl", "armv6l",
+	"s390", "i370", "s390x",
+	"sh", "xtensa",
+	"noarch",
+	NULL,
+    };
+/*@=nullassign@*/
+
+    for (av = arches; *av != NULL; av++) {
+	if (!strcmp(arch, *av))
+	    return 1;
+    }
+    return 0;
+}
+/*@=boundsread@*/
+
+/*@-compdef@*/ /* keyp might no be defined. */
 rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 			const void * keyp, size_t keylen)
 {
     rpmdbMatchIterator mi;
+    const char * arch = NULL;
+    int xx;
+
     if (ts->rdb == NULL && rpmtsOpenDB(ts, ts->dbmode))
 	return NULL;
+
+    /* Parse out "N(EVR).A" tokens from a label key. */
+/*@-bounds -branchstate@*/
+    if (rpmtag == RPMDBI_LABEL && keyp != NULL) {
+	const char * s = keyp;
+	const char *se;
+	size_t slen = strlen(s);
+	char *t = alloca(slen+1);
+	int level = 0;
+	int c;
+
+	keyp = t;
+	while ((c = *s++) != '\0') {
+	    switch (c) {
+	    default:
+		*t++ = c;
+		/*@switchbreak@*/ break;
+	    case '(':
+		/* XXX Fail if nested parens. */
+		if (level++ != 0) {
+		    rpmError(RPMERR_QFMT, _("extra '(' in package label: %s\n"), keyp);
+		    return NULL;
+		}
+		/* Parse explicit epoch. */
+		for (se = s; *se && xisdigit(*se); se++)
+		    {};
+		if (*se == ':') {
+		    /* XXX skip explicit epoch's (for now) */
+		    *t++ = '-';
+		    s = se + 1;
+		} else {
+		    /* No Epoch: found. Convert '(' to '-' and chug. */
+		    *t++ = '-';
+		}
+		/*@switchbreak@*/ break;
+	    case ')':
+		/* XXX Fail if nested parens. */
+		if (--level != 0) {
+		    rpmError(RPMERR_QFMT, _("missing '(' in package label: %s\n"), keyp);
+		    return NULL;
+		}
+		/* Don't copy trailing ')' */
+		/*@switchbreak@*/ break;
+	    }
+	}
+	if (level) {
+	    rpmError(RPMERR_QFMT, _("missing ')' in package label: %s\n"), keyp);
+	    return NULL;
+	}
+	*t = '\0';
+	t = (char *) keyp;
+	t = strrchr(t, '.');
+	/* Is this a valid ".arch" suffix? */
+	if (t != NULL && isArch(t+1)) {
+	   *t++ = '\0';
+	   arch = t;
+	}
+    }
+/*@=bounds =branchstate@*/
+
     mi = rpmdbInitIterator(ts->rdb, rpmtag, keyp, keylen);
+
+    /* Verify header signature/digest during retrieve (if not disabled). */
     if (mi && !(ts->vsflags & RPMVSF_NOHDRCHK))
 	(void) rpmdbSetHdrChk(mi, ts, headerCheck);
+
+    /* Select specified arch only. */
+    if (arch != NULL)
+	xx = rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_DEFAULT, arch);
     return mi;
 }
+/*@=compdef@*/
 
 rpmRC rpmtsFindPubkey(rpmts ts)
 {
     const void * sig = rpmtsSig(ts);
     pgpDig dig = rpmtsDig(ts);
     pgpDigParams sigp = rpmtsSignature(ts);
-    pgpDigParams pubp = rpmtsSignature(ts);
-    rpmRC res;
+    pgpDigParams pubp = rpmtsPubkey(ts);
+    rpmRC res = RPMRC_NOKEY;
+    const char * pubkeysource = NULL;
     int xx;
 
-    if (sig == NULL || dig == NULL || sigp == NULL || pubp == NULL) {
-	res = RPMRC_NOKEY;
+    if (sig == NULL || dig == NULL || sigp == NULL || pubp == NULL)
 	goto exit;
-    }
 
-    if (ts->pkpkt == NULL
-     || memcmp(sigp->signid, ts->pksignid, sizeof(ts->pksignid)))
-    {
-	int ix = -1;
-	rpmdbMatchIterator mi;
-	Header h;
+#if 0
+fprintf(stderr, "==> find sig id %08x %08x ts pubkey id %08x %08x\n",
+pgpGrab(sigp->signid, 4), pgpGrab(sigp->signid+4, 4),
+pgpGrab(ts->pksignid, 4), pgpGrab(ts->pksignid+4, 4));
+#endif
 
+    /* Lazy free of previous pubkey if pubkey does not match this signature. */
+    if (memcmp(sigp->signid, ts->pksignid, sizeof(ts->pksignid))) {
+#if 0
+fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, pgpGrab(ts->pksignid, 4), pgpGrab(ts->pksignid+4, 4));
+#endif
 	ts->pkpkt = _free(ts->pkpkt);
 	ts->pkpktlen = 0;
 	memset(ts->pksignid, 0, sizeof(ts->pksignid));
+    }
+
+    /* Try rpmdb keyring lookup. */
+    if (ts->pkpkt == NULL) {
+	int hx = -1;
+	int ix = -1;
+	rpmdbMatchIterator mi;
+	Header h;
 
 	/* Retrieve the pubkey that matches the signature. */
 	mi = rpmtsInitIterator(ts, RPMTAG_PUBKEYS, sigp->signid, sizeof(sigp->signid));
@@ -203,6 +379,7 @@ rpmRC rpmtsFindPubkey(rpmts ts)
 
 	    if (!headerGetEntry(h, RPMTAG_PUBKEYS, &pt, (void **)&pubkeys, &pc))
 		continue;
+	    hx = rpmdbGetIteratorOffset(mi);
 	    ix = rpmdbGetIteratorFileNum(mi);
 /*@-boundsread@*/
 	    if (ix >= pc
@@ -214,57 +391,61 @@ rpmRC rpmtsFindPubkey(rpmts ts)
 	}
 	mi = rpmdbFreeIterator(mi);
 
-	/* Was a matching pubkey found? */
-	if (ix < 0 || ts->pkpkt == NULL) {
-	    res = RPMRC_NOKEY;
-	    goto exit;
-	}
-
-	/*
-	 * Can the pubkey packets be parsed?
-	 * Do the parameters match the signature?
-	 */
-	if (pgpPrtPkts(ts->pkpkt, ts->pkpktlen, NULL, 0)
-	 && sigp->pubkey_algo == pubp->pubkey_algo
-#ifdef	NOTYET
-	 && sigp->hash_algo == pubp->hash_algo
-#endif
-	 && !memcmp(sigp->signid, pubp->signid, sizeof(sigp->signid)))
-	{
+/*@-branchstate@*/
+	if (ix >= 0) {
+	    char hnum[32];
+	    sprintf(hnum, "h#%d", hx);
+	    pubkeysource = xstrdup(hnum);
+	} else {
 	    ts->pkpkt = _free(ts->pkpkt);
 	    ts->pkpktlen = 0;
-	    res = RPMRC_NOKEY;
-	    goto exit;
 	}
+/*@=branchstate@*/
+    }
 
-	/* XXX Verify the pubkey signature. */
-
-	/* Packet looks good, save the signer id. */
-/*@-boundsread@*/
-	memcpy(ts->pksignid, sigp->signid, sizeof(ts->pksignid));
-/*@=boundsread@*/
-
-	rpmMessage(RPMMESS_DEBUG, "========== %s pubkey id %s\n",
-		(sigp->pubkey_algo == PGPPUBKEYALGO_DSA ? "DSA" :
-		(sigp->pubkey_algo == PGPPUBKEYALGO_RSA ? "RSA" : "???")),
-		pgpHexStr(sigp->signid, sizeof(sigp->signid)));
+    /* Try keyserver lookup. */
+    if (ts->pkpkt == NULL) {
+	const char * fn = rpmExpand("%{_hkp_keyserver_query}",
+			pgpHexStr(sigp->signid, sizeof(sigp->signid)), NULL);
 
+	xx = 0;
+	if (fn && *fn != '%') {
+	    xx = (pgpReadPkts(fn,&ts->pkpkt,&ts->pkpktlen) != PGPARMOR_PUBKEY);
+	}
+	fn = _free(fn);
+/*@-branchstate@*/
+	if (xx) {
+	    ts->pkpkt = _free(ts->pkpkt);
+	    ts->pkpktlen = 0;
+	} else {
+	    /* Save new pubkey in local ts keyring for delayed import. */
+	    pubkeysource = xstrdup("keyserver");
+	}
+/*@=branchstate@*/
     }
 
 #ifdef	NOTNOW
-    {
-	if (ts->pkpkt == NULL) {
-	    const char * pkfn = rpmExpand("%{_gpg_pubkey}", NULL);
-	    if (pgpReadPkts(pkfn, &ts->pkpkt, &ts->pkpktlen) != PGPARMOR_PUBKEY) {
-		pkfn = _free(pkfn);
-		res = RPMRC_NOKEY;
-		goto exit;
-	    }
-	    pkfn = _free(pkfn);
+    /* Try filename from macro lookup. */
+    if (ts->pkpkt == NULL) {
+	const char * fn = rpmExpand("%{_gpg_pubkey}", NULL);
+
+	xx = 0;
+	if (fn && *fn != '%')
+	    xx = (pgpReadPkts(fn,&ts->pkpkt,&ts->pkpktlen) != PGPARMOR_PUBKEY);
+	fn = _free(fn);
+	if (xx) {
+	    ts->pkpkt = _free(ts->pkpkt);
+	    ts->pkpktlen = 0;
+	} else {
+	    pubkeysource = xstrdup("macro");
 	}
     }
 #endif
 
+    /* Was a matching pubkey found? */
+    if (ts->pkpkt == NULL || ts->pkpktlen == 0)
+	goto exit;
+
     /* Retrieve parameters from pubkey packet(s). */
     xx = pgpPrtPkts(ts->pkpkt, ts->pkpktlen, dig, 0);
 
@@ -274,13 +455,31 @@ rpmRC rpmtsFindPubkey(rpmts ts)
      && sigp->hash_algo == pubp->hash_algo
 #endif
      &&	!memcmp(sigp->signid, pubp->signid, sizeof(sigp->signid)) )
-	res = RPMRC_OK;
-    else
-	res = RPMRC_NOKEY;
+    {
 
-    /* XXX Verify the signature signature. */
+	/* XXX Verify any pubkey signatures. */
+
+	/* Pubkey packet looks good, save the signer id. */
+/*@-boundsread@*/
+	memcpy(ts->pksignid, pubp->signid, sizeof(ts->pksignid));
+/*@=boundsread@*/
+
+	if (pubkeysource)
+	    rpmMessage(RPMMESS_DEBUG, "========== %s pubkey id %08x %08x (%s)\n",
+		(sigp->pubkey_algo == PGPPUBKEYALGO_DSA ? "DSA" :
+		(sigp->pubkey_algo == PGPPUBKEYALGO_RSA ? "RSA" : "???")),
+		pgpGrab(sigp->signid, 4), pgpGrab(sigp->signid+4, 4),
+		pubkeysource);
+
+	res = RPMRC_OK;
+    }
 
 exit:
+    pubkeysource = _free(pubkeysource);
+    if (res != RPMRC_OK) {
+	ts->pkpkt = _free(ts->pkpkt);
+	ts->pkpktlen = 0;
+    }
     return res;
 }
 
@@ -289,6 +488,9 @@ int rpmtsCloseSDB(rpmts ts)
     int rc = 0;
 
     if (ts->sdb != NULL) {
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBGET), &ts->sdb->db_getops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBPUT), &ts->sdb->db_putops);
+	(void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DBDEL), &ts->sdb->db_delops);
 	rc = rpmdbClose(ts->sdb);
 	ts->sdb = NULL;
     }
@@ -440,7 +642,7 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	if (fd == NULL || Ferror(fd)) {
 	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), str,
 			Fstrerror(fd));
-            if (fd) {
+            if (fd != NULL) {
                 xx = Fclose(fd);
                 fd = NULL;
             }
@@ -599,6 +801,8 @@ void rpmtsEmpty(rpmts ts)
     pi = rpmtsiFree(pi);
 
     ts->orderCount = 0;
+    ts->ntrees = 0;
+    ts->maxDepth = 0;
 
     ts->numRemovedPackages = 0;
 /*@-nullstate@*/	/* FIX: partial annotations */
@@ -606,6 +810,43 @@ void rpmtsEmpty(rpmts ts)
 /*@=nullstate@*/
 }
 
+static void rpmtsPrintStat(const char * name, /*@null@*/ struct rpmop_s * op)
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/
+{
+    static unsigned int scale = (1000 * 1000);
+    if (op != NULL && op->count > 0)
+	fprintf(stderr, "   %s %6d %6lu.%06lu MB %6lu.%06lu secs\n",
+		name, op->count,
+		(unsigned long)op->bytes/scale, (unsigned long)op->bytes%scale,
+		op->usecs/scale, op->usecs%scale);
+}
+
+static void rpmtsPrintStats(rpmts ts)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/
+{
+    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_TOTAL), 0);
+
+    rpmtsPrintStat("total:       ", rpmtsOp(ts, RPMTS_OP_TOTAL));
+    rpmtsPrintStat("check:       ", rpmtsOp(ts, RPMTS_OP_CHECK));
+    rpmtsPrintStat("order:       ", rpmtsOp(ts, RPMTS_OP_ORDER));
+    rpmtsPrintStat("fingerprint: ", rpmtsOp(ts, RPMTS_OP_FINGERPRINT));
+    rpmtsPrintStat("repackage:   ", rpmtsOp(ts, RPMTS_OP_REPACKAGE));
+    rpmtsPrintStat("install:     ", rpmtsOp(ts, RPMTS_OP_INSTALL));
+    rpmtsPrintStat("erase:       ", rpmtsOp(ts, RPMTS_OP_ERASE));
+    rpmtsPrintStat("scriptlets:  ", rpmtsOp(ts, RPMTS_OP_SCRIPTLETS));
+    rpmtsPrintStat("compress:    ", rpmtsOp(ts, RPMTS_OP_COMPRESS));
+    rpmtsPrintStat("uncompress:  ", rpmtsOp(ts, RPMTS_OP_UNCOMPRESS));
+    rpmtsPrintStat("digest:      ", rpmtsOp(ts, RPMTS_OP_DIGEST));
+    rpmtsPrintStat("signature:   ", rpmtsOp(ts, RPMTS_OP_SIGNATURE));
+    rpmtsPrintStat("dbadd:       ", rpmtsOp(ts, RPMTS_OP_DBADD));
+    rpmtsPrintStat("dbremove:    ", rpmtsOp(ts, RPMTS_OP_DBREMOVE));
+    rpmtsPrintStat("dbget:       ", rpmtsOp(ts, RPMTS_OP_DBGET));
+    rpmtsPrintStat("dbput:       ", rpmtsOp(ts, RPMTS_OP_DBPUT));
+    rpmtsPrintStat("dbdel:       ", rpmtsOp(ts, RPMTS_OP_DBDEL));
+}
+
 rpmts rpmtsFree(rpmts ts)
 {
     if (ts == NULL)
@@ -622,6 +863,8 @@ rpmts rpmtsFree(rpmts ts)
 
     (void) rpmtsCloseSDB(ts);
 
+    ts->sx = rpmsxFree(ts->sx);
+
     ts->removedPackages = _free(ts->removedPackages);
 
     ts->availablePackages = rpmalFree(ts->availablePackages);
@@ -646,6 +889,14 @@ rpmts rpmtsFree(rpmts ts)
     ts->pkpktlen = 0;
     memset(ts->pksignid, 0, sizeof(ts->pksignid));
 
+    if (_rpmts_stats)
+	rpmtsPrintStats(ts);
+
+    /* Free up the memory used by the rpmtsScore */
+/*@-kepttrans -onlytrans @*/
+    ts->score = rpmtsScoreFree(ts->score);
+/*@=kepttrans =onlytrans @*/
+
     (void) rpmtsUnlink(ts, "tsCreate");
 
     /*@-refcounttrans -usereleased @*/
@@ -673,6 +924,36 @@ rpmVSFlags rpmtsSetVSFlags(rpmts ts, rpmVSFlags vsflags)
     return ovsflags;
 }
 
+/* 
+ * This allows us to mark transactions as being of a certain type.
+ * The three types are:
+ * 
+ *     RPM_TRANS_NORMAL 	
+ *     RPM_TRANS_ROLLBACK
+ *     RPM_TRANS_AUTOROLLBACK
+ * 
+ * ROLLBACK and AUTOROLLBACK transactions should always be ran as
+ * a best effort.  In particular this is important to the autorollback 
+ * feature to avoid rolling back a rollback (otherwise known as 
+ * dueling rollbacks (-;).  AUTOROLLBACK's additionally need instance 
+ * counts passed to scriptlets to be altered.
+ */
+void rpmtsSetType(rpmts ts, rpmtsType type)
+{
+    if (ts != NULL) {
+	ts->type = type;
+    }	 
+}
+
+/* Let them know what type of transaction we are */
+rpmtsType rpmtsGetType(rpmts ts) 
+{
+    if (ts != NULL) 
+	return ts->type;
+    else
+	return 0;
+}
+
 int rpmtsUnorderedSuccessors(rpmts ts, int first)
 {
     int unorderedSuccessors = 0;
@@ -686,7 +967,27 @@ int rpmtsUnorderedSuccessors(rpmts ts, int first)
 
 const char * rpmtsRootDir(rpmts ts)
 {
-    return (ts != NULL ? ts->rootDir : NULL);
+    const char * rootDir = NULL;
+
+/*@-branchstate@*/
+    if (ts != NULL && ts->rootDir != NULL) {
+	urltype ut = urlPath(ts->rootDir, &rootDir);
+	switch (ut) {
+	case URL_IS_UNKNOWN:
+	case URL_IS_PATH:
+	    break;
+	case URL_IS_HTTPS:
+	case URL_IS_HTTP:
+	case URL_IS_HKP:
+	case URL_IS_FTP:
+	case URL_IS_DASH:
+	default:
+	    rootDir = "/";
+	    break;
+	}
+    }
+/*@=branchstate@*/
+    return rootDir;
 }
 
 void rpmtsSetRootDir(rpmts ts, const char * rootDir)
@@ -754,18 +1055,21 @@ void rpmtsSetScriptFd(rpmts ts, FD_t scriptFd)
 	    ts->scriptFd = fdFree(ts->scriptFd, "rpmtsSetScriptFd");
 	    ts->scriptFd = NULL;
 	}
+/*@+voidabstract@*/
 	if (scriptFd != NULL)
-	    ts->scriptFd = fdLink(scriptFd, "rpmtsSetScriptFd");
+	    ts->scriptFd = fdLink((void *)scriptFd, "rpmtsSetScriptFd");
+/*@=voidabstract@*/
     }
 }
 
+int rpmtsSELinuxEnabled(rpmts ts)
+{
+    return (ts != NULL ? (ts->selinuxEnabled > 0) : 0);
+}
+
 int rpmtsChrootDone(rpmts ts)
 {
-    int chrootDone = 0;
-    if (ts != NULL) {
-	chrootDone = ts->chrootDone;
-    }
-    return chrootDone;
+    return (ts != NULL ? ts->chrootDone : 0);
 }
 
 int rpmtsSetChrootDone(rpmts ts, int chrootDone)
@@ -780,9 +1084,26 @@ int rpmtsSetChrootDone(rpmts ts, int chrootDone)
     return ochrootDone;
 }
 
+rpmsx rpmtsREContext(rpmts ts)
+{
+    return ( (ts && ts->sx ? rpmsxLink(ts->sx, __func__) : NULL) );
+}
+
+int rpmtsSetREContext(rpmts ts, rpmsx sx)
+{
+    int rc = -1;
+    if (ts != NULL) {
+	ts->sx = rpmsxFree(ts->sx);
+	ts->sx = rpmsxLink(sx, __func__);
+	if (ts->sx != NULL)
+	    rc = 0;
+    }
+    return rc;
+}
+
 int_32 rpmtsGetTid(rpmts ts)
 {
-    int_32 tid = 0;
+    int_32 tid = -1;	/* XXX -1 is time(2) error return. */
     if (ts != NULL) {
 	tid = ts->tid;
     }
@@ -791,7 +1112,7 @@ int_32 rpmtsGetTid(rpmts ts)
 
 int_32 rpmtsSetTid(rpmts ts, int_32 tid)
 {
-    int_32 otid = 0;
+    int_32 otid = -1;	/* XXX -1 is time(2) error return. */
     if (ts != NULL) {
 	otid = ts->tid;
 	ts->tid = tid;
@@ -809,10 +1130,10 @@ int_32 rpmtsSigtag(const rpmts ts)
 
 int_32 rpmtsSigtype(const rpmts ts)
 {
-    int_32 sigtag = 0;
+    int_32 sigtype = 0;
     if (ts != NULL)
-	sigtag = ts->sigtag;
-    return sigtag;
+	sigtype = ts->sigtype;
+    return sigtype;
 }
 
 const void * rpmtsSig(const rpmts ts)
@@ -897,14 +1218,16 @@ int rpmtsInitDSI(const rpmts ts)
     if (rpmtsFilterFlags(ts) & RPMPROB_FILTER_DISKSPACE)
 	return 0;
 
+    rpmMessage(RPMMESS_DEBUG, _("mounted filesystems:\n"));
+    rpmMessage(RPMMESS_DEBUG,
+	_("    i        dev    bsize       bavail       iavail mount point\n"));
+
     rc = rpmGetFilesystemList(&ts->filesystems, &ts->filesystemCount);
     if (rc || ts->filesystems == NULL || ts->filesystemCount <= 0)
 	return rc;
 
     /* Get available space on mounted file systems. */
 
-    rpmMessage(RPMMESS_DEBUG, _("getting list of mounted filesystems\n"));
-
     ts->dsi = _free(ts->dsi);
     ts->dsi = xcalloc((ts->filesystemCount + 1), sizeof(*ts->dsi));
 
@@ -953,6 +1276,10 @@ int rpmtsInitDSI(const rpmts ts)
 	/* XXX Avoid FAT and other file systems that have not inodes. */
 	dsi->iavail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
 				? sfb.f_ffree : -1;
+	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%08x %8u %12ld %12ld %s\n"),
+		i, (unsigned) dsi->dev, (unsigned) dsi->bsize,
+		(signed long) dsi->bavail, (signed long) dsi->iavail,
+		ts->filesystems[i]);
     }
     return rc;
 }
@@ -1027,16 +1354,16 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
     ps = rpmtsProblems(ts);
     for (i = 0; i < ts->filesystemCount; i++, dsi++) {
 
-	if (dsi->bavail > 0 && adj_fs_blocks(dsi->bneeded) > dsi->bavail) {
+	if (dsi->bavail >= 0 && adj_fs_blocks(dsi->bneeded) > dsi->bavail) {
 	    rpmpsAppend(ps, RPMPROB_DISKSPACE,
-			rpmteNEVR(te), rpmteKey(te),
+			rpmteNEVRA(te), rpmteKey(te),
 			ts->filesystems[i], NULL, NULL,
  	   (adj_fs_blocks(dsi->bneeded) - dsi->bavail) * dsi->bsize);
 	}
 
-	if (dsi->iavail > 0 && adj_fs_blocks(dsi->ineeded) > dsi->iavail) {
+	if (dsi->iavail >= 0 && adj_fs_blocks(dsi->ineeded) > dsi->iavail) {
 	    rpmpsAppend(ps, RPMPROB_DISKNODES,
-			rpmteNEVR(te), rpmteKey(te),
+			rpmteNEVRA(te), rpmteKey(te),
 			ts->filesystems[i], NULL, NULL,
  	    (adj_fs_blocks(dsi->ineeded) - dsi->iavail));
 	}
@@ -1101,6 +1428,69 @@ rpmtransFlags rpmtsSetFlags(rpmts ts, rpmtransFlags transFlags)
     return otransFlags;
 }
 
+Spec rpmtsSpec(rpmts ts)
+{
+/*@-compdef -retexpose -usereleased@*/
+    return ts->spec;
+/*@=compdef =retexpose =usereleased@*/
+}
+
+Spec rpmtsSetSpec(rpmts ts, Spec spec)
+{
+    Spec ospec = ts->spec;
+/*@-assignexpose -temptrans@*/
+    ts->spec = spec;
+/*@=assignexpose =temptrans@*/
+    return ospec;
+}
+
+rpmte rpmtsRelocateElement(rpmts ts)
+{
+/*@-compdef -retexpose -usereleased@*/
+    return ts->relocateElement;
+/*@=compdef =retexpose =usereleased@*/
+}
+
+rpmte rpmtsSetRelocateElement(rpmts ts, rpmte relocateElement)
+{
+    rpmte orelocateElement = ts->relocateElement;
+/*@-assignexpose -temptrans@*/
+    ts->relocateElement = relocateElement;
+/*@=assignexpose =temptrans@*/
+    return orelocateElement;
+}
+
+uint_32 rpmtsColor(rpmts ts)
+{
+    return (ts != NULL ? ts->color : 0);
+}
+
+uint_32 rpmtsSetColor(rpmts ts, uint_32 color)
+{
+    uint_32 ocolor = 0;
+    if (ts != NULL) {
+	ocolor = ts->color;
+	ts->color = color;
+    }
+    return ocolor;
+}
+
+uint_32 rpmtsPrefColor(rpmts ts)
+{
+    return (ts != NULL ? ts->prefcolor : 0);
+}
+
+rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
+{
+    rpmop op = NULL;
+
+    if (ts != NULL && opx >= 0 && opx < RPMTS_OP_MAX)
+	op = ts->ops + opx;
+/*@-usereleased -compdef @*/
+    return op;
+/*@=usereleased =compdef @*/
+}
+
 int rpmtsSetNotifyCallback(rpmts ts,
 		rpmCallbackFunction notify, rpmCallbackData notifyData)
 {
@@ -1146,6 +1536,9 @@ rpmts rpmtsCreate(void)
     rpmts ts;
 
     ts = xcalloc(1, sizeof(*ts));
+    memset(&ts->ops, 0, sizeof(ts->ops));
+    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_TOTAL), -1);
+    ts->type = RPMTRANS_TYPE_NORMAL;
     ts->goal = TSM_UNKNOWN;
     ts->filesystemCount = 0;
     ts->filesystems = NULL;
@@ -1165,6 +1558,9 @@ rpmts rpmtsCreate(void)
     ts->tid = (int_32) time(NULL);
     ts->delta = 5;
 
+    ts->color = rpmExpandNumeric("%{?_transaction_color}");
+    ts->prefcolor = rpmExpandNumeric("%{?_prefer_color}")?:2;
+
     ts->numRemovedPackages = 0;
     ts->allocedRemovedPackages = ts->delta;
     ts->removedPackages = xcalloc(ts->allocedRemovedPackages,
@@ -1174,6 +1570,8 @@ rpmts rpmtsCreate(void)
     ts->currDir = NULL;
     ts->chrootDone = 0;
 
+    ts->selinuxEnabled = is_selinux_enabled();
+
     ts->numAddedPackages = 0;
     ts->addedPackages = NULL;
 
@@ -1183,6 +1581,8 @@ rpmts rpmtsCreate(void)
     ts->orderAlloced = 0;
     ts->orderCount = 0;
     ts->order = NULL;
+    ts->ntrees = 0;
+    ts->maxDepth = 0;
 
     ts->probs = NULL;
 
@@ -1192,7 +1592,193 @@ rpmts rpmtsCreate(void)
     memset(ts->pksignid, 0, sizeof(ts->pksignid));
     ts->dig = NULL;
 
+    /* 
+     * We only use the score in an autorollback.  So set this to
+     * NULL by default.
+     */
+    ts->score = NULL;
+
     ts->nrefs = 0;
 
     return rpmtsLink(ts, "tsCreate");
 }
+
+/**********************
+ * Transaction Scores *
+ **********************/
+
+
+rpmRC rpmtsScoreInit(rpmts runningTS, rpmts rollbackTS) 
+{
+    rpmtsScore score;
+    rpmtsi     pi;
+    rpmte      p;
+    int        i;
+    int        tranElements;  /* Number of transaction elements in runningTS */
+    int        found = 0;
+    rpmRC      rc = RPMRC_OK; /* Assume success */
+    rpmtsScoreEntry se;
+
+    rpmMessage(RPMMESS_DEBUG, _("Creating transaction score board(%p, %p)\n"),
+	runningTS, rollbackTS); 
+
+    /* Allocate space for score board */
+    score = xcalloc(1, sizeof(*score));
+    rpmMessage(RPMMESS_DEBUG, _("\tScore board address:  %p\n"), score);
+
+    /* 
+     * Determine the maximum size needed for the entry list.
+     * XXX: Today, I just get the count of rpmts elements, and allocate
+     *      an array that big.  Yes this is guaranteed to waste memory.
+     *      Future updates will hopefully make this more efficient,
+     *      but for now it will work.
+     */
+    tranElements  = rpmtsNElements(runningTS);
+    rpmMessage(RPMMESS_DEBUG, _("\tAllocating space for %d entries\n"), tranElements);
+    score->scores = xcalloc(tranElements, sizeof(score->scores));
+
+    /* Initialize score entry count */
+    score->entries = 0;
+    score->nrefs   = 0;
+
+    /*
+     * Increment through transaction elements and make sure for every 
+     * N there is an rpmtsScoreEntry.
+     */
+    pi = rpmtsiInit(runningTS); 
+    while ((p = rpmtsiNext(pi, TR_ADDED|TR_REMOVED)) != NULL) {
+	found  = 0;
+
+	/* Try to find the entry in the score list */
+	for(i = 0; i < score->entries; i++) {
+	    se = score->scores[i]; 
+  	    if (strcmp(rpmteN(p), se->N) == 0) {
+		found = 1;
+	 	/*@innerbreak@*/ break;
+	    }
+	}
+
+	/* If we did not find the entry then allocate space for it */
+	if (!found) {
+/*@-compdef -usereleased@*/ /* XXX p->fi->te undefined. */
+    	    rpmMessage(RPMMESS_DEBUG, _("\tAdding entry for %s to score board.\n"),
+		rpmteN(p));
+/*@=compdef =usereleased@*/
+    	    se = xcalloc(1, sizeof(*(*(score->scores))));
+    	    rpmMessage(RPMMESS_DEBUG, _("\t\tEntry address:  %p\n"), se);
+	    se->N         = xstrdup(rpmteN(p));
+	    se->te_types  = rpmteType(p); 
+	    se->installed = 0;
+	    se->erased    = 0; 
+	    score->scores[score->entries] = se;
+	    score->entries++;
+	} else {
+	    /* We found this one, so just add the element type to the one 
+	     * already there.
+	     */
+    	    rpmMessage(RPMMESS_DEBUG, _("\tUpdating entry for %s in score board.\n"),
+		rpmteN(p));
+	    score->scores[i]->te_types |= rpmteType(p);
+	}
+	 
+    }
+    pi = rpmtsiFree(pi);
+ 
+    /* 
+     * Attach the score to the running transaction and the autorollback
+     * transaction.
+     */
+    runningTS->score  = score;
+    score->nrefs++;
+    rollbackTS->score = score;
+    score->nrefs++;
+
+    return rc;
+}
+
+rpmtsScore rpmtsScoreFree(rpmtsScore score) 
+{
+    rpmtsScoreEntry se = NULL;
+    int i;
+
+    rpmMessage(RPMMESS_DEBUG, _("May free Score board(%p)\n"), score);
+
+    /* If score is not initialized, then just return.
+     * This is likely the case if autorollbacks are not enabled.
+     */
+    if (score == NULL) return NULL;
+
+    /* Decrement the reference count */
+    score->nrefs--;
+
+    /* Do we have any more references?  If so
+     * just return.
+     */
+    if (score->nrefs > 0) return NULL;
+
+    rpmMessage(RPMMESS_DEBUG, _("\tRefcount is zero...will free\n"));
+    /* No more references, lets clean up  */
+    /* First deallocate the score entries */
+/*@-branchstate@*/
+    for(i = 0; i < score->entries; i++) {
+	/* Get the score for the ith entry */
+	se = score->scores[i]; 
+	
+	/* Deallocate the score entries name */
+	se->N = _free(se->N);
+
+	/* Deallocate the score entry itself */
+	se = _free(se);
+    }
+/*@=branchstate@*/
+
+    /* Next deallocate the score entry table */
+    score->scores = _free(score->scores);
+
+    /* Finally deallocate the score itself */
+    score = _free(score);
+
+    return NULL;
+}
+
+/* 
+ * XXX: Do not get the score and then store it aside for later use.
+ *      we will delete it out from under you.  There is no rpmtsScoreLink()
+ *      as this may be a very temporary fix for autorollbacks.
+ */
+rpmtsScore rpmtsGetScore(rpmts ts) 
+{
+    if (ts == NULL) return NULL;
+    return ts->score;
+}
+
+/* 
+ * XXX: Do not get the score entry and then store it aside for later use.
+ *      we will delete it out from under you.  There is no 
+ *      rpmtsScoreEntryLink() as this may be a very temporary fix 
+ *      for autorollbacks.
+ * XXX: The scores are not sorted.  This should be fixed at earliest
+ *      opportunity (i.e. when we have the whole autorollback working).
+ */
+rpmtsScoreEntry rpmtsScoreGetEntry(rpmtsScore score, const char *N) 
+{
+    int i;
+    rpmtsScoreEntry se;
+    rpmtsScoreEntry ret = NULL; /* Assume we don't find it */
+
+    rpmMessage(RPMMESS_DEBUG, _("Looking in score board(%p) for %s\n"), score, N);
+
+    /* Try to find the entry in the score list */
+    for(i = 0; i < score->entries; i++) {
+	se = score->scores[i]; 
+ 	if (strcmp(N, se->N) == 0) {
+    	    rpmMessage(RPMMESS_DEBUG, _("\tFound entry at address:  %p\n"), se);
+	    ret = se;
+	    break;
+	}
+    }
+	
+/*@-compdef@*/ /* XXX score->scores undefined. */
+    return ret;	
+/*@=compdef@*/
+}
