@@ -8,6 +8,7 @@
 
 #include "rpmps.h"
 #include "rpmsw.h"
+#include "rpmsx.h"
 
 /*@-exportlocal@*/
 /*@unchecked@*/
@@ -38,6 +39,15 @@ typedef enum rpmVSFlags_e {
     /* bit(s) 16-31 unused */
 } rpmVSFlags;
 
+/**
+ * Transaction Types
+ */
+typedef enum rpmtsType_e {
+	RPMTRANS_TYPE_NORMAL       = 0,
+	RPMTRANS_TYPE_ROLLBACK     = (1 << 0),
+	RPMTRANS_TYPE_AUTOROLLBACK = (1 << 1)
+} rpmtsType;
+
 #define	_RPMVSF_NODIGESTS	\
   ( RPMVSF_NOSHA1HEADER |	\
     RPMVSF_NOMD5HEADER |	\
@@ -91,6 +101,94 @@ typedef	enum rpmtsOpX_e {
 #include "rpmhash.h"	/* XXX hashTable */
 #include "rpmal.h"	/* XXX availablePackage/relocateFileList ,*/
 
+/**********************
+ * Transaction Scores *
+ **********************
+ *
+ * In order to allow instance counts to be adjusted properly when an
+ * autorollback transaction is ran, we keep a list that is indexed
+ * by rpm name of whether the rpm has been installed or erased.  This listed
+ * is only updated:
+ *
+ *	iif autorollbacks are enabled.
+ *	iif this is not a rollback or autorollback transaction.
+ *
+ * When creating an autorollback transaction, its rpmts points to the same
+ * rpmtsScore object as the running transaction.  So when the autorollback
+ * transaction runs it can see where each package was in the running transaction
+ * at the point the running transaction failed, and thus on a per package
+ * basis make adjustments to the instance counts.
+ *
+ * XXX: Jeff, I am not convinced that this does not need to be in its own file
+ *      (i.e. rpmtsScore.{h,c}), but I first wanted to get it working.
+ */
+struct rpmtsScoreEntry_s {
+    char *         N;			/*!<Name of package                */
+    rpmElementType te_types;		/*!<te types this entry represents */
+    int            installed;		/*!<Was the new header installed   */
+    int            erased;		/*!<Was the old header removed     */
+};
+
+typedef /*@abstract@*/ struct rpmtsScoreEntry_s * rpmtsScoreEntry;
+
+struct rpmtsScore_s {
+	int entries;			/*!< Number of scores       */
+	rpmtsScoreEntry * scores;	/*!< Array of score entries */
+    	int nrefs;			/*!< Reference count.       */
+};
+
+typedef /*@abstract@*/ struct rpmtsScore_s * rpmtsScore;
+
+
+/** \ingroup rpmts
+ * initialize rpmtsScore for running transaction and autorollback
+ * transaction.
+ * @param runningTS	Running Transaction.
+ * @param rollbackTS	Rollback Transaction.
+ * @return		RPMRC_OK
+ */
+rpmRC rpmtsScoreInit(rpmts runningTS, rpmts rollbackTS)
+	/*@globals fileSystem @*/
+	/*@modifies runningTS, rollbackTS, fileSystem @*/;
+
+/** \ingroup rpmts
+ * Free rpmtsScore provided no more references exist against it.
+ * @param score		rpmtsScore to free
+ * @return		NULL always
+ */
+/*@-exportlocal@*/
+/*@null@*/
+rpmtsScore rpmtsScoreFree(/*@only@*/ /*@null@*/ rpmtsScore score)
+	/*@modifies score @*/;
+/*@=exportlocal@*/
+
+/** \ingroup rpmts
+ * Get rpmtsScore from transaction.
+ * @param ts	RPM Transaction.
+ * @return	rpmtsScore or NULL.
+ */
+/*@exposed@*/ /*@null@*/
+rpmtsScore rpmtsGetScore(rpmts ts)
+	/*@*/;
+
+/** \ingroup rpmts
+ * Get rpmtsScoreEntry from rpmtsScore.
+ * @param score   RPM Transaction Score.
+ * @return	  rpmtsScoreEntry or NULL.
+ */
+/*@null@*/
+rpmtsScoreEntry rpmtsScoreGetEntry(rpmtsScore score, const char *N)
+	/*@*/;
+
+/** \ingroup rpmts
+ * \file lib/rpmts.h
+ * Structures and prototypes used for an "rpmts" transaction set.
+ */
+
+/**************************
+ * END Transaction Scores *
+ **************************/
+
 /*@unchecked@*/
 /*@-exportlocal@*/
 extern int _cacheDependsRC;
@@ -134,6 +232,7 @@ typedef enum tsStage_e {
 struct rpmts_s {
     rpmtransFlags transFlags;	/*!< Bit(s) to control operation. */
     tsmStage goal;		/*!< Transaction goal (i.e. mode) */
+    rpmtsType type;             /*!< default, rollback, autorollback */
 
 /*@refcounted@*/ /*@null@*/
     rpmdb sdb;			/*!< Solve database handle. */
@@ -184,6 +283,9 @@ struct rpmts_s {
     int numAvailablePackages;	/*!< No. available package instances. */
 #endif
 
+/*@refcounted@*/ /*@null@*/
+    rpmsx sx;			/*!< Security context patterns. */
+
 /*@null@*/
     rpmte relocateElement;	/*!< Element to use when relocating packages. */
 
@@ -192,7 +294,10 @@ struct rpmts_s {
     int orderCount;		/*!< No. of transaction elements. */
     int orderAlloced;		/*!< No. of allocated transaction elements. */
     int unorderedSuccessors;	/*!< Index of 1st element of successors. */
+    int ntrees;			/*!< No. of dependency trees. */
+    int maxDepth;		/*!< Maximum depth of dependency tree(s). */
 
+    int selinuxEnabled;		/*!< Is SE linux enabled? */
     int chrootDone;		/*!< Has chroot(2) been been done? */
 /*@only@*/ /*@null@*/
     const char * rootDir;	/*!< Path to top of install tree. */
@@ -204,6 +309,7 @@ struct rpmts_s {
     int_32 tid;			/*!< Transaction id. */
 
     uint_32 color;		/*!< Transaction color bits. */
+    uint_32 prefcolor;		/*!< Preferred file color. */
 
     rpmVSFlags vsflags;		/*!< Signature/digest verification flags. */
 
@@ -228,9 +334,11 @@ struct rpmts_s {
 /*@null@*/
     Spec spec;			/*!< Spec file control structure. */
 
+/*@kept@*/ /*@null@*/
+    rpmtsScore score;		/*!< Transaction Score (autorollback). */
+
 /*@refs@*/
     int nrefs;			/*!< Reference count. */
-
 };
 #endif	/* _RPMTS_INTERNAL */
 
@@ -239,7 +347,12 @@ extern "C" {
 #endif
 
 /** \ingroup rpmts
- * Check that all dependencies can be resolved.
+ * Perform dependency resolution on the transaction set.
+ *
+ * Any problems found by rpmtsCheck() can be examined by retrieving the 
+ * problem set with rpmtsProblems(), success here only means that
+ * the resolution was successfully attempted for all packages in the set.
+ *
  * @param ts		transaction set
  * @return		0 on success
  */
@@ -268,8 +381,17 @@ int rpmtsOrder(rpmts ts)
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
- * Process all package elements in a transaction set.
+ * Process all package elements in a transaction set.  Before calling
+ * rpmtsRun be sure to have:
  *
+ *    - setup the rpm root dir via rpmtsSetRoot().
+ *    - setup the rpm notify callback via rpmtsSetNotifyCallback().
+ *    - setup the rpm transaction flags via rpmtsSetFlags().
+ * 
+ * Additionally, though not required you may want to:
+ *
+ *    - setup the rpm verify signature flags via rpmtsSetVSFlags().
+ *       
  * @param ts		transaction set
  * @param okProbs	previously known problems (or NULL)
  * @param ignoreSet	bits to filter problem types
@@ -382,9 +504,11 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
  * @param ts		rpm transaction
  * @return		RPMRC_OK on success, RPMRC_NOKEY if not found
  */
+/*@-exportlocal@*/
 rpmRC rpmtsFindPubkey(rpmts ts)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState */;
+/*@=exportlocal@*/
 
 /** \ingroup rpmts
  * Close the database used by the transaction to solve dependencies.
@@ -446,6 +570,28 @@ int rpmtsSetSolveCallback(rpmts ts,
 		const void * solveData)
 	/*@modifies ts @*/;
 
+/**
+ * Return the type of a transaction.
+ * @param ts		transaction set
+ * @return		0 it is not, 1 it is.
+ */
+rpmtsType rpmtsGetType(rpmts ts)
+	/*@*/;
+
+/**
+ * Set transaction type.   Allowed types are:
+ *
+ * 	RPMTRANS_TYPE_NORMAL
+ *	RPMTRANS_TYPE_ROLLBACK
+ * 	RPMTRANS_TYPE_AUTOROLLBACK
+ *
+ * @param ts		transaction set
+ * @param type		transaction type
+ * @return		void
+ */
+void rpmtsSetType(rpmts ts, rpmtsType type)
+	/*@modifies ts @*/;
+
 /**
  * Return current transaction set problems.
  * @param ts		transaction set
@@ -565,6 +711,14 @@ FD_t rpmtsScriptFd(rpmts ts)
 void rpmtsSetScriptFd(rpmts ts, /*@null@*/ FD_t scriptFd)
 	/*@modifies ts, scriptFd @*/;
 
+/** \ingroup rpmts
+ * Get selinuxEnabled flag, i.e. is SE linux enabled?
+ * @param ts		transaction set
+ * @return		selinuxEnabled flag
+ */
+int rpmtsSELinuxEnabled(rpmts ts)
+	/*@*/;
+
 /** \ingroup rpmts
  * Get chrootDone flag, i.e. has chroot(2) been performed?
  * @param ts		transaction set
@@ -582,10 +736,28 @@ int rpmtsChrootDone(rpmts ts)
 int rpmtsSetChrootDone(rpmts ts, int chrootDone)
 	/*@modifies ts @*/;
 
+/** \ingroup rpmts
+ * Get file security context patterns.
+ * @param ts		transaction set
+ * @return		file security context patterns
+ */
+/*@null@*/
+rpmsx rpmtsREContext(const rpmts ts)
+	/*@modifies ts @*/;
+
+/** \ingroup rpmts
+ * Get file security context patterns.
+ * @param ts		transaction set
+ * @param sx		security context patterns
+ * @return		0 on success
+ */
+int rpmtsSetREContext(rpmts ts, rpmsx sx)
+	/*@modifies ts, sx @*/;
+
 /** \ingroup rpmts
  * Get transaction id, i.e. transaction time stamp.
  * @param ts		transaction set
- * @return		chrootDone flag
+ * @return		transaction id
  */
 int_32 rpmtsGetTid(rpmts ts)
 	/*@*/;
@@ -816,6 +988,14 @@ rpmte rpmtsSetRelocateElement(rpmts ts, /*@null@*/ rpmte relocateElement)
 uint_32 rpmtsColor(rpmts ts)
 	/*@*/;
 
+/**
+ * Retrieve prefered file color
+ * @param ts		transaction set
+ * @return		color bits
+ */
+uint_32 rpmtsPrefColor(rpmts ts)
+	/*@*/;
+
 /**
  * Set color bits of transaction set.
  * @param ts		transaction set
@@ -831,7 +1011,7 @@ uint_32 rpmtsSetColor(rpmts ts, uint_32 color)
  * @param opx		operation timestamp index
  * @return		pointer to operation timestamp.
  */
-/*@null@*/
+/*@relnull@*/
 rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
 	/*@*/;
 
@@ -889,8 +1069,8 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
  * @return		0 on success
  */
 int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
-	/*@globals fileSystem @*/
-	/*@modifies ts, h, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
  * Retrieve keys from ordered transaction set.
@@ -916,6 +1096,15 @@ int rpmtsGetKeys(rpmts ts,
 /*@only@*/ char * hGetNEVR(Header h, /*@null@*/ /*@out@*/ const char ** np )
 	/*@modifies *np @*/;
 
+/**
+ * Return (malloc'd) header name-version-release.arch string.
+ * @param h		header
+ * @retval np		name tag value
+ * @return		name-version-release string
+ */
+/*@only@*/ char * hGetNEVRA(Header h, /*@null@*/ /*@out@*/ const char ** np )
+	/*@modifies *np @*/;
+
 /**
  * Return header color.
  * @param h		header
@@ -928,4 +1117,5 @@ uint_32 hGetColor(Header h)
 }
 #endif
 
+
 #endif	/* H_RPMTS */
