@@ -3,7 +3,7 @@
 
 #include "rpmfi.h"
 
-/** \ingroup python
+/** \ingroup py_c
  * \file python/rpmfi-py.h
  */
 
@@ -18,13 +18,17 @@ typedef struct rpmfiObject_s {
 /*@unchecked@*/
 extern PyTypeObject rpmfi_Type;
 
+/*@null@*/
 rpmfi fiFromFi(rpmfiObject * fi)
 	/*@*/;
 
+/*@null@*/
 rpmfiObject * rpmfi_Wrap(rpmfi fi)
 	/*@*/;
 
-rpmfiObject * hdr_fiFromHeader(PyObject * s, PyObject * args)
-	/*@*/;
+/*@null@*/
+rpmfiObject * hdr_fiFromHeader(PyObject * s, PyObject * args, PyObject * kwds)
+	/*@globals rpmGlobalMacroContext @*/
+	/*@modifies rpmGlobalMacroContext @*/;
 
 #endif
