@@ -20,6 +20,10 @@ static int _debug = 1;	/* XXX if < 0 debugging, > 0 unusual error returns */
 
 #include "debug.h"
 
+#if !defined(DB_CLIENT)	/* XXX db-4.2.42 retrofit */
+#define	DB_CLIENT	DB_RPCCLIENT
+#endif
+
 /*@access rpmdb @*/
 /*@access dbiIndex @*/
 /*@access dbiIndexSet @*/
@@ -213,6 +217,7 @@ static int db3_fsync_disable(/*@unused@*/ int fd)
     return 0;
 }
 
+#if 0
 #if HAVE_LIBPTHREAD
 #if HAVE_PTHREAD_H
 #include <pthread.h>
@@ -261,13 +266,14 @@ static int db3_pthread_nptl(void)
     return ret;
 }
 #endif
+#endif
 
 /*@-moduncon@*/ /* FIX: annotate db3 methods */
 static int db_init(dbiIndex dbi, const char * dbhome,
 		/*@null@*/ const char * dbfile,
 		/*@unused@*/ /*@null@*/ const char * dbsubfile,
 		/*@out@*/ DB_ENV ** dbenvp)
-	/*@globals rpmGlobalMacroContext,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem @*/
 	/*@modifies dbi, *dbenvp, fileSystem @*/
 {
@@ -348,11 +354,13 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 #endif
 	    if (!xx)
 		break;
-	    sleep(15);
+	    (void) sleep(15);
 	}
     } else {
+#if !(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
 	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
+#endif
 	xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
 		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
 	xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
@@ -369,8 +377,10 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	    const char * tmpdir;
 
 	    root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
+/*@-boundsread@*/
 	    if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 		root = NULL;
+/*@=boundsread@*/
 /*@-mods@*/
 	    tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
 /*@=mods@*/
@@ -420,15 +430,17 @@ static int db_init(dbiIndex dbi, const char * dbhome,
   }
 
 #if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0) || (DB_VERSION_MAJOR == 4)
-    rc = dbenv->open(dbenv, dbhome, eflags, dbi->dbi_perms);
+    rc = (dbenv->open)(dbenv, dbhome, eflags, dbi->dbi_perms);
 #else
-    rc = dbenv->open(dbenv, dbhome, NULL, eflags, dbi->dbi_perms);
+    rc = (dbenv->open)(dbenv, dbhome, NULL, eflags, dbi->dbi_perms);
 #endif
     rc = cvtdberr(dbi, "dbenv->open", rc, _debug);
     if (rc)
 	goto errxit;
 
+/*@-boundswrite@*/
     *dbenvp = dbenv;
+/*@=boundswrite@*/
 
     return 0;
 
@@ -453,7 +465,7 @@ static int db3sync(dbiIndex dbi, unsigned int flags)
     if (db != NULL)
 	rc = db->sync(db, flags);
     /* XXX DB_INCOMPLETE is returned occaisionally with multiple access. */
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 1)
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1)
     _printit = _debug;
 #else
     _printit = (rc == DB_INCOMPLETE ? 0 : _debug);
@@ -469,7 +481,9 @@ static int db3cdup(dbiIndex dbi, DBC * dbcursor, DBC ** dbcp,
 {
     int rc;
 
+/*@-boundswrite@*/
     if (dbcp) *dbcp = NULL;
+/*@=boundswrite@*/
     rc = dbcursor->c_dup(dbcursor, dbcp, flags);
     rc = cvtdberr(dbi, "dbcursor->c_dup", rc, _debug);
     /*@-nullstate @*/ /* FIX: *dbcp can be NULL */
@@ -477,8 +491,9 @@ static int db3cdup(dbiIndex dbi, DBC * dbcursor, DBC ** dbcp,
     /*@=nullstate @*/
 }
 
+/*@-mustmod@*/
 static int db3cclose(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor,
-		unsigned int flags)
+		/*@unused@*/ unsigned int flags)
 	/*@globals fileSystem @*/
 	/*@modifies dbi, fileSystem @*/
 {
@@ -491,6 +506,7 @@ static int db3cclose(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor,
     }
     return rc;
 }
+/*@=mustmod@*/
 
 static int db3copen(dbiIndex dbi, DB_TXN * txnid,
 		/*@null@*/ /*@out@*/ DBC ** dbcp, unsigned int dbiflags)
@@ -502,6 +518,7 @@ static int db3copen(dbiIndex dbi, DB_TXN * txnid,
     int flags;
     int rc;
 
+   /* XXX DB_WRITECURSOR cannot be used with sunrpc dbenv. */
     assert(db != NULL);
     if ((dbiflags & DB_WRITECURSOR) &&
 	(dbi->dbi_eflags & DB_INIT_CDB) && !(dbi->dbi_oflags & DB_RDONLY))
@@ -514,7 +531,7 @@ static int db3copen(dbiIndex dbi, DB_TXN * txnid,
     rc = cvtdberr(dbi, "db->cursor", rc, _debug);
 
     if (dbcp)
-	/*@-onlytrans@*/ *dbcp = dbcursor; /*@=onlytrans@*/
+	/*@-boundswrite -onlytrans@*/ *dbcp = dbcursor; /*@=boundswrite =onlytrans@*/
     else
 	(void) db3cclose(dbi, dbcursor, 0);
 
@@ -522,7 +539,7 @@ static int db3copen(dbiIndex dbi, DB_TXN * txnid,
 }
 
 static int db3cput(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
-		unsigned int flags)
+		/*@unused@*/ unsigned int flags)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
@@ -541,6 +558,7 @@ static int db3cput(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
     return rc;
 }
 
+/*@-mustmod@*/
 static int db3cdel(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 		unsigned int flags)
 	/*@globals fileSystem @*/
@@ -570,7 +588,9 @@ static int db3cdel(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 
     return rc;
 }
+/*@=mustmod@*/
 
+/*@-mustmod@*/
 static int db3cget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 		unsigned int flags)
 	/*@globals fileSystem @*/
@@ -597,7 +617,9 @@ static int db3cget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 
     return rc;
 }
+/*@=mustmod@*/
 
+/*@-mustmod@*/
 static int db3cpget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * pkey,
 		DBT * data, unsigned int flags)
 	/*@globals fileSystem @*/
@@ -618,6 +640,7 @@ static int db3cpget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * pkey,
 
     return rc;
 }
+/*@=mustmod@*/
 
 static int db3ccount(dbiIndex dbi, DBC * dbcursor,
 		/*@null@*/ /*@out@*/ unsigned int * countp,
@@ -632,7 +655,9 @@ static int db3ccount(dbiIndex dbi, DBC * dbcursor,
     rc = dbcursor->c_count(dbcursor, &count, flags);
     rc = cvtdberr(dbi, "dbcursor->c_count", rc, _debug);
     if (rc) return rc;
+/*@-boundswrite@*/
     if (countp) *countp = count;
+/*@=boundswrite@*/
 
     return rc;
 }
@@ -662,6 +687,9 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
 	/*@modifies dbi, fileSystem @*/
 {
     DB * db = dbi->dbi_db;
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
+    DB_TXN * txnid = NULL;
+#endif
     int rc = 0;
 
     assert(db != NULL);
@@ -674,7 +702,11 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
     dbi->dbi_stats = _free(dbi->dbi_stats);
 /* XXX 3.3.4 change. */
 #if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3) || (DB_VERSION_MAJOR == 4)
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
+    rc = db->stat(db, txnid, &dbi->dbi_stats, flags);
+#else
     rc = db->stat(db, &dbi->dbi_stats, flags);
+#endif
 #else
     rc = db->stat(db, &dbi->dbi_stats, NULL, flags);
 #endif
@@ -682,6 +714,7 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
     return rc;
 }
 
+/*@-mustmod@*/
 static int db3associate(dbiIndex dbi, dbiIndex dbisecondary,
 		int (*callback)(DB *, const DBT *, const DBT *, DBT *),
 		unsigned int flags)
@@ -690,22 +723,25 @@ static int db3associate(dbiIndex dbi, dbiIndex dbisecondary,
 {
     DB * db = dbi->dbi_db;
     DB * secondary = dbisecondary->dbi_db;
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 1)
-    DB_TXN * txnid = NULL;
-#endif
     int rc;
 
 /*@-moduncon@*/ /* FIX: annotate db3 methods */
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 1)
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1)
+    DB_TXN * txnid = NULL;
+
+assert(db != NULL);
     rc = db->associate(db, txnid, secondary, callback, flags);
 #else
+assert(db != NULL);
     rc = db->associate(db, secondary, callback, flags);
 #endif
 /*@=moduncon@*/
     rc = cvtdberr(dbi, "db->associate", rc, _debug);
     return rc;
 }
+/*@=mustmod@*/
 
+/*@-mustmod@*/
 static int db3join(dbiIndex dbi, DBC ** curslist, DBC ** dbcp,
 		unsigned int flags)
 	/*@globals fileSystem @*/
@@ -714,16 +750,18 @@ static int db3join(dbiIndex dbi, DBC ** curslist, DBC ** dbcp,
     DB * db = dbi->dbi_db;
     int rc;
 
+assert(db != NULL);
 /*@-moduncon@*/ /* FIX: annotate db3 methods */
     rc = db->join(db, curslist, dbcp, flags);
 /*@=moduncon@*/
     rc = cvtdberr(dbi, "db->join", rc, _debug);
     return rc;
 }
+/*@=mustmod@*/
 
 /*@-moduncon@*/ /* FIX: annotate db3 methods */
 static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
-	/*@globals rpmGlobalMacroContext,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem @*/
 	/*@modifies dbi, fileSystem @*/
 {
@@ -744,8 +782,10 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
      * Get the prefix/root component and directory path.
      */
     root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
+/*@-boundsread@*/
     if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 	root = NULL;
+/*@=boundsread@*/
     home = (dbi->dbi_home ? dbi->dbi_home : rpmdb->db_home);
 
     /*
@@ -806,8 +846,10 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
  /*	dbenv->set_paniccall(???) */
 	/*@=noeffectuncon@*/
+#if !(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
 	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
+#endif
 	xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
 		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
 	xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
@@ -825,14 +867,14 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	    if (rc) goto exit;
 	}
 	    
-	rc = dbenv->open(dbenv, dbhome,
+	rc = (dbenv->open)(dbenv, dbhome,
             DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_USE_ENVIRON, 0);
 	rc = cvtdberr(dbi, "dbenv->open", rc, _debug);
 	if (rc) goto exit;
 
-	/*@-moduncon@*/ /* FIX: annotate db3 methods */
+	/*@-moduncon -nullstate@*/ /* FIX: annotate db3 methods */
 	rc = db_create(&db, dbenv, 0);
-	/*@=moduncon@*/
+	/*@=moduncon =nullstate@*/
 	rc = cvtdberr(dbi, "db_create", rc, _debug);
 
 	if (db != NULL) {
@@ -847,13 +889,11 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 			(dbhome ? dbhome : ""),
 			(dbfile ? dbfile : tagName(dbi->dbi_rpmtag)));
 
-		xx = db->close(db, 0);
-		/* XXX ignore not found error messages. */
-		_printit = (xx == ENOENT ? 0 : _debug);
-		xx = cvtdberr(dbi, "db->close", xx, _printit);
+	        /*
+		 * The DB handle may not be accessed again after
+		 * DB->verify is called, regardless of its return.
+		 */
 		db = NULL;
-		if (rc == 0 && xx) rc = xx;
-
 		dbf = _free(dbf);
 	}
 	xx = dbenv->close(dbenv, 0);
@@ -873,13 +913,13 @@ exit:
 /*@=moduncon@*/
 
 static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem @*/
-	/*@modifies *dbip, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno,
+		fileSystem, internalState @*/
+	/*@modifies *dbip, fileSystem, internalState @*/
 {
-    /*@-nestedextern@*/
+    /*@-nestedextern -shadow@*/
     extern struct _dbiVec db3vec;
-    /*@=nestedextern@*/
+    /*@=nestedextern =shadow@*/
     const char * urlfn = NULL;
     const char * root;
     const char * home;
@@ -892,14 +932,16 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 
     DB * db = NULL;
     DB_ENV * dbenv = NULL;
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 1)
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1)
     DB_TXN * txnid = NULL;
 #endif
     u_int32_t oflags;
     int _printit;
 
+/*@-boundswrite@*/
     if (dbip)
 	*dbip = NULL;
+/*@=boundswrite@*/
 
     /*
      * Parse db configuration parameters.
@@ -916,8 +958,10 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      * Get the prefix/root component and directory path.
      */
     root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
+/*@-boundsread@*/
     if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 	root = NULL;
+/*@=boundsread@*/
     home = (dbi->dbi_home ? dbi->dbi_home : rpmdb->db_home);
 
     /*
@@ -977,6 +1021,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      */
     if (dbi->dbi_use_dbenv) {
 
+#if 0
 #if HAVE_LIBPTHREAD
 	if (rpmdb->db_dbenv == NULL) {
 	    /* Set DB_PRIVATE if posix mutexes are not shared. */
@@ -986,6 +1031,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		rpmMessage(RPMMESS_DEBUG, _("unshared posix mutexes found(%d), adding DB_PRIVATE, using fcntl lock\n"), xx);
 	    }
 	}
+#endif
 #endif
 
 	if (access(dbhome, W_OK) == -1) {
@@ -1243,9 +1289,11 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		if (dbfile)	nb += 1 + strlen(dbfile);
 		dbfullpath = t = alloca(nb + 1);
 
+/*@-boundswrite@*/
 		t = stpcpy(t, dbhome);
 		if (dbfile)
 		    t = stpcpy( stpcpy( t, "/"), dbfile);
+/*@=boundswrite@*/
 #ifdef	HACK	/* XXX necessary to support dbsubfile */
 		dbpath = (!dbi->dbi_use_dbenv && !dbi->dbi_temporary)
 			? dbfullpath : dbfile;
@@ -1254,11 +1302,11 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 			? dbfullpath : dbfile;
 #endif
 
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 1)
-		rc = db->open(db, txnid, dbpath, dbsubfile,
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1)
+		rc = (db->open)(db, txnid, dbpath, dbsubfile,
 		    dbi->dbi_type, oflags, dbi->dbi_perms);
 #else
-		rc = db->open(db, dbpath, dbsubfile,
+		rc = (db->open)(db, dbpath, dbsubfile,
 		    dbi->dbi_type, oflags, dbi->dbi_perms);
 #endif
 
@@ -1309,7 +1357,9 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		    rc = 1;
 		} else {
 		    struct flock l;
+/*@-boundswrite@*/
 		    memset(&l, 0, sizeof(l));
+/*@=boundswrite@*/
 		    l.l_whence = 0;
 		    l.l_start = 0;
 		    l.l_len = 0;
@@ -1343,7 +1393,9 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 
     if (rc == 0 && dbi->dbi_db != NULL && dbip != NULL) {
 	dbi->dbi_vec = &db3vec;
+/*@-boundswrite@*/
 	*dbip = dbi;
+/*@=boundswrite@*/
     } else {
 	dbi->dbi_verify_on_close = 0;
 	(void) db3close(dbi, 0);
