@@ -6,12 +6,7 @@
 
 #define	_USE_COPY_LOAD	/* XXX don't use DB_DBT_MALLOC (yet) */
 
-/*@unchecked@*/
-int _rpmdb_debug = 0;
-
 #include <sys/file.h>
-#include <signal.h>
-#include <sys/signal.h>
 
 #ifndef	DYING	/* XXX already in "system.h" */
 /*@-noparams@*/
@@ -19,7 +14,7 @@ int _rpmdb_debug = 0;
 /*@=noparams@*/
 #if defined(__LCLINT__)
 /*@-declundef -exportheader -redecl @*/ /* LCL: missing annotation */
-extern int fnmatch (const char *pattern, const char *string, int flags)
+extern int fnmatch (const char *__pattern, const char *__name, int __flags)
 	/*@*/;
 /*@=declundef =exportheader =redecl @*/
 #endif
@@ -35,6 +30,7 @@ extern void regfree (/*@only@*/ regex_t *preg)
 
 #include <rpmio_internal.h>
 #include <rpmmacro.h>
+#include <rpmsq.h>
 
 #include "rpmdb.h"
 #include "fprint.h"
@@ -49,6 +45,9 @@ extern void regfree (/*@only@*/ regex_t *preg)
 /*@access rpmdbMatchIterator@*/
 /*@access pgpDig@*/
 
+/*@unchecked@*/
+int _rpmdb_debug = 0;
+
 /*@unchecked@*/
 static int _rebuildinprogress = 0;
 /*@unchecked@*/
@@ -63,6 +62,12 @@ static int _db_filter_dups = 0;
 /*@unchecked@*/
 int dbiTagsMax = 0;
 
+/* We use this to comunicate back to the the rpm transaction
+ *  what their install instance was on a rpmdbAdd().
+ */ 
+/*@unchecked@*/
+unsigned int myinstall_instance = 0;
+
 /* Bit mask macros. */
 /*@-exporttype@*/
 typedef	unsigned int __pbm_bits;
@@ -171,10 +176,11 @@ static int dbiTagToDbix(int rpmtag)
  * Initialize database (index, tag) tuple from configuration.
  */
 static void dbiTagsInit(void)
-	/*@globals rpmGlobalMacroContext, dbiTags, dbiTagsMax @*/
-	/*@modifies rpmGlobalMacroContext, dbiTags, dbiTagsMax @*/
+	/*@globals dbiTags, dbiTagsMax, rpmGlobalMacroContext, h_errno @*/
+	/*@modifies dbiTags, dbiTagsMax, rpmGlobalMacroContext @*/
 {
-/*@observer@*/ static const char * const _dbiTagStr_default =
+/*@observer@*/
+    static const char * const _dbiTagStr_default =
 	"Packages:Name:Basenames:Group:Requirename:Providename:Conflictname:Triggername:Dirnames:Requireversion:Provideversion:Installtid:Sigmd5:Sha1header:Filemd5s:Depends:Pubkeys";
     char * dbiTagStr = NULL;
     char * o, * oe;
@@ -227,17 +233,32 @@ static void dbiTagsInit(void)
 #define	DB1vec		NULL
 #define	DB2vec		NULL
 
+#ifdef HAVE_DB3_DB_H
 /*@-exportheadervar -declundef @*/
-/*@unchecked@*/
+/*@observer@*/ /*@unchecked@*/
 extern struct _dbiVec db3vec;
 /*@=exportheadervar =declundef @*/
 #define	DB3vec		&db3vec
 /*@=redecl@*/
+#else
+#define DB3vec		NULL
+#endif
+
+#ifdef HAVE_SQLITE3_H
+/*@-exportheadervar -declundef @*/
+/*@observer@*/ /*@unchecked@*/
+extern struct _dbiVec sqlitevec;
+/*@=exportheadervar =declundef @*/
+#define	SQLITEvec	&sqlitevec
+/*@=redecl@*/
+#else
+#define SQLITEvec	NULL
+#endif
 
 /*@-nullassign@*/
 /*@observer@*/ /*@unchecked@*/
 static struct _dbiVec *mydbvecs[] = {
-    DB1vec, DB1vec, DB2vec, DB3vec, NULL
+    DB1vec, DB1vec, DB2vec, DB3vec, SQLITEvec, NULL
 };
 /*@=nullassign@*/
 
@@ -262,14 +283,16 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, /*@unused@*/ unsigned int flags)
 /*@=compdef@*/
 
     _dbapi_rebuild = rpmExpandNumeric("%{_dbapi_rebuild}");
-    if (_dbapi_rebuild < 1 || _dbapi_rebuild > 3)
-	_dbapi_rebuild = 3;
-    _dbapi_wanted = (_rebuildinprogress ? -1 : db->db_api);
+    if (_dbapi_rebuild < 1 || _dbapi_rebuild > 4)
+	_dbapi_rebuild = 4;
+/*    _dbapi_wanted = (_rebuildinprogress ? -1 : db->db_api); */
+    _dbapi_wanted = (_rebuildinprogress ? _dbapi_rebuild : db->db_api);
 
     switch (_dbapi_wanted) {
     default:
 	_dbapi = _dbapi_wanted;
-	if (_dbapi < 0 || _dbapi >= 4 || mydbvecs[_dbapi] == NULL) {
+	if (_dbapi < 0 || _dbapi >= 5 || mydbvecs[_dbapi] == NULL) {
+            rpmMessage(RPMMESS_DEBUG, "dbiOpen: _dbiapi failed\n");
 	    return NULL;
 	}
 	errno = 0;
@@ -286,7 +309,7 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, /*@unused@*/ unsigned int flags)
 	}
 	break;
     case -1:
-	_dbapi = 4;
+	_dbapi = 5;
 	while (_dbapi-- > 1) {
 	    if (mydbvecs[_dbapi] == NULL)
 		continue;
@@ -309,6 +332,9 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, /*@unused@*/ unsigned int flags)
     	break;
     }
 
+/* We don't ever _REQUIRE_ conversion... */
+#define	SQLITE_HACK
+#ifdef	SQLITE_HACK_XXX
     /* Require conversion. */
     if (rc && _dbapi_wanted >= 0 && _dbapi != _dbapi_wanted && _dbapi_wanted == _dbapi_rebuild) {
 	rc = (_rebuildinprogress ? 0 : 1);
@@ -326,6 +352,7 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, /*@unused@*/ unsigned int flags)
 	rc = (_rebuildinprogress ? 0 : 1);
 	goto exit;
     }
+#endif
 
 exit:
     if (dbi != NULL && rc == 0) {
@@ -341,8 +368,11 @@ exit:
 	    db->db_bits = PBM_ALLOC(db->db_nbits);
 	}
 /*@=sizeoftype@*/
-    } else
+    }
+#ifdef HAVE_DB3_DB_H
+      else
 	dbi = db3Free(dbi);
+#endif
 
 /*@-compdef -nullstate@*/ /* FIX: db->_dbi may be NULL */
     return dbi;
@@ -370,9 +400,11 @@ union _dbswap {
 };
 
 #define	_DBSWAP(_a) \
+/*@-bounds@*/ \
   { unsigned char _b, *_c = (_a).uc; \
     _b = _c[3]; _c[3] = _c[0]; _c[0] = _b; \
     _b = _c[2]; _c[2] = _c[1]; _c[1] = _b; \
+/*@=bounds@*/ \
   }
 
 /**
@@ -619,9 +651,11 @@ dbiIndexSet dbiFreeIndexSet(dbiIndexSet set) {
 typedef struct miRE_s {
     rpmTag		tag;		/*!< header tag */
     rpmMireMode		mode;		/*!< pattern match mode */
-/*@only@*/ const char *	pattern;	/*!< pattern string */
+/*@only@*/
+    const char *	pattern;	/*!< pattern string */
     int			notmatch;	/*!< like "grep -v" */
-/*@only@*/ regex_t *	preg;		/*!< regex compiled pattern buffer */
+/*@only@*/
+    regex_t *		preg;		/*!< regex compiled pattern buffer */
     int			cflags;		/*!< regcomp(3) flags */
     int			eflags;		/*!< regexec(3) flags */
     int			fnflags;	/*!< fnmatch(3) flags */
@@ -646,9 +680,9 @@ struct _rpmdbMatchIterator {
     int			mi_sorted;
     int			mi_cflags;
     int			mi_modified;
-    unsigned int	mi_prevoffset;
-    unsigned int	mi_offset;
-    unsigned int	mi_filenum;
+    unsigned int	mi_prevoffset;	/* header instance (native endian) */
+    unsigned int	mi_offset;	/* header instance (native endian) */
+    unsigned int	mi_filenum;	/* tag element (native endian) */
     int			mi_nre;
 /*@only@*/ /*@null@*/
     miRE		mi_re;
@@ -660,111 +694,36 @@ struct _rpmdbMatchIterator {
 
 };
 
-/**
- */
-/*@unchecked@*/
-static sigset_t caught;
-
-/* forward ref */
-static void handler(int signum);
-
-/**
- */
-/*@unchecked@*/
-/*@-fullinitblock@*/
-static struct sigtbl_s {
-    int signum;
-    int active;
-    void (*handler) (int signum);
-    struct sigaction oact;
-} satbl[] = {
-    { SIGHUP,	0, handler },
-    { SIGINT,	0, handler },
-    { SIGTERM,	0, handler },
-    { SIGQUIT,	0, handler },
-    { SIGPIPE,	0, handler },
-    { -1,	0, NULL },
-};
-/*@=fullinitblock@*/
-
-/**
- */
-/*@-incondefs@*/
-static void handler(int signum)
-	/*@globals caught, satbl @*/
-	/*@modifies caught @*/
-{
-    struct sigtbl_s * tbl;
-
-    for(tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (tbl->signum != signum)
-	    continue;
-	if (!tbl->active)
-	    continue;
-	(void) sigaddset(&caught, signum);
-	break;
-    }
-}
-/*@=incondefs@*/
-
-/**
- * Enable all signal handlers.
- */
-static int enableSignals(void)
-	/*@globals caught, satbl, fileSystem @*/
-	/*@modifies caught, satbl, fileSystem @*/
-{
-    sigset_t newMask, oldMask;
-    struct sigtbl_s * tbl;
-    struct sigaction act;
-    int rc;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-    rc = 0;
-    for(tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (tbl->active++ > 0)
-	    continue;
-	(void) sigdelset(&caught, tbl->signum);
-	memset(&act, 0, sizeof(act));
-	act.sa_handler = tbl->handler;
-	rc = sigaction(tbl->signum, &act, &tbl->oact);
-	if (rc) break;
-    }
-    return sigprocmask(SIG_SETMASK, &oldMask, NULL);
-}
-
 /*@unchecked@*/
 static rpmdb rpmdbRock;
 
 /*@unchecked@*/ /*@exposed@*/ /*@null@*/
 static rpmdbMatchIterator rpmmiRock;
 
-int rpmdbCheckSignals(void)
-	/*@globals rpmdbRock, rpmmiRock, satbl @*/
+int rpmdbCheckTerminate(int terminate)
+	/*@globals rpmdbRock, rpmmiRock @*/
 	/*@modifies rpmdbRock, rpmmiRock @*/
 {
-    struct sigtbl_s * tbl;
     sigset_t newMask, oldMask;
-    static int terminate = 0;
+    static int terminating = 0;
 
-    if (terminate) return 0;
+    if (terminating) return 1;
 
     (void) sigfillset(&newMask);		/* block all signals */
     (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-    for(tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (tbl->active == 0)
-	    continue;
-	if (sigismember(&caught, tbl->signum))
-	    terminate = 1;
-    }
 
-    if (terminate) {
+    if (sigismember(&rpmsqCaught, SIGINT)
+     || sigismember(&rpmsqCaught, SIGQUIT)
+     || sigismember(&rpmsqCaught, SIGHUP)
+     || sigismember(&rpmsqCaught, SIGTERM)
+     || sigismember(&rpmsqCaught, SIGPIPE)
+     || terminate)
+	terminating = 1;
+
+    if (terminating) {
 	rpmdb db;
 	rpmdbMatchIterator mi;
 
-	rpmMessage(RPMMESS_DEBUG, "Exiting on signal ...\n");
-
 /*@-branchstate@*/
 	while ((mi = rpmmiRock) != NULL) {
 /*@i@*/	    rpmmiRock = mi->mi_next;
@@ -780,51 +739,38 @@ int rpmdbCheckSignals(void)
 	    (void) rpmdbClose(db);
 	}
 /*@=newreftrans@*/
-	exit(EXIT_FAILURE);
     }
-    return sigprocmask(SIG_SETMASK, &oldMask, NULL);
+    sigprocmask(SIG_SETMASK, &oldMask, NULL);
+    return terminating;
 }
 
-/**
- * Disable all signal handlers.
- */
-static int disableSignals(void)
-	/*@globals satbl, fileSystem @*/
-	/*@modifies satbl, fileSystem @*/
+int rpmdbCheckSignals(void)
 {
-    struct sigtbl_s * tbl;
-    sigset_t newMask, oldMask;
-    int rc;
-
-    (void) sigfillset(&newMask);		/* block all signals */
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-    rc = 0;
-    for(tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (--tbl->active > 0)
-	    continue;
-	rc = sigaction(tbl->signum, &tbl->oact, NULL);
-	if (rc) break;
+    if (rpmdbCheckTerminate(0)) {
+/*@-abstract@*/ /* sigset_t is abstract type */
+	rpmMessage(RPMMESS_DEBUG, "Exiting on signal(0x%lx) ...\n", *((unsigned long *)&rpmsqCaught));
+	exit(EXIT_FAILURE);
+/*@=abstract@*/
     }
-    return sigprocmask(SIG_SETMASK, &oldMask, NULL);
+    return 0;
 }
 
 /**
  * Block all signals, returning previous signal mask.
  */
 static int blockSignals(/*@unused@*/ rpmdb db, /*@out@*/ sigset_t * oldMask)
-	/*@globals satbl, fileSystem @*/
-	/*@modifies *oldMask, satbl, fileSystem @*/
+	/*@globals fileSystem @*/
+	/*@modifies *oldMask, fileSystem @*/
 {
-    struct sigtbl_s * tbl;
     sigset_t newMask;
 
     (void) sigfillset(&newMask);		/* block all signals */
     (void) sigprocmask(SIG_BLOCK, &newMask, oldMask);
-    for(tbl = satbl; tbl->signum >= 0; tbl++) {
-	if (tbl->active == 0)
-	    continue;
-	(void) sigdelset(&newMask, tbl->signum);
-    }
+    (void) sigdelset(&newMask, SIGINT);
+    (void) sigdelset(&newMask, SIGQUIT);
+    (void) sigdelset(&newMask, SIGHUP);
+    (void) sigdelset(&newMask, SIGTERM);
+    (void) sigdelset(&newMask, SIGPIPE);
     return sigprocmask(SIG_BLOCK, &newMask, NULL);
 }
 
@@ -857,6 +803,22 @@ static struct rpmdb_s dbTemplate = {
 };
 /*@=fullinitblock@*/
 
+static int isTemporaryDB(int rpmtag) 
+{
+    int rc = 0;
+    switch (rpmtag) {
+    case RPMDBI_AVAILABLE:
+    case RPMDBI_ADDED:
+    case RPMDBI_REMOVED:
+    case RPMDBI_DEPENDS:
+	rc = 1;
+	break;
+    default:
+	break;
+    }
+    return rc;
+}
+
 int rpmdbOpenAll(rpmdb db)
 {
     int dbix;
@@ -868,6 +830,9 @@ int rpmdbOpenAll(rpmdb db)
     for (dbix = 0; dbix < dbiTagsMax; dbix++) {
 	if (db->_dbi[dbix] != NULL)
 	    continue;
+	/* Filter out temporary databases */
+	if (isTemporaryDB(dbiTags[dbix])) 
+	    continue;
 	(void) dbiOpen(db, dbiTags[dbix], db->db_flags);
     }
     return rc;
@@ -949,7 +914,11 @@ int rpmdbClose(rpmdb db)
     /*@=usereleased@*/
 
 exit:
-    (void) disableSignals();
+    (void) rpmsqEnable(-SIGHUP,	NULL);
+    (void) rpmsqEnable(-SIGINT,	NULL);
+    (void) rpmsqEnable(-SIGTERM,NULL);
+    (void) rpmsqEnable(-SIGQUIT,NULL);
+    (void) rpmsqEnable(-SIGPIPE,NULL);
     return rc;
 }
 /*@=incondefs@*/
@@ -964,6 +933,8 @@ int rpmdbSync(rpmdb db)
 	int xx;
 	if (db->_dbi[dbix] == NULL)
 	    continue;
+	if (db->_dbi[dbix]->dbi_no_dbsync)
+	    continue;
     	xx = dbiSync(db->_dbi[dbix], 0);
 	if (xx && rc == 0) rc = xx;
     }
@@ -975,7 +946,7 @@ static /*@only@*/ /*@null@*/
 rpmdb newRpmdb(/*@kept@*/ /*@null@*/ const char * root,
 		/*@kept@*/ /*@null@*/ const char * home,
 		int mode, int perms, int flags)
-	/*@globals _db_filter_dups, rpmGlobalMacroContext @*/
+	/*@globals _db_filter_dups, rpmGlobalMacroContext, h_errno @*/
 	/*@modifies _db_filter_dups, rpmGlobalMacroContext @*/
 {
     rpmdb db = xcalloc(sizeof(*db), 1);
@@ -1001,10 +972,29 @@ rpmdb newRpmdb(/*@kept@*/ /*@null@*/ const char * root,
     if (perms >= 0)	db->db_perms = perms;
     if (flags >= 0)	db->db_flags = flags;
 
-    /*@-nullpass@*/
-    db->db_root = rpmGetPath( (root && *root ? root : _DB_ROOT), NULL);
+/*@-nullpass@*/
+    /* HACK: no URL's for root prefixed dbpath yet. */
+    if (root && *root) {
+	const char * rootpath = NULL;
+	urltype ut = urlPath(root, &rootpath);
+	switch (ut) {
+	case URL_IS_PATH:
+	case URL_IS_UNKNOWN:
+	    db->db_root = rpmGetPath(root, NULL);
+	    break;
+	case URL_IS_HTTPS:
+	case URL_IS_HTTP:
+	case URL_IS_FTP:
+	case URL_IS_HKP:
+	case URL_IS_DASH:
+	default:
+	    db->db_root = rpmGetPath(_DB_ROOT, NULL);
+	    break;
+	}
+    } else
+	db->db_root = rpmGetPath(_DB_ROOT, NULL);
     db->db_home = rpmGetPath( (home && *home ? home : _DB_HOME), NULL);
-    /*@=nullpass@*/
+/*@=nullpass@*/
     if (!(db->db_home && db->db_home[0] != '%')) {
 	rpmError(RPMERR_DBOPEN, _("no dbpath has been set\n"));
 	db->db_root = _free(db->db_root);
@@ -1028,7 +1018,7 @@ static int openDatabase(/*@null@*/ const char * prefix,
 		/*@null@*/ const char * dbpath,
 		int _dbapi, /*@null@*/ /*@out@*/ rpmdb *dbp,
 		int mode, int perms, int flags)
-	/*@globals rpmdbRock, rpmGlobalMacroContext,
+	/*@globals rpmdbRock, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies rpmdbRock, *dbp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
@@ -1046,7 +1036,7 @@ static int openDatabase(/*@null@*/ const char * prefix,
     }
 
     /* Insure that _dbapi has one of -1, 1, 2, or 3 */
-    if (_dbapi < -1 || _dbapi > 3)
+    if (_dbapi < -1 || _dbapi > 4)
 	_dbapi = -1;
     if (_dbapi == 0)
 	_dbapi = 1;
@@ -1060,7 +1050,11 @@ static int openDatabase(/*@null@*/ const char * prefix,
     if (db == NULL)
 	return 1;
 
-    (void) enableSignals();
+    (void) rpmsqEnable(SIGHUP,	NULL);
+    (void) rpmsqEnable(SIGINT,	NULL);
+    (void) rpmsqEnable(SIGTERM,NULL);
+    (void) rpmsqEnable(SIGQUIT,NULL);
+    (void) rpmsqEnable(SIGPIPE,NULL);
 
     db->db_api = _dbapi;
 
@@ -1073,16 +1067,8 @@ static int openDatabase(/*@null@*/ const char * prefix,
 	    int rpmtag;
 
 	    /* Filter out temporary databases */
-	    switch ((rpmtag = dbiTags[dbix])) {
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB((rpmtag = dbiTags[dbix])))
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
-	    default:
-		/*@switchbreak@*/ break;
-	    }
 
 	    dbi = dbiOpen(db, rpmtag, 0);
 	    if (dbi == NULL) {
@@ -1219,7 +1205,7 @@ int rpmdbVerify(const char * prefix)
  */
 static int rpmdbFindByFile(rpmdb db, /*@null@*/ const char * filespec,
 		DBT * key, DBT * data, /*@out@*/ dbiIndexSet * matches)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies db, *key, *data, *matches, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
 	/*@requires maxSet(matches) >= 0 @*/
@@ -1391,8 +1377,10 @@ key->size = strlen(name);
 
     xx = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, 0);
     rc = dbiGet(dbi, dbcursor, key, data, DB_SET);
+#ifndef	SQLITE_HACK
     xx = dbiCclose(dbi, dbcursor, 0);
     dbcursor = NULL;
+#endif
 
     if (rc == 0) {		/* success */
 	dbiIndexSet matches;
@@ -1414,6 +1402,11 @@ key->size = strlen(name);
 	rc = -1;
     }
 
+#ifdef	SQLITE_HACK
+    xx = dbiCclose(dbi, dbcursor, 0);
+    dbcursor = NULL;
+#endif
+
     return rc;
 }
 
@@ -1435,7 +1428,7 @@ static rpmRC dbiFindMatches(dbiIndex dbi, DBC * dbcursor,
 		/*@null@*/ const char * version,
 		/*@null@*/ const char * release,
 		/*@out@*/ dbiIndexSet * matches)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, *data, *matches,
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 	/*@requires maxSet(matches) >= 0 @*/
@@ -1531,7 +1524,7 @@ exit:
  */
 static rpmRC dbiFindByLabel(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 		/*@null@*/ const char * arg, /*@out@*/ dbiIndexSet * matches)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, *data, *matches,
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 	/*@requires maxSet(matches) >= 0 @*/
@@ -1627,7 +1620,7 @@ static rpmRC dbiFindByLabel(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
  */
 static int miFreeHeader(rpmdbMatchIterator mi, dbiIndex dbi)
 	/*@globals fileSystem, internalState @*/
-	/*@modifies mi, fileSystem, internalState @*/
+	/*@modifies mi, dbi, fileSystem, internalState @*/
 {
     int rc = 0;
 
@@ -1835,12 +1828,13 @@ static /*@only@*/ char * mireDup(rpmTag tag, rpmMireMode *modep,
 	nb = strlen(pattern) + sizeof("^$");
 
 	/* Find no. of bytes needed for pattern. */
-	/* periods are escaped, splats become '.*' */
+	/* periods and plusses are escaped, splats become '.*' */
 	c = '\0';
 	brackets = 0;
 	for (s = pattern; *s != '\0'; s++) {
 	    switch (*s) {
 	    case '.':
+	    case '+':
 	    case '*':
 		if (!brackets) nb++;
 		/*@switchbreak@*/ break;
@@ -1867,6 +1861,7 @@ static /*@only@*/ char * mireDup(rpmTag tag, rpmMireMode *modep,
 	for (s = pattern; *s != '\0'; s++, t++) {
 	    switch (*s) {
 	    case '.':
+	    case '+':
 		if (!brackets) *t++ = '\\';
 		/*@switchbreak@*/ break;
 	    case '*':
@@ -2049,8 +2044,10 @@ static int mireSkip (const rpmdbMatchIterator mi)
 	int anymatch;
 
 	if (!hge(mi->mi_h, mire->tag, &t, (void **)&u, &c)) {
-	    if (mire->tag != RPMTAG_EPOCH)
+	    if (mire->tag != RPMTAG_EPOCH) {
+		ntags++;
 		continue;
+	    }
 	    t = RPM_INT32_TYPE;
 /*@-immediatetrans@*/
 	    u.i32p = &zero;
@@ -2095,8 +2092,16 @@ static int mireSkip (const rpmdbMatchIterator mi)
 		    }
 		}
 		/*@switchbreak@*/ break;
-	    case RPM_NULL_TYPE:
 	    case RPM_BIN_TYPE:
+		{
+		const char * str = bin2hex((const char*) u.ptr, c);
+		rc = miregexec(mire, str);
+		if ((!rc && !mire->notmatch) || (rc && mire->notmatch))
+		    anymatch++;
+		_free(str);
+		}
+		/*@switchbreak@*/ break;
+	    case RPM_NULL_TYPE:
 	    default:
 		/*@switchbreak@*/ break;
 	    }
@@ -2109,7 +2114,8 @@ static int mireSkip (const rpmdbMatchIterator mi)
 	}
 /*@=boundsread@*/
 
-	u.ptr = hfd(u.ptr, t);
+	if (t != RPM_BIN_TYPE)
+	    u.ptr = hfd(u.ptr, t);
 
 	ntags++;
 	if (anymatch)
@@ -2148,7 +2154,7 @@ int rpmdbSetHdrChk(rpmdbMatchIterator mi, rpmts ts,
     int rc = 0;
     if (mi == NULL)
 	return 0;
-/*@-assignexpose -newreftrans @*/
+/*@-assignexpose -newreftrans @*/ /* XXX forward linkage prevents rpmtsLink */
 /*@i@*/ mi->mi_ts = ts;
     mi->mi_hdrchk = hdrchk;
 /*@=assignexpose =newreftrans @*/
@@ -2197,14 +2203,19 @@ top:
     uhlen = 0;
 
     do {
+union _dbswap mi_offset;
+
   	/*@-branchstate -compmempass @*/
 	if (mi->mi_set) {
 	    if (!(mi->mi_setx < mi->mi_set->count))
 		return NULL;
 	    mi->mi_offset = dbiIndexRecordOffset(mi->mi_set, mi->mi_setx);
 	    mi->mi_filenum = dbiIndexRecordFileNumber(mi->mi_set, mi->mi_setx);
-	    keyp = &mi->mi_offset;
-	    keylen = sizeof(mi->mi_offset);
+mi_offset.ui = mi->mi_offset;
+if (dbiByteSwapped(dbi) == 1)
+    _DBSWAP(mi_offset);
+	    keyp = &mi_offset;
+	    keylen = sizeof(mi_offset.ui);
 	} else {
 
 	    key->data = keyp = (void *)mi->mi_keyp;
@@ -2230,8 +2241,12 @@ top:
 	     * skipped.
 	     */
 /*@-boundswrite@*/
-	    if (keyp && mi->mi_setx && rc == 0)
-		memcpy(&mi->mi_offset, keyp, sizeof(mi->mi_offset));
+	    if (keyp && mi->mi_setx && rc == 0) {
+		memcpy(&mi_offset, keyp, sizeof(mi_offset.ui));
+if (dbiByteSwapped(dbi) == 1)
+    _DBSWAP(mi_offset);
+		mi->mi_offset = mi_offset.ui;
+	    }
 /*@=boundswrite@*/
 
 	    /* Terminate on error or end of keys */
@@ -2278,10 +2293,11 @@ top:
 /*@-boundsread -branchstate -sizeoftype @*/
     if (mi->mi_hdrchk && mi->mi_ts) {
 	rpmRC rpmrc = RPMRC_NOTFOUND;
-	pbm_set * set = NULL;
 
 	/* Don't bother re-checking a previously read header. */
 	if (mi->mi_db->db_bits) {
+	    pbm_set * set;
+
 	    set = PBM_REALLOC((pbm_set **)&mi->mi_db->db_bits,
 			&mi->mi_db->db_nbits, mi->mi_offset);
 	    if (PBM_ISSET(mi->mi_offset, set))
@@ -2301,8 +2317,13 @@ top:
 	    msg = _free(msg);
 
 	    /* Mark header checked. */
-	    if (set && rpmrc == RPMRC_OK)
+	    if (mi->mi_db && mi->mi_db->db_bits && rpmrc == RPMRC_OK) {
+		pbm_set * set;
+
+		set = PBM_REALLOC((pbm_set **)&mi->mi_db->db_bits,
+			&mi->mi_db->db_nbits, mi->mi_offset);
 		PBM_SET(mi->mi_offset, set);
+	    }
 
 	    /* Skip damaged and inconsistent headers. */
 	    if (rpmrc == RPMRC_FAIL)
@@ -2370,8 +2391,8 @@ static void rpmdbSortIterator(/*@null@*/ rpmdbMatchIterator mi)
 
 /*@-bounds@*/ /* LCL: segfault */
 static int rpmdbGrowIterator(/*@null@*/ rpmdbMatchIterator mi, int fpNum)
-	/*@globals rpmGlobalMacroContext, fileSystem @*/
-	/*@modifies mi, rpmGlobalMacroContext, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies mi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     DBC * dbcursor;
     DBT * key;
@@ -2397,14 +2418,20 @@ static int rpmdbGrowIterator(/*@null@*/ rpmdbMatchIterator mi, int fpNum)
 
     xx = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, 0);
     rc = dbiGet(dbi, dbcursor, key, data, DB_SET);
+#ifndef	SQLITE_HACK
     xx = dbiCclose(dbi, dbcursor, 0);
     dbcursor = NULL;
+#endif
 
     if (rc) {			/* error/not found */
 	if (rc != DB_NOTFOUND)
 	    rpmError(RPMERR_DBGETINDEX,
 		_("error(%d) getting \"%s\" records from %s index\n"),
 		rc, key->data, tagName(dbi->dbi_rpmtag));
+#ifdef	SQLITE_HACK
+	xx = dbiCclose(dbi, dbcursor, 0);
+	dbcursor = NULL;
+#endif
 	return rc;
     }
 
@@ -2413,6 +2440,11 @@ static int rpmdbGrowIterator(/*@null@*/ rpmdbMatchIterator mi, int fpNum)
     for (i = 0; i < set->count; i++)
 	set->recs[i].fpNum = fpNum;
 
+#ifdef	SQLITE_HACK
+    xx = dbiCclose(dbi, dbcursor, 0);
+    dbcursor = NULL;
+#endif
+
 /*@-branchstate@*/
     if (mi->mi_set == NULL) {
 	mi->mi_set = set;
@@ -2483,6 +2515,7 @@ rpmdbMatchIterator rpmdbInitIterator(rpmdb db, rpmTag rpmtag,
     if (dbi == NULL)
 	return NULL;
 
+    /* Chain cursors for teardown on abnormal exit. */
     mi = xcalloc(1, sizeof(*mi));
     mi->mi_next = rpmmiRock;
     rpmmiRock = mi;
@@ -2490,6 +2523,10 @@ rpmdbMatchIterator rpmdbInitIterator(rpmdb db, rpmTag rpmtag,
     key = &mi->mi_key;
     data = &mi->mi_data;
 
+    /*
+     * Handle label and file name special cases.
+     * Otherwise, retrieve join keys for secondary lookup.
+     */
 /*@-branchstate@*/
     if (rpmtag != RPMDBI_PACKAGES && keyp) {
 	DBC * dbcursor = NULL;
@@ -2497,7 +2534,6 @@ rpmdbMatchIterator rpmdbInitIterator(rpmdb db, rpmTag rpmtag,
 	int xx;
 
 	if (isLabel) {
-	    /* XXX HACK to get rpmdbFindByLabel out of the API */
 	    xx = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, 0);
 	    rc = dbiFindByLabel(dbi, dbcursor, key, data, keyp, &set);
 	    xx = dbiCclose(dbi, dbcursor, 0);
@@ -2523,8 +2559,9 @@ if (key->data && key->size == 0) key->size++;	/* XXX "/" fixup. */
 			rc, (key->data ? key->data : "???"), tagName(dbi->dbi_rpmtag));
 	    }
 
-if (rc == 0)
-(void) dbt2set(dbi, data, &set);
+	    /* Join keys need to be native endian internally. */
+	    if (rc == 0)
+		(void) dbt2set(dbi, data, &set);
 
 	    xx = dbiCclose(dbi, dbcursor, 0);
 	    dbcursor = NULL;
@@ -2539,17 +2576,31 @@ if (rc == 0)
     }
 /*@=branchstate@*/
 
+    /* Copy the retrieval key, byte swapping header instance if necessary. */
     if (keyp) {
-	char * k;
+	switch (rpmtag) {
+	case RPMDBI_PACKAGES:
+	  { union _dbswap *k;
 
-	if (rpmtag != RPMDBI_PACKAGES && keylen == 0)
-	    keylen = strlen(keyp);
-	k = xmalloc(keylen + 1);
+assert(keylen == sizeof(k->ui));		/* xxx programmer error */
+	    k = xmalloc(sizeof(*k));
+	    memcpy(k, keyp, keylen);
+	    if (dbiByteSwapped(dbi) == 1)
+		_DBSWAP(*k);
+	    mi_keyp = k;
+	  } break;
+	default:
+	  { char * k;
+	    if (keylen == 0)
+		keylen = strlen(keyp);
+	    k = xmalloc(keylen + 1);
 /*@-boundsread@*/
-	memcpy(k, keyp, keylen);
+	    memcpy(k, keyp, keylen);
 /*@=boundsread@*/
-	k[keylen] = '\0';	/* XXX for strings */
-	mi_keyp = k;
+	    k[keylen] = '\0';	/* XXX assumes strings */
+	    mi_keyp = k;
+	  } break;
+	}
     }
 
     mi->mi_keyp = mi_keyp;
@@ -2585,6 +2636,7 @@ int rpmdbRemove(rpmdb db, /*@unused@*/ int rid, unsigned int hdrNum,
 DBC * dbcursor = NULL;
 DBT * key = alloca(sizeof(*key));
 DBT * data = alloca(sizeof(*data));
+union _dbswap mi_offset;
     HGE_t hge = (HGE_t)headerGetEntryMinMemory;
     HFD_t hfd = headerFreeData;
     Header h;
@@ -2648,23 +2700,23 @@ memset(data, 0, sizeof(*data));
 /*@=boundsread@*/
 
 	    /*@-branchstate@*/
-	    switch (rpmtag) {
 	    /* Filter out temporary databases */
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB(rpmtag)) 
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
+
+	    switch (rpmtag) {
 	    case RPMDBI_PACKAGES:
 		dbi = dbiOpen(db, rpmtag, 0);
 		if (dbi == NULL)	/* XXX shouldn't happen */
 		    continue;
 	      
 /*@-immediatetrans@*/
-		key->data = &hdrNum;
+mi_offset.ui = hdrNum;
+if (dbiByteSwapped(dbi) == 1)
+    _DBSWAP(mi_offset);
+		key->data = &mi_offset;
 /*@=immediatetrans@*/
-		key->size = sizeof(hdrNum);
+		key->size = sizeof(mi_offset.ui);
 
 		rc = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, DB_WRITECURSOR);
 		rc = dbiGet(dbi, dbcursor, key, data, DB_SET);
@@ -2899,11 +2951,20 @@ DBT * data = alloca(sizeof(*data));
     int count = 0;
     dbiIndex dbi;
     int dbix;
+    union _dbswap mi_offset;
     unsigned int hdrNum = 0;
     int ret = 0;
     int rc;
     int xx;
 
+    /* Reinitialize to zero, so in the event the add fails
+     * we won't have bogus information (i.e. the last succesful
+     * add).
+     */
+/*@-mods@*/
+    myinstall_instance = 0;
+/*@=mods@*/
+
     if (db == NULL)
 	return 0;
 
@@ -2965,14 +3026,21 @@ memset(data, 0, sizeof(*data));
 
 /*@-bounds@*/
 	hdrNum = 0;
-	if (ret == 0 && datap)
-	    memcpy(&hdrNum, datap, sizeof(hdrNum));
-	++hdrNum;
 	if (ret == 0 && datap) {
-	    memcpy(datap, &hdrNum, sizeof(hdrNum));
+	    memcpy(&mi_offset, datap, sizeof(mi_offset.ui));
+	    if (dbiByteSwapped(dbi) == 1)
+		_DBSWAP(mi_offset);
+	    hdrNum = mi_offset.ui;
+	}
+	++hdrNum;
+	mi_offset.ui = hdrNum;
+	if (dbiByteSwapped(dbi) == 1)
+	    _DBSWAP(mi_offset);
+	if (ret == 0 && datap) {
+	    memcpy(datap, &mi_offset, sizeof(mi_offset.ui));
 	} else {
-	    datap = &hdrNum;
-	    datalen = sizeof(hdrNum);
+	    datap = &mi_offset;
+	    datalen = sizeof(mi_offset.ui);
 	}
 /*@=bounds@*/
 
@@ -3004,8 +3072,14 @@ memset(data, 0, sizeof(*data));
     /* Now update the indexes */
 
     if (hdrNum)
-    {	dbiIndexItem rec = dbiIndexNewItem(hdrNum, 0);
+    {	
+	dbiIndexItem rec = dbiIndexNewItem(hdrNum, 0);
 
+	/* Save the header number for the current transaction. */
+/*@-mods@*/
+	myinstall_instance = hdrNum;
+/*@=mods@*/
+	
 	if (dbiTags != NULL)
 	for (dbix = 0; dbix < dbiTagsMax; dbix++) {
 	    const char *av[1];
@@ -3024,22 +3098,24 @@ memset(data, 0, sizeof(*data));
 	    rpmtag = dbiTags[dbix];
 /*@=boundsread@*/
 
-	    switch (rpmtag) {
 	    /* Filter out temporary databases */
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB(rpmtag)) 
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
+
+	    switch (rpmtag) {
 	    case RPMDBI_PACKAGES:
 		dbi = dbiOpen(db, rpmtag, 0);
 		if (dbi == NULL)	/* XXX shouldn't happen */
 		    continue;
 		xx = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, DB_WRITECURSOR);
 
-key->data = (void *) &hdrNum;
-key->size = sizeof(hdrNum);
+mi_offset.ui = hdrNum;
+if (dbiByteSwapped(dbi) == 1)
+    _DBSWAP(mi_offset);
+/*@-immediatetrans@*/
+key->data = (void *) &mi_offset;
+/*@=immediatetrans@*/
+key->size = sizeof(mi_offset.ui);
 data->data = headerUnload(h);
 data->size = headerSizeof(h, HEADER_MAGIC_NO);
 
@@ -3311,12 +3387,6 @@ static struct skipDir_s {
 /*@observer@*/ /*@null@*/
     const char * dn;
 } skipDirs[] = {
-    _skip("/usr/share/zoneinfo"),
-    _skip("/usr/share/locale"),
-    _skip("/usr/share/i18n"),
-    _skip("/usr/share/doc"),
-    _skip("/usr/lib/locale"),
-    _skip("/usr/src/debug"),
     { 0, NULL }
 };
 
@@ -3351,11 +3421,11 @@ DBT * data;
     Header h;
     int i, xx;
 
-    if (db == NULL) return 0;
+    if (db == NULL) return 1;
 
     mi = rpmdbInitIterator(db, RPMTAG_BASENAMES, NULL, 0);
     if (mi == NULL)	/* XXX should  never happen */
-	return 0;
+	return 1;
 
 key = &mi->mi_key;
 data = &mi->mi_data;
@@ -3468,7 +3538,7 @@ if (key->size == 0) key->size++;	/* XXX "/" fixup. */
  * @return		1 if file exists, 0 if not
  */
 static int rpmioFileExists(const char * urlfn)
-        /*@globals fileSystem, internalState @*/
+        /*@globals h_errno, fileSystem, internalState @*/
         /*@modifies fileSystem, internalState @*/
 {
     const char *fn;
@@ -3479,8 +3549,10 @@ static int rpmioFileExists(const char * urlfn)
     if (*fn == '\0') fn = "/";
     /*@=branchstate@*/
     switch (urltype) {
-    case URL_IS_FTP:	/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:	/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:	/* XXX WRONG WRONG WRONG */
+    case URL_IS_FTP:	/* XXX WRONG WRONG WRONG */
+    case URL_IS_HKP:	/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
 	if (Stat(fn, &buf)) {
@@ -3502,7 +3574,7 @@ static int rpmioFileExists(const char * urlfn)
 
 static int rpmdbRemoveDatabase(const char * prefix,
 		const char * dbpath, int _dbapi)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 { 
     int i;
@@ -3523,6 +3595,8 @@ static int rpmdbRemoveDatabase(const char * prefix,
     filename = alloca(strlen(prefix) + strlen(dbpath) + 40);
 
     switch (_dbapi) {
+    case 4:
+	/*@fallthrough@*/
     case 3:
 	if (dbiTags != NULL)
 	for (i = 0; i < dbiTagsMax; i++) {
@@ -3558,8 +3632,8 @@ static int rpmdbRemoveDatabase(const char * prefix,
 
 static int rpmdbMoveDatabase(const char * prefix,
 		const char * olddbpath, int _olddbapi,
-		const char * newdbpath, int _newdbapi)
-	/*@globals fileSystem, internalState @*/
+		const char * newdbpath, /*@unused@*/ int _newdbapi)
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     int i;
@@ -3594,6 +3668,8 @@ static int rpmdbMoveDatabase(const char * prefix,
     nfilename = alloca(strlen(prefix) + strlen(newdbpath) + 40);
 
     switch (_olddbapi) {
+    case 4:
+        /* Fall through */
     case 3:
 	if (dbiTags != NULL)
 	for (i = 0; i < dbiTagsMax; i++) {
@@ -3601,16 +3677,8 @@ static int rpmdbMoveDatabase(const char * prefix,
 	    int rpmtag;
 
 	    /* Filter out temporary databases */
-	    switch ((rpmtag = dbiTags[i])) {
-	    case RPMDBI_AVAILABLE:
-	    case RPMDBI_ADDED:
-	    case RPMDBI_REMOVED:
-	    case RPMDBI_DEPENDS:
+	    if (isTemporaryDB((rpmtag = dbiTags[i])))
 		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
-	    default:
-		/*@switchbreak@*/ break;
-	    }
 
 	    base = tagName(rpmtag);
 	    sprintf(ofilename, "%s/%s/%s", prefix, olddbpath, base);
@@ -3643,12 +3711,12 @@ static int rpmdbMoveDatabase(const char * prefix,
 	for (i = 0; i < 16; i++) {
 	    sprintf(ofilename, "%s/%s/__db.%03d", prefix, olddbpath, i);
 	    (void)rpmCleanPath(ofilename);
-	    if (!rpmioFileExists(ofilename))
-		continue;
-	    xx = unlink(ofilename);
+	    if (rpmioFileExists(ofilename))
+		xx = unlink(ofilename);
 	    sprintf(nfilename, "%s/%s/__db.%03d", prefix, newdbpath, i);
 	    (void)rpmCleanPath(nfilename);
-	    xx = unlink(nfilename);
+	    if (rpmioFileExists(nfilename))
+		xx = unlink(nfilename);
 	}
 	break;
     case 2:
@@ -3656,6 +3724,7 @@ static int rpmdbMoveDatabase(const char * prefix,
     case 0:
 	break;
     }
+#ifdef	SQLITE_HACK_XXX
     if (rc || _olddbapi == _newdbapi)
 	return rc;
 
@@ -3670,6 +3739,7 @@ static int rpmdbMoveDatabase(const char * prefix,
 	    rpmMessage(RPMMESS_DEBUG,
 		_("removing %s after successful db3 rebuild.\n"), mdb1);
     }
+#endif
     return rc;
 }
 
@@ -3712,7 +3782,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     dbpath = rootdbpath = rpmGetPath(prefix, tfn, NULL);
     if (!(prefix[0] == '/' && prefix[1] == '\0'))
-	dbpath += strlen(prefix);
+	dbpath += strlen(prefix) - 1;
     tfn = _free(tfn);
 
     /*@-nullpass@*/
@@ -3735,7 +3805,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     newdbpath = newrootdbpath = rpmGetPath(prefix, tfn, NULL);
     if (!(prefix[0] == '/' && prefix[1] == '\0'))
-	newdbpath += strlen(prefix);
+	newdbpath += strlen(prefix) - 1;
     tfn = _free(tfn);
 
     rpmMessage(RPMMESS_DEBUG, _("rebuilding database %s into %s\n"),
@@ -3757,9 +3827,10 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     removedir = 1;
 
+    _rebuildinprogress = 0;
+
     rpmMessage(RPMMESS_DEBUG, _("opening old database with dbapi %d\n"),
 		_dbapi);
-    _rebuildinprogress = 1;
 /*@-boundswrite@*/
     if (openDatabase(prefix, dbpath, _dbapi, &olddb, O_RDONLY, 0644, 
 		     RPMDB_FLAG_MINIMAL)) {
@@ -3768,8 +3839,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
 /*@=boundswrite@*/
     _dbapi = olddb->db_api;
-    _rebuildinprogress = 0;
-
+    _rebuildinprogress = 1;
     rpmMessage(RPMMESS_DEBUG, _("opening new database with dbapi %d\n"),
 		_dbapi_rebuild);
     (void) rpmDefineMacro(NULL, "_rpmdb_rebuild %{nil}", -1);
@@ -3779,13 +3849,15 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	goto exit;
     }
 /*@=boundswrite@*/
+
+    _rebuildinprogress = 0;
+
     _dbapi_rebuild = newdb->db_api;
     
     {	Header h = NULL;
 	rpmdbMatchIterator mi;
 #define	_RECNUM	rpmdbGetIteratorOffset(mi)
 
-	/* RPMDBI_PACKAGES */
 	mi = rpmdbInitIterator(olddb, RPMDBI_PACKAGES, NULL, 0);
 	if (ts && hdrchk)
 	    (void) rpmdbSetHdrChk(mi, ts, hdrchk);
