@@ -59,6 +59,10 @@ static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
 #   define _STAT_VER		0
 #   define __fxstat64(_stat_ver, _fd, _sbp)	fstat((_fd), (_sbp))
 #endif
+#if defined(__APPLE__)
+#   define __errno_location()	(__error())
+#   define _STAT_VER		0
+#endif
 #include "system.h"
 #include "fts.h"
 #include "rpmio.h"
@@ -81,9 +85,10 @@ static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
 #endif
 
 
-/*@only@*/
+/*@only@*/ /*@null@*/
 static FTSENT *	fts_alloc(FTS * sp, const char * name, int namelen)
 	/*@*/;
+/*@null@*/
 static FTSENT *	fts_build(FTS * sp, int type)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies *sp, fileSystem, internalState @*/;
@@ -187,11 +192,13 @@ Fts_open(char * const * argv, int options,
 		/* Use fchdir(2) speedup only if local DASDI. */
 		switch (urlIsURL(*argv)) {
 		case URL_IS_DASH:
+		case URL_IS_HKP:
 /*@-boundswrite@*/
 			__set_errno (ENOENT);
 /*@=boundswrite@*/
 			goto mem3;
 			/*@notreached@*/ /*@switchbreak@*/ break;
+		case URL_IS_HTTPS:
 		case URL_IS_HTTP:
 		case URL_IS_FTP:
 			SET(FTS_NOCHDIR);
@@ -202,6 +209,8 @@ Fts_open(char * const * argv, int options,
 		}
 
 		p = fts_alloc(sp, *argv, len);
+		if (p == NULL)
+			goto mem3;
 		p->fts_level = FTS_ROOTLEVEL;
 		p->fts_parent = parent;
 		p->fts_accpath = p->fts_name;
@@ -223,7 +232,8 @@ Fts_open(char * const * argv, int options,
 			if (root == NULL)
 				tmp = root = p;
 			else {
-				tmp->fts_link = p;
+				if (tmp != NULL)	/* XXX can't happen */
+					tmp->fts_link = p;
 				tmp = p;
 			}
 		}
@@ -295,6 +305,9 @@ Fts_close(FTS * sp)
 	register FTSENT *freep, *p;
 	int saved_errno;
 
+	if (sp == NULL)
+		return 0;
+
 	/*
 	 * This still works if we haven't read anything -- the dummy structure
 	 * points to the root list, so we step through to the end of the root
@@ -357,7 +370,7 @@ Fts_read(FTS * sp)
 	int saved_errno;
 
 	/* If finished or unrecoverable error, return NULL. */
-	if (sp->fts_cur == NULL || ISSET(FTS_STOP))
+	if (sp == NULL || sp->fts_cur == NULL || ISSET(FTS_STOP))
 		return (NULL);
 
 	/* Set current node pointer. */
@@ -878,7 +891,8 @@ mem1:				saved_errno = errno;
 		if (len == sp->fts_pathlen || nitems == 0)
 			--cp;
 /*@-boundswrite@*/
-		*cp = '\0';
+		if (cp != NULL)	/* XXX can't happen */
+			*cp = '\0';
 /*@=boundswrite@*/
 	}
 
@@ -1022,7 +1036,7 @@ fts_sort(FTS * sp, FTSENT * head, int nitems)
 		sp->fts_array = a;
 	}
 /*@-boundswrite@*/
-	for (ap = sp->fts_array, p = head; p; p = p->fts_link)
+	for (ap = sp->fts_array, p = head; p != NULL; p = p->fts_link)
 		*ap++ = p;
 	qsort((void *)sp->fts_array, nitems, sizeof(*sp->fts_array),
 		sp->fts_compar);
@@ -1141,7 +1155,7 @@ fts_padjust(FTS * sp, FTSENT * head)
 	(p)->fts_path = addr;						\
 } while (0)
 	/* Adjust the current set of children. */
-	for (p = sp->fts_child; p; p = p->fts_link)
+	for (p = sp->fts_child; p != NULL; p = p->fts_link)
 		ADJUST(p);
 
 	/* Adjust the rest of the tree, including the current level. */
