@@ -55,6 +55,10 @@
 # define _D_EXACT_NAMLEN(d) ((d)->d_reclen)
 #endif
 
+#if defined(__APPLE__)
+# define _D_EXACT_NAMLEN(d) (strlen((d)->d_name))
+#endif
+
 #endif
 
 #include <sys/types.h>
@@ -62,14 +66,14 @@
 #include <dirent.h>
 
 typedef struct {
-/*@owned@*/
+/*@owned@*/ /*@relnull@*/
 	struct _ftsent *fts_cur;	/*!< current node */
 /*@owned@*/ /*@null@*/
 	struct _ftsent *fts_child;	/*!< linked list of children */
 /*@owned@*/ /*@null@*/
 	struct _ftsent **fts_array;	/*!< sort array */
 	dev_t fts_dev;			/*!< starting device # */
-/*@owned@*/
+/*@owned@*/ /*@relnull@*/
 	char *fts_path;			/*!< path for this descent */
 	int fts_rfd;			/*!< fd for root */
 	int fts_pathlen;		/*!< sizeof(path) */
@@ -94,29 +98,30 @@ typedef struct {
 		/*@globals fileSystem @*/
 		/*@modifies *st, fileSystem @*/;
 
-#define	FTS_COMFOLLOW	0x0001		/*!< follow command line symlinks */
-#define	FTS_LOGICAL	0x0002		/*!< logical walk */
-#define	FTS_NOCHDIR	0x0004		/*!< don't change directories */
-#define	FTS_NOSTAT	0x0008		/*!< don't get stat info */
-#define	FTS_PHYSICAL	0x0010		/*!< physical walk */
-#define	FTS_SEEDOT	0x0020		/*!< return dot and dot-dot */
-#define	FTS_XDEV	0x0040		/*!< don't cross devices */
-#define FTS_WHITEOUT	0x0080		/*!< return whiteout information */
-#define	FTS_OPTIONMASK	0x00ff		/*!< valid user option mask */
+#define	FTS_COMFOLLOW	0x0001		/* follow command line symlinks */
+#define	FTS_LOGICAL	0x0002		/* logical walk */
+#define	FTS_NOCHDIR	0x0004		/* don't change directories */
+#define	FTS_NOSTAT	0x0008		/* don't get stat info */
+#define	FTS_PHYSICAL	0x0010		/* physical walk */
+#define	FTS_SEEDOT	0x0020		/* return dot and dot-dot */
+#define	FTS_XDEV	0x0040		/* don't cross devices */
+#define FTS_WHITEOUT	0x0080		/* return whiteout information */
+#define	FTS_OPTIONMASK	0x00ff		/* valid user option mask */
 
-#define	FTS_NAMEONLY	0x0100		/*!< (private) child names only */
-#define	FTS_STOP	0x0200		/*!< (private) unrecoverable error */
+#define	FTS_NAMEONLY	0x0100		/* (private) child names only */
+#define	FTS_STOP	0x0200		/* (private) unrecoverable error */
 	int fts_options;		/*!< fts_open options, global flags */
 } FTS;
 
 typedef struct _ftsent {
 /*@dependent@*/
 	struct _ftsent *fts_cycle;	/*!< cycle node */
-/*@dependent@*/
+/*@dependent@*/ /*@relnull@*/
 	struct _ftsent *fts_parent;	/*!< parent directory */
-/*@dependent@*/
+/*@dependent@*/ /*@null@*/
 	struct _ftsent *fts_link;	/*!< next file in directory */
 	long fts_number;	        /*!< local numeric value */
+/*@null@*/
 	void *fts_pointer;	        /*!< local address value */
 /*@dependent@*/
 	char *fts_accpath;		/*!< access path */
@@ -135,30 +140,30 @@ typedef struct _ftsent {
 #define	FTS_ROOTLEVEL		 0
 	short fts_level;		/*!< depth (-1 to N) */
 
-#define	FTS_D		 1		/*!< preorder directory */
-#define	FTS_DC		 2		/*!< directory that causes cycles */
-#define	FTS_DEFAULT	 3		/*!< none of the above */
-#define	FTS_DNR		 4		/*!< unreadable directory */
-#define	FTS_DOT		 5		/*!< dot or dot-dot */
-#define	FTS_DP		 6		/*!< postorder directory */
-#define	FTS_ERR		 7		/*!< error; errno is set */
-#define	FTS_F		 8		/*!< regular file */
-#define	FTS_INIT	 9		/*!< initialized only */
-#define	FTS_NS		10		/*!< stat(2) failed */
-#define	FTS_NSOK	11		/*!< no stat(2) requested */
-#define	FTS_SL		12		/*!< symbolic link */
-#define	FTS_SLNONE	13		/*!< symbolic link without target */
-#define FTS_W		14		/*!< whiteout object */
+#define	FTS_D		 1		/* preorder directory */
+#define	FTS_DC		 2		/* directory that causes cycles */
+#define	FTS_DEFAULT	 3		/* none of the above */
+#define	FTS_DNR		 4		/* unreadable directory */
+#define	FTS_DOT		 5		/* dot or dot-dot */
+#define	FTS_DP		 6		/* postorder directory */
+#define	FTS_ERR		 7		/* error; errno is set */
+#define	FTS_F		 8		/* regular file */
+#define	FTS_INIT	 9		/* initialized only */
+#define	FTS_NS		10		/* stat(2) failed */
+#define	FTS_NSOK	11		/* no stat(2) requested */
+#define	FTS_SL		12		/* symbolic link */
+#define	FTS_SLNONE	13		/* symbolic link without target */
+#define FTS_W		14		/* whiteout object */
 	u_short fts_info;		/*!< user flags for FTSENT structure */
 
-#define	FTS_DONTCHDIR	 0x01		/*!< don't chdir .. to the parent */
-#define	FTS_SYMFOLLOW	 0x02		/*!< followed a symlink to get here */
+#define	FTS_DONTCHDIR	 0x01		/* don't chdir .. to the parent */
+#define	FTS_SYMFOLLOW	 0x02		/* followed a symlink to get here */
 	u_short fts_flags;		/*!< private flags for FTSENT structure */
 
-#define	FTS_AGAIN	 1		/*!< read node again */
-#define	FTS_FOLLOW	 2		/*!< follow symbolic link */
-#define	FTS_NOINSTR	 3		/*!< no instructions */
-#define	FTS_SKIP	 4		/*!< discard node */
+#define	FTS_AGAIN	 1		/* read node again */
+#define	FTS_FOLLOW	 2		/* follow symbolic link */
+#define	FTS_NOINSTR	 3		/* no instructions */
+#define	FTS_SKIP	 4		/* discard node */
 	u_short fts_instr;		/*!< fts_set() instructions */
 
 /*@dependent@*/
@@ -174,7 +179,7 @@ __BEGIN_DECLS
  * @param instr
  * @return		file set member
  */
-/*@unused@*/ /*@dependent@*/
+/*@dependent@*/ /*@null@*/
 FTSENT	*Fts_children (FTS * sp, int instr) __THROW
 	/*@globals fileSystem, internalState @*/
 	/*@modifies *sp, fileSystem, internalState @*/;
@@ -184,8 +189,7 @@ FTSENT	*Fts_children (FTS * sp, int instr) __THROW
  * @param sp		file hierarchy state
  * @return		0 on sucess, -1 on error
  */
-/*@unused@*/
-int	 Fts_close (/*@only@*/ FTS * sp) __THROW
+int	 Fts_close (/*@only@*/ /*@null@*/ FTS * sp) __THROW
 	/*@globals fileSystem, internalState @*/
 	/*@modifies *sp, fileSystem, internalState @*/;
 
@@ -196,7 +200,7 @@ int	 Fts_close (/*@only@*/ FTS * sp) __THROW
  * @param compar	traversal ordering (or NULL)
  * @return 		file hierarchy state (or NULL on error)
  */
-/*@unused@*/ /*@only@*/
+/*@only@*/ /*@null@*/
 FTS	*Fts_open (char * const * argv, int options,
 		   /*@null@*/
 		   int (*compar) (const FTSENT **, const FTSENT **)) __THROW
@@ -207,8 +211,8 @@ FTS	*Fts_open (char * const * argv, int options,
  * @param sp		file hierarchy state
  * @return		file set member
  */
-/*@unused@*/ /*@null@*/
-FTSENT	*Fts_read (FTS * sp) __THROW
+/*@null@*/
+FTSENT	*Fts_read (/*@null@*/ FTS * sp) __THROW
 	/*@globals fileSystem, internalState @*/
 	/*@modifies *sp, fileSystem, internalState @*/;
 
@@ -219,7 +223,6 @@ FTSENT	*Fts_read (FTS * sp) __THROW
  * @param instr		new disposition for file set member
  * @return		0 on sucess, -1 on error
  */
-/*@unused@*/
 int	 Fts_set (FTS * sp, FTSENT * p, int instr) __THROW
 	/*@modifies *p @*/;
 
