@@ -10,7 +10,7 @@
 #include <sys/stat.h>
 #include <dirent.h>
 /*@-noparams@*/
-#include <glob.h>
+#include "glob.h"
 /*@=noparams@*/
 #include <stdio.h>
 #include <stdlib.h>
@@ -282,7 +282,7 @@ int Fclose( /*@killref@*/ FD_t fd)
  */
 /*@null@*/ FD_t	Fopen(/*@null@*/ const char * path,
 			/*@null@*/ const char * fmode)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 
 
@@ -325,28 +325,28 @@ int Fcntl(FD_t fd, int op, void *lip)
  * mkdir(2) clone.
  */
 int Mkdir(const char * path, mode_t mode)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * chdir(2) clone.
  */
 int Chdir(const char * path)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * rmdir(2) clone.
  */
 int Rmdir(const char * path)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
  * rename(2) clone.
  */
 int Rename(const char * oldpath, const char * newpath)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
@@ -360,7 +360,7 @@ int Link(const char * oldpath, const char * newpath)
  * unlink(2) clone.
  */
 int Unlink(const char * path)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
@@ -368,7 +368,7 @@ int Unlink(const char * path)
  */
 /*@-incondefs@*/
 int Readlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies *buf, errno, fileSystem, internalState @*/
 	/*@requires maxSet(buf) >= (bufsiz - 1) @*/
 	/*@ensures maxRead(buf) <= bufsiz @*/;
@@ -378,14 +378,14 @@ int Readlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
  * stat(2) clone.
  */
 int Stat(const char * path, /*@out@*/ struct stat * st)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies *st, errno, fileSystem, internalState @*/;
 
 /**
  * lstat(2) clone.
  */
 int Lstat(const char * path, /*@out@*/ struct stat * st)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies *st, errno, fileSystem, internalState @*/;
 
 /**
@@ -395,6 +395,17 @@ int Access(const char * path, int amode)
 	/*@globals errno, fileSystem @*/
 	/*@modifies errno, fileSystem @*/;
 
+/**
+ * glob_pattern_p(3) clone.
+ */
+int Glob_pattern_p (const char *pattern, int quote)
+	/*@*/;
+
+/**
+ * glob_error(3) clone.
+ */
+int Glob_error(const char * epath, int eerrno)
+	/*@*/;
 
 /**
  * glob(3) clone.
@@ -418,7 +429,7 @@ void Globfree( /*@only@*/ glob_t * pglob)
  */
 /*@null@*/
 DIR * Opendir(const char * path)
-	/*@globals errno, fileSystem, internalState @*/
+	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
 /**
@@ -482,8 +493,8 @@ off_t	fdSize(FD_t fd)
  */
 /*@-incondefs@*/
 ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
-	/*@globals errno, fileSystem @*/
-	/*@modifies *cookie, *buf, errno, fileSystem @*/
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *cookie, *buf, errno, fileSystem, internalState @*/
 	/*@requires maxSet(buf) >= (count - 1) @*/
 	/*@ensures maxRead(buf) == result @*/ ;
 #define	fdRead(_fd, _buf, _count)	fdio->read((_fd), (_buf), (_count))
@@ -492,8 +503,8 @@ ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
 /**
  */
 ssize_t	fdWrite(void * cookie, const char * buf, size_t count)
-	/*@globals errno, fileSystem @*/
-	/*@modifies *cookie, errno, fileSystem @*/;
+	/*@globals errno, fileSystem, internalState @*/
+	/*@modifies *cookie, errno, fileSystem, internalState @*/;
 #define	fdWrite(_fd, _buf, _count)	fdio->write((_fd), (_buf), (_count))
 
 /**
@@ -552,25 +563,42 @@ int fdReadable(FD_t fd, int secs)
  * @return		0 on success, errno (or -1) on error
  */
 int rpmioMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 
+/**
+ * Escape isspace(3) characters in string.
+ * @param s             string
+ * @return              escaped string
+ */
+char * rpmEscapeSpaces(const char * s);
+
 /**
  * FTP and HTTP error codes.
  */
 /*@-typeuse@*/
 typedef enum ftperrCode_e {
-    FTPERR_BAD_SERVER_RESPONSE	= -1,	/*!< Bad server response */
-    FTPERR_SERVER_IO_ERROR	= -2,	/*!< Server I/O error */
-    FTPERR_SERVER_TIMEOUT	= -3,	/*!< Server timeout */
-    FTPERR_BAD_HOST_ADDR	= -4,	/*!< Unable to lookup server host address */
-    FTPERR_BAD_HOSTNAME		= -5,	/*!< Unable to lookup server host name */
-    FTPERR_FAILED_CONNECT	= -6,	/*!< Failed to connect to server */
-    FTPERR_FILE_IO_ERROR	= -7,	/*!< Failed to establish data connection to server */
-    FTPERR_PASSIVE_ERROR	= -8,	/*!< I/O error to local file */
-    FTPERR_FAILED_DATA_CONNECT	= -9,	/*!< Error setting remote server to passive mode */
-    FTPERR_FILE_NOT_FOUND	= -10,	/*!< File not found on server */
-    FTPERR_NIC_ABORT_IN_PROGRESS= -11,	/*!< Abort in progress */
+    FTPERR_NE_ERROR		= -1,	/*!< Generic error. */
+    FTPERR_NE_LOOKUP		= -2,	/*!< Hostname lookup failed. */
+    FTPERR_NE_AUTH		= -3,	/*!< Server authentication failed. */
+    FTPERR_NE_PROXYAUTH		= -4,	/*!< Proxy authentication failed. */
+    FTPERR_NE_CONNECT		= -5,	/*!< Could not connect to server. */
+    FTPERR_NE_TIMEOUT		= -6,	/*!< Connection timed out. */
+    FTPERR_NE_FAILED		= -7,	/*!< The precondition failed. */
+    FTPERR_NE_RETRY		= -8,	/*!< Retry request. */
+    FTPERR_NE_REDIRECT		= -9,	/*!< Redirect received. */
+
+    FTPERR_BAD_SERVER_RESPONSE	= -81,	/*!< Bad server response */
+    FTPERR_SERVER_IO_ERROR	= -82,	/*!< Server I/O error */
+    FTPERR_SERVER_TIMEOUT	= -83,	/*!< Server timeout */
+    FTPERR_BAD_HOST_ADDR	= -84,	/*!< Unable to lookup server host address */
+    FTPERR_BAD_HOSTNAME		= -85,	/*!< Unable to lookup server host name */
+    FTPERR_FAILED_CONNECT	= -86,	/*!< Failed to connect to server */
+    FTPERR_FILE_IO_ERROR	= -87,	/*!< Failed to establish data connection to server */
+    FTPERR_PASSIVE_ERROR	= -88,	/*!< I/O error to local file */
+    FTPERR_FAILED_DATA_CONNECT	= -89,	/*!< Error setting remote server to passive mode */
+    FTPERR_FILE_NOT_FOUND	= -90,	/*!< File not found on server */
+    FTPERR_NIC_ABORT_IN_PROGRESS= -91,	/*!< Abort in progress */
     FTPERR_UNKNOWN		= -100	/*!< Unknown or unexpected error */
 } ftperrCode;
 /*@=typeuse@*/
@@ -593,7 +621,7 @@ typedef enum ftperrCode_e {
 /*@-redecl@*/
 /*@unused@*/
 /*@observer@*/ const char * urlStrerror(const char * url)
-	/*@globals internalState @*/
+	/*@globals h_errno, internalState @*/
 	/*@modifies internalState @*/;
 /*@=redecl@*/
 
@@ -616,7 +644,7 @@ int ufdGetFile( /*@killref@*/ FD_t sfd, FD_t tfd)
 /*@unused@*/ int timedRead(FD_t fd, /*@out@*/ void * bufptr, int length)
 	/*@globals fileSystem @*/
 	/*@modifies fd, *bufptr, fileSystem @*/;
-#define	timedRead	ufdio->read
+#define	timedRead	(ufdio->read)
 
 /*@-exportlocal@*/
 /**
