@@ -9,16 +9,33 @@
 #endif
 
 #include <rpmio_internal.h>
+
+#define	_RPMDAV_INTERNAL
+#include <rpmdav.h>
+
 #include "ugid.h"
 #include "debug.h"
 
-/*@access DIR@*/
-/*@access FD_t@*/
-/*@access urlinfo@*/
+/*@access DIR @*/
+/*@access FD_t @*/
+/*@access urlinfo @*/
+
+/**
+ * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
+ * @param p		memory to free
+ * @retval		NULL always
+ */
+/*@unused@*/ static inline /*@null@*/ void *
+_free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
+	/*@modifies p@*/
+{
+    if (p != NULL)	free((void *)p);
+    return NULL;
+}
 
 /* =============================================================== */
 static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     int rc;
@@ -34,21 +51,21 @@ static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
 }
 
 static int ftpChdir(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     return ftpCmd("CWD", path, NULL);
 }
 
 static int ftpRmdir(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     return ftpCmd("RMD", path, NULL);
 }
 
 static int ftpRename(const char * oldpath, const char * newpath)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     int rc;
@@ -58,14 +75,13 @@ static int ftpRename(const char * oldpath, const char * newpath)
 }
 
 static int ftpUnlink(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
     return ftpCmd("DELE", path, NULL);
 }
 
 /* =============================================================== */
-/* XXX rebuilddb.c: analogues to mkdir(2)/rmdir(2). */
 int Mkdir (const char * path, mode_t mode)
 {
     const char * lpath;
@@ -75,13 +91,19 @@ int Mkdir (const char * path, mode_t mode)
     case URL_IS_FTP:
 	return ftpMkdir(path, mode);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davMkdir(path, mode);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -98,13 +120,21 @@ int Chdir (const char * path)
     case URL_IS_FTP:
 	return ftpChdir(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef	NOTYET
+	return davChdir(path);
+#else
+	return -2;
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -121,13 +151,19 @@ int Rmdir (const char * path)
     case URL_IS_FTP:
 	return ftpRmdir(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davRmdir(path);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -148,12 +184,18 @@ int Rename (const char * oldpath, const char * newpath)
 
     oldut = urlPath(oldpath, &oe);
     switch (oldut) {
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davRename(oldpath, newpath);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -169,6 +211,7 @@ fprintf(stderr, "*** rename old %*s new %*s\n", (int)(oe - oldpath), oldpath, (i
 	    return -2;
 	return ftpRename(oldpath, newpath);
 	/*@notreached@*/ break;
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
 	oldpath = oe;
@@ -177,6 +220,7 @@ fprintf(stderr, "*** rename old %*s new %*s\n", (int)(oe - oldpath), oldpath, (i
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -192,12 +236,14 @@ int Link (const char * oldpath, const char * newpath)
 
     oldut = urlPath(oldpath, &oe);
     switch (oldut) {
-    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -205,6 +251,7 @@ int Link (const char * oldpath, const char * newpath)
 
     newut = urlPath(newpath, &ne);
     switch (newut) {
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
@@ -219,6 +266,7 @@ fprintf(stderr, "*** link old %*s new %*s\n", (int)(oe - oldpath), oldpath, (int
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -236,13 +284,19 @@ int Unlink(const char * path) {
     case URL_IS_FTP:
 	return ftpUnlink(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davUnlink(path);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -692,7 +746,7 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 	    goto error;
 	
 #ifdef HAVE_ST_RDEV
-	st->st_rdev = ((maj & 0xff) << 8) | (min & 0xffff00ff);
+	st->st_rdev = ((maj & 0x000000ffU) << 8) | (min & 0x000000ffU);
 #endif
 	st->st_size = 0;
 	
@@ -821,12 +875,13 @@ static /*@only@*/ char * ftpBuf = NULL;
 #define alloca_strdup(_s)       strcpy(alloca(strlen(_s)+1), (_s))
 
 /*@-boundswrite@*/
-/*@-mods@*/
 static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 		/*@out@*/ /*@null@*/ struct stat * st,
 		/*@out@*/ /*@null@*/ char * rlbuf, size_t rlbufsiz)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, *rlbuf, fileSystem, internalState @*/
+	/*@globals ftpBufAlloced, ftpBuf,
+		h_errno, fileSystem, internalState @*/
+	/*@modifies *st, *rlbuf, ftpBufAlloced, ftpBuf,
+		fileSystem, internalState @*/
 {
     FD_t fd;
     const char * path;
@@ -1011,7 +1066,6 @@ exit:
     (void) ufdClose(fd);
     return rc;
 }
-/*@=mods@*/
 /*@=boundswrite@*/
 
 static const char * statstr(const struct stat * st,
@@ -1020,56 +1074,54 @@ static const char * statstr(const struct stat * st,
 {
     sprintf(buf,
 	"*** dev %x ino %x mode %0o nlink %d uid %d gid %d rdev %x size %x\n",
-	(unsigned)st->st_dev,
-	(unsigned)st->st_ino,
-	st->st_mode,
-	(unsigned)st->st_nlink,
-	st->st_uid,
-	st->st_gid,
-	(unsigned)st->st_rdev,
-	(unsigned)st->st_size);
+	(unsigned int)st->st_dev,
+	(unsigned int)st->st_ino,
+	(unsigned int)st->st_mode,
+	(unsigned int)st->st_nlink,
+	(unsigned int)st->st_uid,
+	(unsigned int)st->st_gid,
+	(unsigned int)st->st_rdev,
+	(unsigned int)st->st_size);
     return buf;
 }
 
 /*@unchecked@*/
 static int ftp_st_ino = 0xdead0000;
 
+/* FIXME: borked for path with trailing '/' */
 static int ftpStat(const char * path, /*@out@*/ struct stat *st)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, fileSystem, internalState @*/
+	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
+	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
 {
     char buf[1024];
     int rc;
     rc = ftpNLST(path, DO_FTP_STAT, st, NULL, 0);
     /* XXX fts(3) needs/uses st_ino, make something up for now. */
-    /*@-mods@*/
     if (st->st_ino == 0)
 	st->st_ino = ftp_st_ino++;
-    /*@=mods@*/
 if (_ftp_debug)
 fprintf(stderr, "*** ftpStat(%s) rc %d\n%s", path, rc, statstr(st, buf));
     return rc;
 }
 
+/* FIXME: borked for path with trailing '/' */
 static int ftpLstat(const char * path, /*@out@*/ struct stat *st)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *st, fileSystem, internalState @*/
+	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
+	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
 {
     char buf[1024];
     int rc;
     rc = ftpNLST(path, DO_FTP_LSTAT, st, NULL, 0);
     /* XXX fts(3) needs/uses st_ino, make something up for now. */
-    /*@-mods@*/
     if (st->st_ino == 0)
 	st->st_ino = ftp_st_ino++;
-    /*@=mods@*/
 if (_ftp_debug)
 fprintf(stderr, "*** ftpLstat(%s) rc %d\n%s\n", path, rc, statstr(st, buf));
     return rc;
 }
 
 static int ftpReadlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies *buf, fileSystem, internalState @*/
 {
     int rc;
@@ -1079,45 +1131,13 @@ fprintf(stderr, "*** ftpReadlink(%s) rc %d\n", path, rc);
     return rc;
 }
 
-struct __dirstream {
-    int fd;			/* File descriptor.  */
-    char * data;		/* Directory block.  */
-    size_t allocation;		/* Space allocated for the block.  */
-    size_t size;		/* Total valid data in the block.  */
-    size_t offset;		/* Current offset into the block.  */
-    off_t filepos;		/* Position of next entry to read.  */
-#if defined(HAVE_PTHREAD_H) && !defined(__LCLINT__)
-    pthread_mutex_t lock;	/* Mutex lock for this structure.  */
-#endif
-};
-
-#if !defined(DT_DIR)
-# define DT_UNKNOWN	0
-# define DT_FIFO	1
-# define DT_CHR		2
-# define DT_DIR		4
-# define DT_BLK		6
-# define DT_REG		8
-# define DT_LNK		10
-# define DT_SOCK	12
-# define DT_WHT		14
-typedef struct __dirstream *	FTPDIR;
-#else
-typedef DIR *			FTPDIR;
-#endif
-
-/*@unchecked@*/
-static int ftpmagicdir = 0x8440291;
-#define	ISFTPMAGIC(_dir) (!memcmp((_dir), &ftpmagicdir, sizeof(ftpmagicdir)))
-
 /*@-boundswrite@*/
-/*@-type@*/ /* FIX: abstract DIR */
 /*@null@*/
 static DIR * ftpOpendir(const char * path)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
-    FTPDIR mydir;
+    AVDIR avdir;
     struct dirent * dp;
     size_t nb;
     const char * s, * sb, * se;
@@ -1165,23 +1185,29 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 	}
     }
 
-    nb += sizeof(*mydir) + sizeof(*dp) + ((ac + 1) * sizeof(*av)) + (ac + 1);
-    mydir = xcalloc(1, nb);
+    nb += sizeof(*avdir) + sizeof(*dp) + ((ac + 1) * sizeof(*av)) + (ac + 1);
+    avdir = xcalloc(1, nb);
     /*@-abstract@*/
-    dp = (struct dirent *) (mydir + 1);
+    dp = (struct dirent *) (avdir + 1);
     av = (const char **) (dp + 1);
     dt = (char *) (av + (ac + 1));
     t = (char *) (dt + ac + 1);
     /*@=abstract@*/
 
-    mydir->fd = ftpmagicdir;
+    avdir->fd = avmagicdir;
 /*@-usereleased@*/
-    mydir->data = (char *) dp;
+    avdir->data = (char *) dp;
 /*@=usereleased@*/
-    mydir->allocation = nb;
-    mydir->size = ac;
-    mydir->offset = -1;
-    mydir->filepos = 0;
+    avdir->allocation = nb;
+    avdir->size = ac;
+    avdir->offset = -1;
+    avdir->filepos = 0;
+
+#if defined(HAVE_PTHREAD_H)
+/*@-moduncon -noeffectuncon@*/
+    (void) pthread_mutex_init(&avdir->lock, NULL);
+/*@=moduncon =noeffectuncon@*/
+#endif
 
     ac = 0;
     /*@-dependenttrans -unrecog@*/
@@ -1246,83 +1272,11 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
     av[ac] = NULL;
 
 /*@-kepttrans@*/
-    return (DIR *) mydir;
+    return (DIR *) avdir;
 /*@=kepttrans@*/
 }
 /*@=boundswrite@*/
 
-/*@dependent@*/ /*@null@*/
-static struct dirent * ftpReaddir(DIR * dir)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/
-{
-    FTPDIR mydir = (FTPDIR)dir;
-    struct dirent * dp;
-    const char ** av;
-    unsigned char * dt;
-    int ac;
-    int i;
-
-    /*@+voidabstract@*/
-    if (mydir == NULL || !ISFTPMAGIC(mydir) || mydir->data == NULL) {
-	/* XXX TODO: EBADF errno. */
-	return NULL;
-    }
-    /*@=voidabstract@*/
-
-    dp = (struct dirent *) mydir->data;
-    av = (const char **) (dp + 1);
-    ac = mydir->size;
-    dt = (char *) (av + (ac + 1));
-    i = mydir->offset + 1;
-
-/*@-boundsread@*/
-    if (i < 0 || i >= ac || av[i] == NULL)
-	return NULL;
-/*@=boundsread@*/
-
-    mydir->offset = i;
-
-    /* XXX glob(3) uses REAL_DIR_ENTRY(dp) test on d_ino */
-    dp->d_ino = i + 1;		/* W2DO? */
-    dp->d_reclen = 0;		/* W2DO? */
-
-#if !defined(hpux) && !defined(sun)
-    dp->d_off = 0;		/* W2DO? */
-/*@-boundsread@*/
-    dp->d_type = dt[i];
-/*@=boundsread@*/
-#endif
-
-    strncpy(dp->d_name, av[i], sizeof(dp->d_name));
-/*@+voidabstract@*/
-if (_ftp_debug)
-fprintf(stderr, "*** ftpReaddir(%p) %p \"%s\"\n", (void *)mydir, dp, dp->d_name);
-/*@=voidabstract@*/
-    
-    return dp;
-}
-/*@=type@*/
-
-static int ftpClosedir(/*@only@*/ DIR * dir)
-	/*@globals fileSystem @*/
-	/*@modifies dir, fileSystem @*/
-{
-    FTPDIR mydir = (FTPDIR)dir;
-
-    /*@+voidabstract@*/
-if (_ftp_debug)
-fprintf(stderr, "*** ftpClosedir(%p)\n", (void *)mydir);
-    if (mydir == NULL || !ISFTPMAGIC(mydir)) {
-	/* XXX TODO: EBADF errno. */
-	return -1;
-    }
-    free((void *)mydir);
-    /*@=voidabstract@*/
-    mydir = NULL;
-    return 0;
-}
-
 int Stat(const char * path, struct stat * st)
 {
     const char * lpath;
@@ -1334,13 +1288,19 @@ fprintf(stderr, "*** Stat(%s,%p)\n", path, st);
     case URL_IS_FTP:
 	return ftpStat(path, st);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davStat(path, st);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1359,13 +1319,19 @@ fprintf(stderr, "*** Lstat(%s,%p)\n", path, st);
     case URL_IS_FTP:
 	return ftpLstat(path, st);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davLstat(path, st);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1382,18 +1348,26 @@ int Readlink(const char * path, char * buf, size_t bufsiz)
     case URL_IS_FTP:
 	return ftpReadlink(path, buf, bufsiz);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef	NOTYET
+	return davReadlink(path, buf, bufsiz);
+#else
+	return -2;
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
     }
-/*@-compdef@*/
+/*@-compdef@*/ /* FIX: *buf is undefined */
     return readlink(path, buf, bufsiz);
 /*@=compdef@*/
 }
@@ -1406,14 +1380,16 @@ int Access(const char * path, int amode)
 if (_rpmio_debug)
 fprintf(stderr, "*** Access(%s,%d)\n", path, amode);
     switch (ut) {
-    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1421,6 +1397,52 @@ fprintf(stderr, "*** Access(%s,%d)\n", path, amode);
     return access(path, amode);
 }
 
+/* glob_pattern_p() taken from bash
+ * Copyright (C) 1985, 1988, 1989 Free Software Foundation, Inc.
+ *
+ * Return nonzero if PATTERN has any special globbing chars in it.
+ */
+int Glob_pattern_p (const char * pattern, int quote)
+{
+    const char *p;
+    int open = 0;
+    char c;
+  
+    (void) urlPath(pattern, &p);
+    while ((c = *p++) != '\0')
+	switch (c) {
+	case '?':
+	case '*':
+	    return (1);
+	case '\\':
+	    if (quote && *p != '\0')
+		p++;
+	    continue;
+
+	case '[':
+	    open = 1;
+	    continue;
+	case ']':
+	    if (open)
+		return (1);
+	    continue;
+
+	case '+':
+	case '@':
+	case '!':
+	    if (*p == '(')
+		return (1);
+	    continue;
+	}
+
+    return (0);
+}
+
+int Glob_error(/*@unused@*/const char * epath, /*@unused@*/ int eerrno)
+{
+    return 1;
+}
+
 int Glob(const char *pattern, int flags,
 	int errfunc(const char * epath, int eerrno), glob_t *pglob)
 {
@@ -1432,6 +1454,8 @@ if (_rpmio_debug)
 fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)errfunc, pglob);
 /*@=castfcnptr@*/
     switch (ut) {
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
     case URL_IS_FTP:
 /*@-type@*/
 	pglob->gl_closedir = Closedir;
@@ -1441,14 +1465,15 @@ fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)e
 	pglob->gl_stat = Stat;
 /*@=type@*/
 	flags |= GLOB_ALTDIRFUNC;
+	flags &= ~GLOB_TILDE;
 	break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
 	pattern = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1474,13 +1499,19 @@ fprintf(stderr, "*** Opendir(%s)\n", path);
     case URL_IS_FTP:
 	return ftpOpendir(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:	
+    case URL_IS_HTTP:
+#ifdef WITH_NEON
+	return davOpendir(path);
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return NULL;
 	/*@notreached@*/ break;
@@ -1490,13 +1521,18 @@ fprintf(stderr, "*** Opendir(%s)\n", path);
     /*@=dependenttrans@*/
 }
 
-/*@+voidabstract@*/
 struct dirent * Readdir(DIR * dir)
 {
 if (_rpmio_debug)
 fprintf(stderr, "*** Readdir(%p)\n", (void *)dir);
-    if (dir == NULL || ISFTPMAGIC(dir))
-	return ftpReaddir(dir);
+    if (dir == NULL)
+	return NULL;
+    if (ISAVMAGIC(dir))
+	return avReaddir(dir);
+#ifdef WITH_NEON
+    if (ISDAVMAGIC(dir))
+	return davReaddir(dir);
+#endif
     return readdir(dir);
 }
 
@@ -1504,8 +1540,13 @@ int Closedir(DIR * dir)
 {
 if (_rpmio_debug)
 fprintf(stderr, "*** Closedir(%p)\n", (void *)dir);
-    if (dir == NULL || ISFTPMAGIC(dir))
-	return ftpClosedir(dir);
+    if (dir == NULL)
+	return 0;
+    if (ISAVMAGIC(dir))
+	return avClosedir(dir);
+#ifdef WITH_NEON
+    if (ISDAVMAGIC(dir))
+	return davClosedir(dir);
+#endif
     return closedir(dir);
 }
-/*@=voidabstract@*/
